*****************************************************************************
*                                         Copyright (c) Motorola 2002		*
*  File Name:	HID-KBD.ASM													*
*                        													*
*  Description:	A keyboard demo for JB16 USB ICP							*
*                                                                          	*
*  Assembler:	CodeWarrior													*
*  Version:		2.1								                        	*
*																			*
*  Current Revision:	1.0													*
*  Current Revision Release Date: 2002.10.10 by Derek Lau					*
*																			*
*  Updated History															*
*   Rev		YYYY.MM.DD  Author			Description of Change				*
*	---		----------- ------			---------------------				*
*	0.1		2002.01.03	Derek Lau		copy from jb8-HID.asm				*
*	1.0		2002.10.10	Derek Lau		general release						*
*****************************************************************************
* This Program is a freeware to demonstrate the operation of Motorola		*
* Microcontroller. Motorola reserves the right to make changes without		*
* further notice to any product herein to improve reliability, function, or *
* design. Motorola does	not assume any liability arising out of the 		*
* application or use of any product, circuit, or software described herein; *
* neither does it convey any license under its patent rights nor the rights *
* of others. Motorola products are not designed, intended, or authorized 	*
* for use as components	in systems intended for surgical implant into the	*
* body, or other applications intended to support life, or for any other	*
* application in which the failure of the Motorola product could create a	*
* situation where personal injury or death may occur. Should Buyer purchase	*
* or use Motorola products for any such intended or unauthorized 			*
* application, Buyer shall indemnify and hold Motorola and its officers,	*
* employees, subsidiaries, affiliates, and distributors harmless against	*
* all claims, costs, damages, and expenses, and reasonable attorney fees	*
* arising out of, directly or indirectly, any claim of personal injury or	*
* death associated with such unintended or unauthorized use, even if such	*
* claim alleges that Motorola was negligent regarding the design or 		*
* manufacture of the part. Motorola	and the Motorola logo* are registered	*
* trademarks of Motorola Inc												*
*****************************************************************************
* Note:																		*
*																			*
* This interrupt routine has been tested under Win98 and Win2000 only		*
*																			*
* ====== Compile Options =================================================	*
;
 BASE 10


* --- Option to Check Request Fields & Device States
*     that may cause un-specified behavior of the device -------
;; (If conditions for un-specified behavior found, the device is stalled.)
CHK_UNSPECIFIED	set	0           ; check



* ====== MACRO Definition ================================================
;; INCLUDE    "macro8.asm"


 page
* ====== USB Subroutines(Top Level Handler) ==============================
* USB_Handler       Check Transaction Type & Branch to the Service Routine


* --------------------------------------------------------------------- *
* USB_Handler - Check Transaction Type & Branch to the Service Routine  *
*   ----  -----  ------  -------  --------  ---  ----  ---------------- *
*   IDLE  SETUP  IN_END  OUT_END  STAT_END  sup  data  Action           *
*   ----  -----  ------  -------  --------  ---  ----  ---------------- *
*     1     x       x        x        x      x     x   exit             *
*     0     1       x        x        x      0     x   jsr PROC_SETUP   *
*     0     x       1        x        x      x     0   no operation     *
*     0     x       x        1        x      x     0   jsr PROC_DATA    *
*     0     x       x        x        1      1     1   bring dev. idle  *
*   ----  -----  ------  -------  --------  ---  ----  ---------------- *
* In      : V_Transact          transfer status of control transfer     *
*           V_PROCESS           processing status of control transfer   *
* Out     : <nil>                                                       *
* Call    : PROC_SETUP, PROC_DATA, ECHO_IN_STATUS                       *
* --------------------------------------------------------------------- *
USB_Handler:
* --- Check Transact Status ----------------------------------------------
        brset   b_IDLE,V_Transact,UHR_EXIT  ; device Idle ?
*
*
*
* --- Process SETUP Packet -----------------------------------------------
UHR_SETUP:
        brset   b_sup,V_PROCESS,UHR_DATA     ; processed SETUP packet ?
        brclr   b_SETUP,V_Transact,UHR_DATA  ; ~SETUP stage completed ?
*
        jsr     PROC_SETUP              ; process SETUP packet
*
*
*
* --- Process IN/OUT DATA Packet -----------------------------------------
UHR_DATA:
;        brset   b_Stalled,V_Dev_Status,UHR_EXIT  ; device stalled ?
        brset   b_data,V_PROCESS,UHR_STAT      ; processed DATA packet ?
*
* --- IN DATA Stage Completed ----------------------------------
UHR_IN:
        brclr   b_IN_END,V_Transact,UHR_OUT    ; ~IN DATA stage completed ?
        bset    b_data,V_PROCESS        ; processed DATA packet
*
;; (OUT Status is ACK handshake, which is already enabled by
;;  clearing b_RXD0F bit in USB_ISR routine.)
;;        jsr     Echo_OUT_Status         ; response with OUT Status stage
*
* --- OUT DATA Stage Completed ---------------------------------
UHR_OUT:
        brclr   b_OUT_END,V_Transact,UHR_STAT  ; ~OUT DATA stage completed ?
*
        jsr     PROC_DATA               ; process Rx Data of the request
        jsr     ECHO_IN_STATUS          ; response with IN Status stage
*
*
*
* --- STATUS Stage Completed ---------------------------------------------
UHR_STAT:
        brclr   b_STAT_END,V_Transact,UHR_EXIT  ; ~STATUS stage completed ?
*
* (STATUS transact aborts all pending transactions and packets processing)
        MOV     #(1<<b_IDLE),V_Transact   ; device idle
        clr     V_PROCESS               ; reset processing status
*
;;        bra     UHR_EXIT
*
*
*
* ------------------------------------------------------------------------
UHR_EXIT:
        rts




 page
* ====== USB Subroutines(process SETUP packet) ===========================
* PROC_SETUP        branch to serve corresp. Device Request
*                     after SETUP stage


* --------------------------------------------------------------------- *
* PROC_SETUP - branch to serve corresp. Device Request                  *
* In      : b_Type[0..1]    =0      Standard Device Request             *
*                           =1      Class-specific Request              *
*                           =2      Vendor-specific Request             *
* Out     : <nil>                                                       *
* Call    : <see tables of PST_STD_REQ & PST_HID_REQ>                   *
* Note    :- b_sup is set after completely processed the SETUP packet   *
* --------------------------------------------------------------------- *
PROC_SETUP:
        bset    b_sup,V_PROCESS         ; processed SETUP packet
*
* --- Check Request Type: Standard or Class ------------------------------
        lda     V_bmReqType
        and     #%01100000
        KCMPEQ  %00000000,PST_STANDARD  ; Standard Device Request ?
        KCMPEQ  %00100000,PST_CLASS     ; Class-specific Request ?
        bra     PST_STALL               ; device stalled
*
* --- Parser for Standard Device Requests --------------------------------
* (IMPORTANT:- the parser branches to corresp. service routine
*              and never returns.)
PST_STANDARD:
 IFEQ CHK_UNSPECIFIED
*
* Check Request for device in DEFAULT state ------------------------------
PST_Chk_State:
	brclr	b_DEFAULT,V_Dev_State,PST_End_CState	; ~device in DEFAULT
*
	lda     V_bRequest
        KCMPEQ  K_SET_ADDRESS,PST_End_CState  ; SET_ADDRESS request ?
        KCMPEQ  K_GET_DESC,PST_End_CState     ; GET_DESC request ?
        bra     PST_STALL
PST_End_CState:
 ENDIF                  ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	
        lda     V_bRequest
        KCMPHI  12,PST_STALL            ; Request Code > 12 ?
        ldx     #3
        lsla
        add	V_bRequest		; V_bRequest * 3
        tax
        jmp     PST_STD_REQ,x           ; goto corresp. service routine
*
* --- Table of Standard Device Requests ------------------------
PST_STD_REQ:                            ; Req. Code:
        jmp     SRQ_GET_STATUS          ;  =  0
        jmp     SRQ_CLR_FEATURE         ;  =  1
        jmp     PST_STALL               ;  =  2
        jmp     SRQ_SET_FEATURE         ;  =  3
        jmp     PST_STALL               ;  =  4
        jmp     SRQ_SET_ADDRESS         ;  =  5
        jmp     SRQ_GET_DESC            ;  =  6
        jmp     SRQ_SET_DESC            ;  =  7     <not supported>
        jmp     SRQ_GET_CONFIG          ;  =  8
        jmp     SRQ_SET_CONFIG          ;  =  9
        jmp     SRQ_GET_IF              ;  = 10
        jmp     SRQ_SET_IF              ;  = 11
        jmp     SRQ_SYNC_FRAME          ;  = 12     <not supported>
*
*
*
* --- Echo Device Stalled ------------------------------------------------
PST_STALL:
        jsr     Echo_STALLED
        bra     PST_EXIT
*
*
*
* --- Parser for HID Class-specific Requests -----------------------------
* (IMPORTANT:- the parser branches to corresp. service routine
*              and never returns.)
PST_CLASS:
        lda     V_bmReqType
        and     #%01111111              ; neglect b_Dir bit
        KCMPEQ  %00100001,PST_CLSCTI    ; recipient = interface ?
        KCMPEQ  %00100010,PST_CLSCTI    ; recipient = endpoint ?
        bra     PST_STALL
PST_CLSCTI:
        lda     V_bRequest
        KCMPHI  11,PST_STALL            ; Request Code > 11 ?
        lsla
        add	V_bRequest
	tax
        jmp     PST_HID_REQ,x           ; goto corresp. service routine
*
* --- Table of HID Class-specific Requests ---------------------
PST_HID_REQ:                            ; Req. Code:
        jmp     PST_STALL               ;  =  0
        jmp     HCR_GET_REPORT          ;  =  1
        jmp     HCR_GET_IDLE            ;  =  2  Ŀ not used by
        jmp     HCR_GET_PROTOCOL        ;  =  3   Monitor Class
        jmp     PST_STALL               ;  =  4
        jmp     PST_STALL               ;  =  5
        jmp     PST_STALL               ;  =  6
        jmp     PST_STALL               ;  =  7
        jmp     PST_STALL               ;  =  8
        jmp     HCR_SET_REPORT          ;  =  9
        jmp     HCR_SET_IDLE            ;  = 10  Ŀ not used by
        jmp     HCR_SET_PROTOCOL        ;  = 11   Monitor Class
*
*
*
* ------------------------------------------------------------------------
PST_EXIT:
        rts




 page
* ====== USB Subroutines(Standard Device Requests) =======================
* SRQ_SET_FEATURE   serve SET_FEATURE Request
* SRQ_CLR_FEATURE   serve CLEAR_FEATURE Request
* SRQ_SET_CONFIG    serve SET_CONFIGURATION Request
* SRQ_GET_CONFIG    serve GET_CONFIGURATION Request
* SRQ_SET_IF        serve SET_INTERFACE Request
* SRQ_GET_IF        serve GET_INTERFACE Request
* SRQ_SET_ADDRESS   serve SET_ADDRESS Request
* SRQ_GET_STATUS    serve GET_STATUS Request
* SRQ_SYNC_FRAME    serve SYNCH_FRAME Request           [not supported]
* SRQ_SET_DESC      serve SET_DESCRIPTOR Request        [not supported]
* SRQ_GET_DESC      serve GET_DESCRIPTOR Request


* [Standard Device Request] ------------------------------------------- *
* SRQ_CLR_FEATURE - serve CLEAR_FEATURE Request                         *
* SRQ_SET_FEATURE - serve SET_FEATURE Request                           *
* In      : V_bmReqType     =0      Device Feature Request              *
*                           =1      Interface Feature Request           *
*                           =2      Endpoint Feature Request            *
*           V_wIndex_L      =0      <for Device>                        *
*                           =I/F #  <for Interface>                     *
*                           =EP #   <for Endpoint>                      *
*           V_wValue_L      Feature Selector                            *
*                           =0      ENDPOINT_STALL       <for Endpoint> *
*                           =1      DEVICE_REMOTE_WAKEUP <for device>   *
* Out     : b_RWAKEUP       =0      disable Device Remote Wakeup        *
*                           =1      enable Device Remote Wakeup         *
*           b_Stalled       =0      device NOT stalled                  *
*                           =1      device stalled                      *
* Call    : Echo_STALLED, RST_STALLED                                   *
* --------------------------------------------------------------------- *
K_RM_WAKEUP     equ     1       ; Ŀ Feature Selector Options
K_EP_STALL      equ     0       ; 
*
*
*
SRQ_CLR_FEATURE:
SRQ_SET_FEATURE:
*
* --- Check Constant Fields ----------------------------------------------
        lda     V_wValue_H
        ora     V_wIndex_H
 IFEQ CHK_UNSPECIFIED     ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        ora     V_wLength_L
        ora     V_wLength_H
 ENDIF                  ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        bne     SCF_STALL
*
*
* --- Check Recipient ----------------------------------------------------
        lda     V_bmReqType
        beq	SCF_DEVICE    ; recipient = device ?
        KCMPEQ  %00000001,SCF_INTERFACE ;           = interface ?
        KCMPEQ  %00000010,SCF_ENDPOINT  ;           = endpoint ?
*
* --- Echo Device Stalled ------------------------------------------------
SCF_STALL:
        jsr     Echo_STALLED
        bra     SCF_EXIT
*
*
*
* ------------------------------------------------------------------------
*
*
* --- Serve CLEAR_FEATURE/SET_FEATURE to Device --------------------------
SCF_DEVICE:
        lda     V_wIndex_L              ; must be zero
        bne     SCF_STALL               ;  for Device Feature
*
        lda     V_wValue_L              ; Feature Selector
        KCMPNE  K_RM_WAKEUP,SCF_STALL   ; = ~Device_Remote_Wakeup ?
*
        lda     V_bRequest
        KCMPEQ  K_CLR_FEATURE,SCF_DEV_CLR  ; CLEAR_FEATURE request ?
SCF_DEV_SET:
        bset    b_RWAKEUP,V_Dev_Status    ; enable Device Remote Wakeup
        bra     SCF_EXIT
SCF_DEV_CLR:
        bclr    b_RWAKEUP,V_Dev_Status    ; disable Device Remote Wakeup
        bra     SCF_EXIT
*
*
*
* --- Serve CLEAR_FEATURE/SET_FEATURE to Interface -----------------------
SCF_INTERFACE:
        bra     SCF_STALL
*
*
*
* --- Serve CLEAR_FEATURE/SET_FEATURE to Endpoint ------------------------
SCF_ENDPOINT:
        lda     V_wValue_L              ; Feature Selector
        KCMPNE  K_EP_STALL,SCF_STALL    ; = ~ENDPOINT_STALL request ?
*
        lda     V_bRequest
        KCMPEQ  K_CLR_FEATURE,SCF_EP_CLR  ; CLEAR_FEATURE request ?
*
*
* --- Serve SET_FEATURE to Endpoint ----------------------------
SCF_EP_SET:
        lda     V_wIndex_L              ; endpoint #
        KCMPEQ  $81,SCF_SET_EP1         ; endpoint #1 ?
        KCMPEQ  $82,SCF_SET_EP2         ; endpoint #2 ?
        bra     SCF_STALL
SCF_SET_EP1:
	brclr	b_CONFIG,V_Dev_State,SCF_STALL	; ~device in CONFIG?
        bset    b_STALL1,UCR1           ; stalled Epnt 1
        bra     SCF_EXIT
*
SCF_SET_EP2:
	brclr	b_CONFIG,V_Dev_State,SCF_STALL	; ~device in CONFIG?
        bset    b_STALL2,UCR2
        bra     SCF_EXIT
*
* --- Serve CLEAR_FEATURE to Endpoint --------------------------
SCF_EP_CLR:
        lda     V_wIndex_L              ; endpoint #
        KCMPEQ  $81,SCF_CLR_EP1         ; = Interrupt IN epnt 1 ?
        KCMPEQ	$82,SCF_CLR_EP2		; =  EP2
        and     #%00001111              ; neglect DIR bit
        beq     SCF_CLR_EP0             ;   = 0 ?
        bra     SCF_STALL

SCF_CLR_EP2:
        bclr    b_STALL2,UCR2           ; un-stalled EP2
        bra     SCF_EXIT

SCF_CLR_EP1:
        bclr    b_STALL1,UCR1           ; un-stalled EP1
        bra     SCF_EXIT

SCF_CLR_EP0:
        bclr    b_OSTALL0,UCR3           ; un-stalled EP0
        bclr    b_ISTALL0,UCR3

* ------------------------------------------------------------------------
SCF_EXIT:
        rts
*
*
*




* [Standard Device Request] ------------------------------------------- *
* SRQ_SET_CONFIG - serve SET_CONFIGURATION Request                      *
* In      : V_wValue_L      config. value                               *
* Out     : V_ConfigID      ID to select a config. descriptor           *
* Call    : Echo_STALLED                                                *
* --------------------------------------------------------------------- *
SRQ_SET_CONFIG:
*
* --- Check Constant Fields ----------------------------------------------
* --- Check Constant Fields ----------------------------------------------
 IFEQ CHK_UNSPECIFIED     ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        lda     V_wValue_H
        ora     V_wIndex_L
        ora     V_wIndex_H
        ora     V_wLength_L
        ora     V_wLength_H
        bne     SSC_STALL
 ENDIF                  ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
*
* --- Check Recipient ----------------------------------------------------
        lda     V_bmReqType
        beq     SSC_Update
;;        bne     SSC_STALL               ; recipient = ~device ?
*
* --- Echo Device Stalled -------
SSC_STALL:
        jsr     Echo_STALLED
        bra     SSC_EXIT
*
*
* --- Change to Selected Configuration -----------------------------------
SSC_Update:
        lda     V_wValue_L             ; config. ID
        beq     SSC_UNCONFIG
        CMPEQ   CFG_bCfgValue,SSC_Config  ; config ID != CFG_bCfgValue ?
        bra     SSC_STALL
*
* --- Set Device to Unconfigured State --------------------------
SSC_UNCONFIG:
        clr     V_ConfigID
*
        jsr     Kbd_Address             ; bring keyboard from
                                        ;  CONFIG to ADDRESS state
        bra     SSC_EXIT
*
* --- Set Device to One of the Configured States -------------
SSC_Config:
        sta     V_ConfigID
        jsr     Kbd_Config              ; bring keyboard to CONFIG state
*
* ------------------------------------------------------------------------
SSC_EXIT:
        rts




* [Standard Device Request] ------------------------------------------- *
* SRQ_GET_CONFIG - serve GET_CONFIGURATION Request                      *
* In      : V_ConfigID      ID to select a config. descriptor           *
* Out     : <nil>                                                       *
* Call    : Echo_STALLED, TRAN_IN                                       *
* Note    :- b_IN is set after IN transact started                      *
* --------------------------------------------------------------------- *
SRQ_GET_CONFIG:
*
* --- Check Constant Fields ----------------------------------------------
 IFEQ CHK_UNSPECIFIED     ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        lda     V_wLength_H
        ora     V_wValue_L
        ora     V_wValue_H
        ora     V_wIndex_L
        ora     V_wIndex_H
        bne     SGC_STALL
*
        lda     V_wLength_L            ; data size != 1 byte ?
        KCMPNE  1,SGC_STALL
 ENDIF                  ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
*
* --- Check Recipient ----------------------------------------------------
        lda     V_bmReqType
        KCMPEQ  %10000000,SGC_RTN_DATA  ; recipient = device ?
*
*
* --- Echo Device Stalled ------------------------------------------------
SGC_STALL:
        jsr     Echo_STALLED
        bra     SGC_EXIT
*
*
* --- Return Config. ID --------------------------------------------------
SGC_RTN_DATA:
        LMOV    V_ConfigID,(Q_Tx_Buf+0)  ; update USB Buffer Queue
*
        MOV     #$01,V_Tx_Size          ; negotiated data size=1 byte
        clr     V_Tx_Ptr
*
        MOV    #high(Q_Tx_Buf),V_Off_LDA_H  ; make D_Long_IDX routine
        MOV    #low(Q_Tx_Buf),V_Off_LDA_L  ; point to QD_Tx_Buf
                                           ; (used in TRAN_TX routine)
*
        bset    b_IN,V_Transact        ; IN transact started
        jsr     TRAN_IN                 ; force an IN transact
*
* ------------------------------------------------------------------------
SGC_EXIT:
        rts
*



* [Standard Device Request] ------------------------------------------- *
* SRQ_SET_IF - serve SET_INTERFACE Request                              *
* In      : <nil>                                                       *
* Out     : <nil>                                                       *
* Call    : <nil>                                                       *
* --------------------------------------------------------------------- *
SRQ_SET_IF:
        brclr   b_CONFIG,V_Dev_State,SSI_STALL     ; ~device in CONFIG ?

* --- Check Constant Fields ----------------------------------------------
        lda     V_wValue_H
        ora     V_wIndex_H
 IFEQ CHK_UNSPECIFIED     ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        ora     V_wLength_L
        ora     V_wLength_H
 ENDIF                  ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        bne     SSI_STALL
*
*
* --- Check Recipient ----------------------------------------------------
        lda     V_bmReqType
        KCMPNE  %00000001,SSI_STALL
*
        lda     V_wIndex_L             ; Interface # = 0 ?
;        KCMPEQ	1,ITS_S_IF_1		;             = 1 ?
;        bne     SSI_STALL               ;
	bra	SSI_STALL
*
ITS_S_IF_1:
        lda     V_wValue_L		; Only Default Setting supported
        beq     SSI_UPDATE              ; Alt. Setting = 0 ?
*
*
* --- Echo Device Stalled ------------------------------------------------
SSI_STALL:
        jsr     Echo_STALLED
        bra     SSI_EXIT
*
*
* --- Update I/F Alternate Setting # -------------------------------------
SSI_UPDATE:
;; (no Alt. Setting)
*
        bclr    b_STALL1,UCR1           ; un-stalled EP1
        bclr    b_STALL2,UCR2           ; un-stalled EP2
*
* ------------------------------------------------------------------------
SSI_EXIT:
        rts



* [Standard Device Request] ------------------------------------------- *
* SRQ_GET_IF - serve GET_INTERFACE Request                              *
* In      : <nil>                                                       *
* Out     : <nil>                                                       *
* Call    : Echo_STALLED, TRAN_IN                                       *
* --------------------------------------------------------------------- *
SRQ_GET_IF:
        brclr   b_CONFIG,V_Dev_State,SGI_STALL     ; ~device in CONFIG state ?

* --- Check Constant Fields ----------------------------------------------
        lda     V_wIndex_H
 IFEQ CHK_UNSPECIFIED     ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        ora     V_wLength_H
        ora     V_wValue_L
        ora     V_wValue_H
 ENDIF                  ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        bne     SGI_STALL
*
 IFEQ CHK_UNSPECIFIED     ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        lda     V_wLength_L            ; data size != 1 byte ?
        KCMPNE  1,SGI_STALL
 ENDIF                  ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
*
* --- Check Recipient ----------------------------------------------------
        lda     V_bmReqType
        KCMPNE  %10000001,SGI_STALL     ; recipient = ~device ?
*
        lda     V_wIndex_L              ; Interface # ?
	beq	SGI_RTN_DATA		; = 0 ?
        cbeqa	#1,SGI_RTN_DATA		; = 1 ?
;        beq     SGI_RTN_DATA
*
*
* --- Echo Device Stalled ------------------------------------------------
SGI_STALL:
        jsr     Echo_STALLED
        bra     SGI_EXIT
*
* --- Return I/F Setting Byte --------------------------------------------
SGI_RTN_DATA:
;; (no Alt. Setting)
        clra
        
        sta     (Q_Tx_Buf+0)            ; always return 0
*
        MOV     #$01,V_Tx_Size           ; negotiated data size=1 byte
        clr     V_Tx_Ptr
*
        MOV     #high(Q_Tx_Buf),V_Off_LDA_H  ; make Long_IDX routine
        MOV     #low(Q_Tx_Buf),V_Off_LDA_L  ; point to Q_Tx_Buf
                                        ; (used in TRAN_TX routine)
*
        bset    b_IN,V_Transact         ; IN transact started
        jsr     TRAN_IN                 ; force an IN transact
;        bra     SGI_EXIT
*
*
*
* ------------------------------------------------------------------------
SGI_EXIT:
        rts




* [Standard Device Request] ------------------------------------------- *
* SRQ_SET_ADDRESS - serve SET_ADDRESS Request                           *
* In      : V_wValue_L      new device address (cannot be zero)         *
* Out     : b_UPDADDR       =0      no pending address update           *
*                           =1      update device addr after IN Status  *
* Call    : Echo_STALLED                                                *
* --------------------------------------------------------------------- *
SRQ_SET_ADDRESS:
*
* --- Check Constant Fields ----------------------------------------------
* - ensure that V_wIndex_L/H, V_wLength_L/H & V_wValue_H are zero
*
 IFEQ CHK_UNSPECIFIED     ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        lda     V_wValue_H
        ora     V_wIndex_L
        ora     V_wIndex_H
        ora     V_wLength_L
        ora     V_wLength_H
        bne     SSA_STALL
 ENDIF                  ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
*
* --- Check Recipient ----------------------------------------------------
        lda     V_bmReqType
        beq     SSA_UPDATE              ; recipient = device ?
*
*
* --- Echo Device Stalled ------------------------------------------------
SSA_STALL:
        jsr     Echo_STALLED
        bra     SSA_EXIT
*
*
* --- Prepare to Set Address ---------------------------------------------
SSA_UPDATE:
        bset    b_UPDADDR,V_Dev_Status   ; update device addr
*
* ------------------------------------------------------------------------
SSA_EXIT:
        rts




* [Standard Device Request] ------------------------------------------- *
* SRQ_GET_STATUS - serve GET_STATUS Request                             *
* In      : <nil>                                                       *
* Out     : <nil>                                                       *
* Call    : Echo_STALLED, TRAN_IN                                       *
* Note    :- b_IN is set after IN transact started                      *
* --------------------------------------------------------------------- *
SRQ_GET_STATUS:
*
* --- Check Constant Fields ----------------------------------------------
 IFEQ CHK_UNSPECIFIED     ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        lda     V_wValue_L
        ora     V_wValue_H
        ora     V_wLength_H
        bne     SGS_STALL
*
        lda     V_wLength_L            ; data size != 2 byte ?
        KCMPNE  2,SGS_STALL
 ENDIF                  ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
*
* --- Check Recipient ----------------------------------------------------
        lda     V_bmReqType
        KCMPEQ  %10000000,SGS_DEVICE    ; recipient = device ?
        KCMPEQ  %10000001,SGS_INTERFACE ;           = interface ?
        KCMPEQ  %10000010,SGS_ENDPOINT  ;           = endpoint ?

* --- Echo Device Stalled ------------------------------------------------
SGS_STALL:
        jsr     Echo_STALLED
	bra	SGS_Exit
*
*
*
*
* --- Serve GET_STATUS to Device -----------------------------------------
SGS_DEVICE:
 IFEQ CHK_UNSPECIFIED     ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        lda     V_wIndex_L
        ora     V_wIndex_H
        bne     SGS_STALL
 ENDIF
        lda     V_Dev_Status              ; return b_SelPwr & b_RWAKEUP
        and     #%00000011
        sta     (Q_Tx_Buf+0)
*
        clra
        sta     (Q_Tx_Buf+1)
*
        bra     SGS_TXDATA
*
*
*
* --- Serve GET_STATUS to Interface --------------------------------------
SGS_INTERFACE:
        lda     V_wIndex_L             ; Interface #
        ora     V_wIndex_H
        KCMPEQ	1,SGS_IF1
        bne     SGS_STALL               ;   = 0 ?
*
SGS_IF1:
        clra
        sta     (Q_Tx_Buf+0)
        sta     (Q_Tx_Buf+1)
*
        bra     SGS_TXDATA
*
*
* --- Serve GET_STATUS to Endpoint ---------------------------------------
SGS_ENDPOINT:
        lda     V_wIndex_H
        bne     SGS_STALL
*
        lda     V_wIndex_L             ; Endpoint #
        KCMPEQ  $81,SGS_EP1             ;   = Interrupt IN Epnt 1 ?
        KCMPEQ	$82,SGS_EP2		;   = Interrupt IN Epnt 2 ?
        and     #%00001111              ;   (neglect DIR bit)
        beq     SGS_EP0                 ;   = Control Epnt 0 ?
        bra     SGS_STALL
*
* --- Return Endpoint 1 STALL status -------------
SGS_EP1:
        brclr   b_CONFIG,V_Dev_State,SGS_STALL  ; ~device in CONFIG state ?
*
        clra
        brclr   b_STALL1,UCR1,SGS_EP1_NOSTALL
SGS_EP1_STALL:
        lda     #$01                    ; signal stalled status
SGS_EP1_NOSTALL:
        sta     (Q_Tx_Buf+0)
*
        clra
        sta     (Q_Tx_Buf+1)
*
        bra     SGS_TXDATA
*
*
* --- Return Endpoint 1 STALL status -------------
SGS_EP2:
        brclr   b_CONFIG,V_Dev_State,SGS_STALL  ; ~device in CONFIG state ?
*
        clra
        brclr   b_STALL2,UCR2,SGS_EP2_NOSTALL
SGS_EP2_STALL:
        lda     #$01                    ; signal stalled status
SGS_EP2_NOSTALL:
        sta     (Q_Tx_Buf+0)
*
        clra
        sta     (Q_Tx_Buf+1)
*
        bra     SGS_TXDATA
*
* --- Return Endpoint 0 STALL status -------------
;; (if Epnt 0 stalled, no echo for GET_STATUS request)
SGS_EP0:
        clra
        sta     (Q_Tx_Buf+0)
        sta     (Q_Tx_Buf+1)
*
*
*
* --- Initialize IN Data Stage -------------------------------------------
SGS_TXDATA:
        MOV     #$02,V_Tx_Size          ; negotiated data size=2 bytes
        clr     V_Tx_Ptr
*
        MOV     #high(Q_Tx_Buf),V_Off_LDA_H  ; make D_Long_IDX routine
        MOV     #low(Q_Tx_Buf),V_Off_LDA_L  ; point to QD_Tx_Buf
                                            ; (used in TRAN_TX routine)
*
        bset    b_IN,V_Transact        ; IN transact started
        jsr     TRAN_IN                 ; force an IN transact
*
*
* ------------------------------------------------------------------------
SGS_Exit:
        rts



* [Standard Device Request] ------------------------------------------- *
* SRQ_SYNC_FRAME - serve SYNCH_FRAME Request                            *
*                  (this option is NOT supported)                       *
* In      : <nil>                                                       *
* Out     : <nil>                                                       *
* Call    : Echo_STALLED                                                *
* Note    :- call to unsupported request will cause device stalled.     *
* --------------------------------------------------------------------- *
SRQ_SYNC_FRAME:
;;        jsr     Echo_STALLED
*
*
*
* ------------------------------------------------------------------------
SSF_EXIT:
        rts




* [Standard Device Request] ------------------------------------------- *
* SRQ_SET_DESC - serve SET_DESCRIPTOR Request                           *
*                (this option is NOT supported)                         *
* In      : <nil>                                                       *
* Out     : <nil>                                                       *
* Call    : Echo_STALLED                                                *
* Note    :- call to unsupported request will cause device stalled.     *
* --------------------------------------------------------------------- *
SRQ_SET_DESC:
        bra     SSD_STALL               ; Request NOT supported
;; (SET_DESCRIPTOR not supported)
;;*
;;* --- Check Parameters ---------------------------------------------------
;;        lda     V_bmReqType
;;        bne     SSD_STALL               ; recipient = ~device ?
;;*
;;        lda     V_wValue_H
;;        KCMPHI  5,SSD_STALL             ; unsupported Desc. Type ?
;;*
;;;; (already checked in TSP_OUT routine)
;;        lda     V_wLength_L
;;        beq     SSD_STALL               ; Rx bytes = 0 ?
;;;; (already checked in TSP_OUT routine)
;;;;        cmp     #K_BUF_SIZE
;;;;        bhi     TSP_STALL               ; Rx bytes > buffer size ?
;;*
;;        bclr    b_data,V_PROCESS        ; process after OUT Data Rxd
;;*
;;        bra     SSD_EXIT
*
* --- Echo Device Stalled ------------------------------------------------
SSD_STALL:
;;        jsr     Echo_STALLED
*
* ------------------------------------------------------------------------
SSD_EXIT:
        rts




* [Standard Device Request] ------------------------------------------- *
* SRQ_GET_DESC - serve GET_DESCRIPTOR Request                           *
* In      : <nil>                                                       *
* Out     : <nil>                                                       *
* Call    : Echo_STALLED, TRAN_IN                                       *
* Note    :- only support GET_DESC request for DEVICE, CONFIG, STRING,  *
*            HID, & REPORT Desc.                                        *
*          - for GET_DESC request of Config Desc., it returns the       *
*            Config Desc., all I/F Desc., all HID Desc., and all        *
*            Endpoint Desc.                                             *
*          - b_IN is set after IN transact started                      *
* --------------------------------------------------------------------- *
SRQ_GET_DESC:
*
* --- Check Constant Fields ----------------------------------------------
        lda     V_wLength_H
        beq     SGD_NOVERFLOW
        MOV     #$FF,V_wLength_L        ; signal req. data size overflow
SGD_NOVERFLOW:
*
*
* --- Check Recipient ----------------------------------------------------
        brclr   b_Dir,V_bmReqType,SGD_STALL ; ~device-to-host ?
*
        lda     V_bmReqType
        and     #%00011111
        KCMPEQ  0,SGD_DEVICE            ; recipient = device ?
        KCMPEQ  1,SGD_IFACE             ; recipient = interface ?
        KCMPEQ  2,SGD_EPNT              ; recipient = endpoint ?
        bra     SGD_STALL               ; ~recipient = device ?
*
*
*
* --- Serve GET_DESC for DEVICE Recipient --------------------------------
SGD_DEVICE:
*
* --- Check Descriptor Type(for DEVICE) ------------------------

        lda     V_wValue_H
        KCMPEQ  K_DTYPE_DEV,SGD_DESC_DEVICE  ; get Device Desc. ?
        KCMPEQ  K_DTYPE_CFG,SGD_DESC_CONFIG  ; get Config Desc. ?
        KCMPEQ  K_DTYPE_STG,SGD_DESC_STRING  ; get String Desc. ?
        bra     SGD_STALL
*
*
*
* --- Serve GET_DESC for DEVICE Desc. ------------------------------------
SGD_DESC_DEVICE:
* (no Lang ID)
        lda     V_wIndex_L		; Lang. ID should be zero
        ora     V_wIndex_H
        bne     SGD_STALL
*
* --- Init. Long_IDX point to DEVICE_DESC queue ---
        LMOV    (Dev_bLength),V_Tx_Size
;		lda		$FEC3
*
        mov		#high(DEVICE_DESC),V_Off_LDA_H		; make Long_IDX routine
        mov    #low(DEVICE_DESC),V_Off_LDA_L      ; point to DEVICE_DESC queue
*                                               ; (used in TRAN_TX routine)
        jmp     SGD_TX_INIT
*
*
*
* --- Serve GET_DESC for CONFIG Desc. ------------------------------------
SGD_DESC_CONFIG:
* (no Lang ID)
        lda     V_wIndex_L		; Lang. ID should be zero
        ora     V_wIndex_H
        bne     SGD_STALL
*
* --- Init. Long_IDX point to CONFIG_DESC queue ---
        LMOV    CFG_wTotLength,V_Tx_Size
*
        MOV     #high(CONFIG_DESC),V_Off_LDA_H  ; make Long_IDX routine
        MOV     #low(CONFIG_DESC),V_Off_LDA_L  ; point to CONFIG_DESC queue
*                                               ; (used in TRAN_TX routine)
        jmp     SGD_TX_INIT
*
*
*
* --- Serve GET_DESC for STRING Desc. ------------------------------------
SGD_DESC_STRING:
;;        lda     V_wIndex_L
;;        CMPNE   STG0_bString,SGD_STALL  ; diff. Lang ID ?
*
        lda     V_wValue_L
        KCMPHI  (STG_PTR_NUM-1),SGD_STALL  ; invalid desc. idx ?
*
* --- Init. Long_IDX point to STRING_DESC[V_wValue_L] queue -------------
        lsla
        tax
*
        lda     STG_PTR_TABLE,x         ; make Long_IDX routine
        sta     V_Off_LDA_H                 ; point to STRING_DESC[
        lda     (STG_PTR_TABLE+1),x     ; V_wValue_L] queue
        sta     V_Off_LDA_L                 ; (used in TRAN_TX routine)
*
        clrx                            ; get desc. size
        MOV     #V_Tx_Size,V_Off_STA
        jsr     Long_IDX

        bra     SGD_TX_INIT
*
*
*
* --- Serve GET_DESC for INTERFACE Recipient -----------------------------
SGD_IFACE:
* --- Check Interface # ----------------------------------------
        lda     V_wIndex_L             ; Interface #
        beq     SGD_IFACE_0             ;  = 0 ?
		KCMPEQ	1,SGD_IFACE_1		;  = 1 ?

* --- Echo Device Stalled ------------------------------------------------
* (placed here to enable 8-bit "BRA SGD_STALL" instructions in
*  SRQ_GET_DESC routine)
SGD_STALL:
        jsr     Echo_STALLED
        bra     SGD_EXIT
*
*
* --- Serve GET_DESC for ENDPOINT Recipient ------------------------------
SGD_EPNT:
*
* --- Check Endpoint # -----------------------------------------
        lda     V_wIndex_L             ; Endpoint #
        and     %00001111               ; neglect endpoint direction
        beq     SGD_EPNT_1              ;  = 0 ?
        KCMPEQ  1,SGD_EPNT_1            ;  = 1 ?
        KCMPEQ	2,SGD_EPNT_2		;  = 2 ?
        bra     SGD_STALL
*
*
* --- Check Descriptor Type(for Keyboard INTERFACE/ENDPOINT) -------------
SGD_IFACE_0:
SGD_EPNT_1:
        lda     V_wValue_H
        KCMPEQ  K_DTYPE_HID,SGD_DESC_HID0  ; get HID Desc. ?      Ŀ
        KCMPEQ  K_DTYPE_RPT,SGD_DESC_RPT0  ; get Report Desc. ?     Class

        bra     SGD_STALL
*
SGD_IFACE_1:
SGD_EPNT_2:
        lda     V_wValue_H
        KCMPEQ  K_DTYPE_HID,SGD_DESC_HID1  ; get HID Desc. ?      Ŀ
        KCMPEQ  K_DTYPE_RPT,SGD_DESC_RPT1  ; get Report Desc. ?     Class
        bra     SGD_STALL

*
* --- Serve GET_DESC for Keyboard HID Desc. ------------------------------
SGD_DESC_HID0:
* (no Lang ID)
*
* --- Init. Long_IDX point to KHID_DESC queue ---
        LMOV    HID0_bLength,V_Tx_Size
*
        mov     #high(HID0_DESC),V_Off_LDA_H        ; make Long_IDX routine
        mov     #low(HID0_DESC),V_Off_LDA_L        ; point to KHID_DESC queue
*                                               ; (used in TRAN_TX routine)
*
        bra     SGD_TX_INIT
*
SGD_DESC_HID1:
* (no Lang ID)
*
* --- Init. Long_IDX point to KHID_DESC queue ---
        LMOV    HID1_bLength,V_Tx_Size
*
        mov    #high(HID1_DESC),V_Off_LDA_H        ; make Long_IDX routine
        mov    #low(HID1_DESC),V_Off_LDA_L        ; point to KHID_DESC queue
*                                               ; (used in TRAN_TX routine)
*
        bra     SGD_TX_INIT
*
*
* --- Serve GET_DESC for Keyboard REPORT Desc. ---------------------------
SGD_DESC_RPT0:
* (no Lang ID)
*
* --- Init. Long_IDX point to KREPORT_DESC queue ---
        LMOV    HID0_wCDesLgth,V_Tx_Size
*
        MOV     #high(REPORT0_DESC),V_Off_LDA_H ; make Long_IDX routine
        MOV     #low(REPORT0_DESC),V_Off_LDA_L ; point to KREPORT_DESC queue
*                                               ; (used in TRAN_TX routine)
*
        bra     SGD_TX_INIT
*
*
* --- Serve GET_DESC for I/F #1 REPORT Desc. ------------------------------
SGD_DESC_RPT1:
* (no Lang ID)
*
* --- Init. D_Long_IDX point to REPORT0_DESC queue ---
        LMOV    HID1_wCDesLgth,V_Tx_Size
*
        MOV     #high(REPORT1_DESC),V_Off_LDA_H  ; make D_Long_IDX routine
        MOV     #low(REPORT1_DESC),V_Off_LDA_L  ; point to REPORT0_DESC
                                               ; (used in TRAN_TX routine)
*
        bra     SGD_TX_INIT
*
*
*
* --- Initialize IN Data Stage -------------------------------------------
SGD_TX_INIT:
        lda     V_wLength_H
        bne     SGD_LGTH
*
        lda     V_Tx_Size
        CMPLO   V_wLength_L,SGD_LGTH    ; actual length > neg. length ?
        MOV     V_wLength_L,V_Tx_Size   ; use negotiated data length
SGD_LGTH:
*
        clr     V_Tx_Ptr
*
        bset    b_IN,V_Transact         ; IN transact started
        jsr     TRAN_IN                 ; force an IN transact
*
*
*
* ------------------------------------------------------------------------
SGD_EXIT:
        rts




 page
* ====== USB Subroutines(Class Specific Requests) ========================
* HCR_SET_PROTOCOL  serve SET_PROTOCOL Request
* HCR_GET_PROTOCOL  serve GET_PROTOCOL Request
* HCR_SET_IDLE      serve SET_IDLE Request
* HCR_GET_IDLE      serve GET_IDLE Request
* HCR_SET_REPORT    serve SET_REPORT Request
* HCR_GET_REPORT    serve GET_REPORT Request


* [Class Specific Request] -------------------------------------------- *
* HCR_SET_PROTOCOL - serve SET_PROTOCOL Request                         *
* In      : V_wValue_L      =0      Boot Protocol                       *
*                           =1      Report Protocol                     *
* Out     : V_Protocol      =V_wValue_L                                 *
* Call    : Echo_STALLED                                                *
* --------------------------------------------------------------------- *
HCR_SET_PROTOCOL:
*
* --- Check Constant Fields ----------------------------------------------
        lda     V_wValue_H
        ora     V_wIndex_H
 IFEQ CHK_UNSPECIFIED     ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        ora     V_wLength_L
        ora     V_wLength_H
 ENDIF                  ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        bne     HSP_STALL
*
*
* --- Check Recipient & Parameters ---------------------------------------
        brset   b_Dir,V_bmReqType,HSP_STALL  ; ~host-to-device ?
*
        lda     V_wIndex_L             ; Interface # = 0 ?
	beq	HSP_IF0
        KCMPEQ	1,HSP_IF1
*
*
* --- Echo Device Stalled ------------------------------------------------
HSP_STALL:
        jsr     Echo_STALLED
        bra     HSP_EXIT
*
*
* --- Change to Selected Protocol for Kbd I/F 0 --------------------------
HSP_IF0:
        lda     V_wValue_L
        KCMPHI  1,HSP_STALL
        sta     V_IF0_Ptcl
        bra     HSP_EXIT
*
* --- Change to Selected Protocol for I/F 0 ------------------------------
HSP_IF1:
        lda     V_wValue_L             ; update selected protocol
        KCMPHI  1,HSP_STALL
        sta     V_IF1_Ptcl

* ------------------------------------------------------------------------
HSP_EXIT:
        rts




* [Class Specific Request] -------------------------------------------- *
* HCR_GET_PROTOCOL - serve GET_PROTOCOL Request                         *
* In      : V_Protocol      =0      Boot Protocol                       *
*                           =1      Report Protocol                     *
* Out     : <nil>                                                       *
* Call    : Echo_STALLED, TRAN_IN                                       *
* --------------------------------------------------------------------- *
HCR_GET_PROTOCOL:
*
* --- Check Constant Fields ----------------------------------------------
        lda     V_wIndex_H
        ora     V_wLength_H
 IFEQ CHK_UNSPECIFIED     ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        ora     V_wValue_L
        ora     V_wValue_H
 ENDIF                  ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        bne     HGP_STALL
*
        lda     V_wLength_L
        KCMPNE  1,HGP_STALL             ; ~1-byte Data ?
*
*
* --- Check Recipient & Parameters ---------------------------------------
        brclr   b_Dir,V_bmReqType,HSP_STALL  ; ~device-to-host ?
*
        lda     V_wIndex_L              ; Interface # ?
	beq	HGP_IF0			;  = 0 ?
        KCMPEQ	1,HGP_IF1		;  = 1 ?
;        beq     HGP_IF0                   
*
*
* --- Echo Device Stalled ------------------------------------------------
HGP_STALL:
        jsr     Echo_STALLED
        bra     HGP_EXIT
*
*
* --- Return I/F #1 Protocol Number ---------------------------------------
*  Always return protocol# = 1 (Report Protocol)
HGP_IF1:
        lda	V_IF1_Ptcl	; return I/F #1 protocol#
        bra	HGP_TXDTA
*
*
* --- Return I/F 0 Protocol Number ---------------------------------------
HGP_IF0:
        lda	V_IF0_Ptcl
        sta	(Q_Tx_Buf+0)  ; return I/F #0 protocol#
*
*
* --- Initialize IN Data Stage -------------------------------------------
HGP_TXDTA:
        sta	(Q_Tx_Buf+0)  ; return I/F #0 protocol#

        MOV     #$01,V_Tx_Size          ; negotiated data size=1 byte
        
        clr     V_Tx_Ptr
*
        MOV     #high(Q_Tx_Buf),V_Off_LDA_H  ; make D_Long_IDX routine
        MOV     #low(Q_Tx_Buf),V_Off_LDA_L  ; point to QD_Tx_Buf
                                            ; (used in TRAN_TX routine)
*
        bset    b_IN,V_Transact        ; IN transact started
        jsr     TRAN_IN                 ; force an IN transact
*
*
* ------------------------------------------------------------------------
HGP_EXIT:
        rts




* [Class Specific Request] -------------------------------------------- *
* HCR_SET_IDLE - serve SET_IDLE Request                                 *
* In      : V_wValue_L      Report ID                                   *
*           V_wValue_H      Report Idle Duration [in 4ms]               *
* Out     : V_Kbd_Idle      =V_wValue_H (keyboard report idle timeout)  *
* Call    : Echo_STALLED                                                *
* --------------------------------------------------------------------- *
HCR_SET_IDLE:
*
* --- Check Constant Fields ----------------------------------------------
        lda     V_wIndex_H

 IFEQ CHK_UNSPECIFIED     ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        ora     V_wLength_L
        ora     V_wLength_H
 ENDIF                  ; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        bne     HSI_STALL
*
*
* --- Check Recipient & Parameters ---------------------------------------
        lda     V_bmReqType
        KCMPEQ  %00100001,HSI_CHK_IF
        KCMPEQ  %00100010,HSI_CHK_EPNT
        bra     HSI_STALL
HSI_CHK_IF:
        lda     V_wIndex_L              ; Interface # ?
	beq	HSI_IF0			; = 0 ?
        cbeqa	#1,HSI_IF1		; = 1 ?
;        beq     HSI_IF0                 ;   
        bra     HSI_STALL
HSI_CHK_EPNT:
        lda     V_wIndex_L             ; Endpoint #
        cbeqa	#1,HSI_EP1              ;  = 1 ?
        cbeqa	#2,HSI_EP2		;  = 2 ?
*
*
* --- Echo Device Stalled ------------------------------------------------
HSI_STALL:
        jsr     Echo_STALLED
        bra     HSI_EXIT
*
*
* --- Change to I/F 0 Report Idle Timeout Period -------------------------
HSI_EP1:
HSI_IF0:
        lda     V_wValue_H
        sta     V_IF0_Idle              ; update I/F 0 report idle rate
        sta     VC_IF0_Idle             ; update I/F 0 idle timeout cntr
	bra	HSI_EXIT


HSI_EP2:
HSI_IF1:
	lda	V_wValue_H
        ldx     V_wValue_L
        beq	HSI_Set_AllRPID

* --- Set Individual Idle Rate -----------------------------------------
        sta     (V_IF1R1_Idle-1),x	; update IF1 ID#x report idle rate
        sta     (VC_IF1R1_Idle-1),x	; update IF1 ID#x idle timeout cntr
        rts

* --- Set all Idel Rate when ID = 0 --------------------------------------

HSI_Set_AllRPID:

	ldx	#K_N_IF1RPTID		; Total 1 Report ID for IF1

HSI_S_A_L:
	sta	(V_IF1R1_Idle-1),x	; Set the reload value for IF1
        sta	(VC_IF1R1_Idle),x	; Set the counter for IF1
	dbnzx	HSI_S_A_L

; ------------------------------------------------------------------------
HSI_EXIT:
	rts



* [Class Specific Request] -------------------------------------------- *
* HCR_GET_IDLE - serve GET_IDLE Request                                 *
* In      : V_Kbd_Idle      =V_wValue_H (keyboard report idle timeout)  *
* Out     : <nil>                                                       *
* Call    : Echo_STALLED, TRAN_IN                                       *
* --------------------------------------------------------------------- *
HCR_GET_IDLE:
* --- Check Constant Fields ----------------------------------------------
        lda     V_wValue_H
        ora     V_wIndex_H
        ora     V_wLength_H
        bne     HGI_STALL
*
        lda     V_wLength_L
        KCMPNE  1,HGI_STALL             ; ~1-byte Data ?
*
*
* --- Check Recipient & Parameters ---------------------------------------
        lda     V_bmReqType
        KCMPEQ  %10100001,HGI_CHK_IF
        KCMPEQ  %10100010,HGI_CHK_EPNT
        bra     HGI_STALL
HGI_CHK_IF:
        lda     V_wIndex_L              ; Interface #
	beq	HGI_IF0			;  = 0 ?
        cbeqa	#1,HGI_IF1		;  = 1 ?
;       beq     HGI_IF0                
        bra     HGI_STALL
HGI_CHK_EPNT:
        lda     V_wIndex_L             ; Endpoint #
        cbeqa	#1,HGI_EP1              ;  = 1 ?
        cbeqa	#2,HGI_EP2		;  = 2 ?
*
*
* --- Echo Device Stalled ------------------------------------------------
HGI_STALL:
        jsr     Echo_STALLED
        bra     HGI_EXIT
*
*
* --- Return I/F 0 Report Idle Timeout Period ----------------------------
HGI_EP1:
HGI_IF0:
        LMOV    V_IF0_Idle,(Q_Tx_Buf+0)  ; return I/F 0 report idle rate
*
*
*
* --- Initialize IN Data Stage -------------------------------------------
HGI_TXDTA:
        MOV     #$01,V_Tx_Size          ; negotiated data size=1 byte
        clr     V_Tx_Ptr
*
        MOV     #high(Q_Tx_Buf),V_Off_LDA_H  ; make D_Long_IDX routine
        MOV     #low(Q_Tx_Buf),V_Off_LDA_L  ; point to QD_Tx_Buf
                                            ; (used in TRAN_TX routine)
*
        bset    b_IN,V_Transact        ; IN transact started
        jsr     TRAN_IN                 ; force an IN transact
*
*
* ------------------------------------------------------------------------
HGI_EXIT:
        rts

*
* --- Return I/F #1 Report Idle Timeout Period ----------------------------
HGI_EP2:
HGI_IF1:
;        mov     V_IF1_Idle,(Q_Tx_Buf+0)    ;store the Idle Rate

        ldx     V_wValue_L              ; Get the Report ID
        lda	(V_IF1R1_Idle-1),x	; Get the Idle Rate

        sta     (Q_Tx_Buf+0)            ; return I/F #1 report idle rate
*
	bra     HGI_TXDTA
*


* [Class Specific Request] -------------------------------------------- *
* HCR_SET_REPORT - serve SET_REPORT Request                             *
* In      : <nil>                                                       *
* Out     : <nil>                                                       *
* Call    : Echo_STALLED                                                *
* Note    :- see also HDA_SET_REPORT                                    *
* --------------------------------------------------------------------- *
HCR_SET_REPORT:
*
* --- Check Constant Fields ----------------------------------------------
        lda     V_wIndex_H
        ora     V_wLength_H
        beq     HSR_EXIT
*
*
* --- Echo Device Stalled ------------------------------------------------
HSR_STALL:
        jsr     Echo_STALLED
*
*
* ------------------------------------------------------------------------
HSR_EXIT:
        rts




* [Class Specific Request] -------------------------------------------- *
* HCR_GET_REPORT - serve GET_REPORT Request                             *
* In      : QC_IN0_Rpt      IN Report of I/F 0                          *
*           Q_OUT0_Rpt      OUT Report of I/F 0                         *
*           K_IRPT0_SIZE    size of QC_IN0_Rpt                          *
*           K_ORPT0_SIZE    size of Q_OUT0_Rpt                          *
* Out     : QD_Tx_Buf       = QC_IN0_Rpt(for get IN Report, I/F 0)      *
*                           = Q_OUT0_Rpt(for get OUT Report, I/F 0)     *
*           V_Tx_Size      = K_IRPT0_SIZE(for get IN Report, I/F 0)    *
*                           = K_ORPT0_SIZE(for get OUT Report, I/F 0)   *
* Call    : Echo_STALLED, TRAN_IN                                       *
* Note    :- b_IN is set after IN transact started                      *
* --------------------------------------------------------------------- *
HCR_GET_REPORT:
*
* --- Check Constant Fields ----------------------------------------------
*
*
* --- Check Recipient & Parameters ---------------------------------------
        lda     V_bmReqType
        KCMPEQ  %10100001,HGR_CHK_IF
        KCMPEQ  %10100010,HGR_CHK_EPNT
        bra     HGR_STALL
HGR_CHK_IF:
        lda     V_wIndex_L              ; Interface #
	beq	HGR_IF0			; = 0 ?
        cbeqa	#1,HGR_IF1		;  = 1 ?
;        beq     HGR_IF0                
        bra     HGR_STALL
HGR_CHK_EPNT:
        lda     V_wIndex_L             ; Endpoint #
        KCMPEQ  1,HGR_EP1               ;  = 1 ?
        KCMPEQ	2,HGR_EP2		;  = 2 ?
*
*
* --- Echo Device Stalled ------------------------------------------------
HGR_STALL:
        jsr     Echo_STALLED

        rts			; EXIT the Get REPORT request
*
*
* --- Get IN/OUT Reports of ----------------------------------------------
HGR_EP1:
HGR_IF0:
        lda     V_wIndex_H
        ora     V_wLength_H
        bne     HGR_STALL

        lda     V_wValue_H
        KCMPEQ  $01,HGR_INPUT           ; Report Type=Input ?
        KCMPEQ  $02,HGR_OUTPUT          ; Report Type=Output ?
        KCMPEQ	$03,HGR_Feature			; Report Type=Feature ?
        bra     HGR_STALL
*
* --- Return IN Report -----------------------------------------
HGR_INPUT:
        ldx     V_wLength_L
        KCPXHI  K_ORPT0_SIZE,HGR_STALL  ; ~Report Length > Buffer Size ?
        stx     V_Tx_Size
HGR_IP_ILOOP:
        lda     (QC_IN0_Rpt-1),x
        sta     (Q_Tx_Buf-1),x
        DBNZX   HGR_IP_ILOOP            ; copy data to USB Buffer Queue
*
        bra     HGR_TX_INIT


HGR_Feature:
* --- Init. Long_IDX point to KeyID begin address queue ---
*
        ldx     V_wLength_L
        KCPXHI  K_Feat0_Size,HGR_STALL  ; ~Report Length > Buffer Size ?
        stx     V_Tx_Size
HGR_FE_ILOOP:
        lda     (Q_Feat0_Rpt-1),x
        sta     (Q_Tx_Buf-1),x
        DBNZX   HGR_FE_ILOOP            ; copy data to USB Buffer Queue
*
		lda		#$FF
		sta		Q_Feat0_Rpt

        bra     HGR_TX_INIT

*
* --- Return OUT Report ----------------------------------------
HGR_OUTPUT:
         ldx     V_wLength_L
        KCPXHI  K_ORPT0_SIZE,HGR_STALL  ; Report Length > Buffer Size ?
        stx     V_Tx_Size
HGR_OP_OLOOP:
        lda     (Q_OUT0_Rpt-1),x
        sta     (Q_Tx_Buf-1),x
        DBNZX   HGR_OP_OLOOP            ; copy data to USB Buffer Queue
*
        bra     HGR_TX_INIT
*
*
* --- Get IN/OUT Reports of ----------------------------------------------
HGR_EP2:
HGR_IF1:
        lda     V_wIndex_H
        ora     V_wLength_H
        bne     HGR_STALL

        lda     V_wValue_H
        KCMPEQ  $01,HGR_INPUT_1           ; Report Type=Input ?
        KCMPEQ  $02,HGR_OUTPUT_1          ; Report Type=Output ?
        bra     HGR_STALL
*
* --- Return IN Report -----------------------------------------
HGR_INPUT_1:
        ldx     V_wLength_L
        KCPXHI  K_ORPT1_SIZE,HGR_STALL  ; Report Length > Buffer Size ?
        stx     V_Tx_Size		; Set the move counter

HGR_IP_ACPI:
;	lda	#K_Dummy		; Force the dummy bytes = $55
;	sta	V_ACPI_Dummy1
;	sta	V_ACPI_Dummy2

*
* --- Return OUT Report ----------------------------------------
HGR_OUTPUT_1:
        ldx     V_wLength_L
        KCPXHI  K_ORPT1_SIZE,HGR_STALL  ; Report Length > Buffer Size ?
        stx     V_Tx_Size
HGR_OP_OLOOP_1:
        lda     (Q_Out1_Rpt-1),x
        sta     (Q_Tx_Buf-1),x
        DBNZX   HGR_OP_OLOOP_1          ; copy data to USB Buffer Queue
*

*
*
* --- Initialize IN Data Stage -------------------------------------------
HGR_TX_INIT:
        clr     V_Tx_Ptr
*
        MOV     #high(Q_Tx_Buf),V_Off_LDA_H  ; make D_Long_IDX routine
        MOV     #low(Q_Tx_Buf),V_Off_LDA_L  ; point to QD_Tx_Buf
                                            ; (used in TRAN_TX routine)
*
        bset    b_IN,V_Transact        ; IN transact started
        jsr     TRAN_IN                 ; force an IN transact
*
*
* ------------------------------------------------------------------------
HGR_EXIT:
        rts




 page
* ====== USB Subroutines(process SETUP packet) ===========================
* PROC_DATA         branch to serve corresp. Device Request
*                     after OUT DATA stage
* HDA_SET_REPORT    process data of SET_REPORT Request


* --------------------------------------------------------------------- *
* PROC_DATA - branch to serve corresp. Dev Request after OUT DATA stage *
* In      : b_Type[0..1]    =0      Standard Device Request             *
*                           =1      Class-specific Request              *
*                           =2      Vendor-specific Request             *
* Out     : <nil>                                                       *
* Call    : <see tables of PDA_STD_REQ & PDA_HID_REQ>                   *
* Note    :- b_OUT is cleared after completely processed the DATA pkt   *
* --------------------------------------------------------------------- *
PROC_DATA:
        bset    b_data,V_PROCESS       ; processed DATA packet
*
*
* --- Check Request Type: Standard or Class ------------------------------
        lda     V_bmReqType
        and     #%01100000
        KCMPEQ  %00000000,PDA_STANDARD  ; Standard Device Request ?
        KCMPEQ  %00100000,PDA_CLASS     ; Class-specific Request ?
;; (Vendor-specific Request not supported)
;;        KCMPEQ  %01000000,PDA_VENDOR    ; Vendor-specific Request ?
*
;;        bra     PDA_STALL               ; device stalled
*
*
* --- Echo Device Stalled ------------------------------------------------
PDA_STALL:
        jsr     Echo_STALLED
        bra     PDA_EXIT
*
*
* --- Process Data of Standard Device Requests ---------------------------
PDA_STANDARD:
* (OUT Data stage is only supported in SET_DESC Req.)
        lda     V_bRequest
        KCMPNE  K_SET_DESC,PDA_EXIT     ; ~Data of SET_DESC Request ?
*
;; (SET_DESC Request is NOT supported)
        bra     PDA_STALL
*
*
* --- Process Data of HID Class-specific Requests ------------------------
PDA_CLASS:
        lda     V_bRequest
        KCMPNE  K_SET_REPORT,PDA_EXIT   ; ~Data of SET_REPORT Request ?
        bra     HDA_SET_REPORT          ; process the OUT Data pkt
*
*
* ------------------------------------------------------------------------
PDA_EXIT:
        rts




* [Class Specific Request] -------------------------------------------- *
* HDA_SET_REPORT - process data of SET_REPORT Request                   *
*                  (support only OUT report of I/F 0)                   *
* In      : QD_Rx_Buf           Rx Buffer Queue of Epnt 0               *
* Out     : Q_OUT0_Rpt          OUT Report Queue of I/F 0               *
*                               (updated with new OUT report received)  *
*           B_NORPT0            =1  new OUT report received             *
* Call    : Echo_STALLED                                                *
* Note    :- see also HCR_SET_REPORT                                    *
* --------------------------------------------------------------------- *
HDA_SET_REPORT:
        lda     V_wValue_H             ; Report Type
        KCMPEQ  $01,HDSR_INPUT          ;  =Input ?
        KCMPEQ  $02,HDSR_OUTPUT         ;  =Output ?
		KCMPEQ	$03,HDSR_Feature		;  =Feature ?
*
*
* --- Echo Device Stalled ------------------------------------------------
HDSR_STALL:
        jsr     Echo_STALLED
        jmp     HDSR_EXIT
*
*
* --- Update IN Report ---------------------------------------------------
HDSR_INPUT:
*
        jmp     HDSR_EXIT
*
*
* --- Update OUT Report --------------------------------------------------
HDSR_OUTPUT:
        lda     V_bmReqType
        KCMPEQ  %00100001,HDSR_CHK_IF
        KCMPEQ  %00100010,HDSR_CHK_EPNT
HDSR_CHK_IF:
        lda     V_wIndex_L              ; Interface #
	beq	HDSR_IF0		; = 0 ?
        cbeqa	#1,HDSR_IF1		; = 1 ?
;        beq     HDSR_IF0                ;  = 0 ?
        bra     HDSR_STALL
HDSR_CHK_EPNT:
        lda     V_wIndex_L             ; Endpoint #
        KCMPEQ  1,HDSR_EP1              ;  = 1 ?
        KCMPEQ	2,HDSR_EP2		;  = 2 ?
        bra     HDSR_STALL
*
* --- Update IN/OUT Reports ------------------------------------
HDSR_EP1:
HDSR_IF0:
        ldx     V_Rx_Ptr
        KCPXHI  K_ORPT0_SIZE,HDSR_STALL  ; Report Length > Buffer Size ?
HDSR_IF0_OLOOP:
        lda     (Q_Rx_Buf-1),x         ; copy data from USB Buffer
        sta     (Q_OUT0_Rpt-1),x        ;  to OUT Report Buffer Queue
        DBNZX   HDSR_IF0_OLOOP
*
		jsr		Prog_ICP_Flag		; ICP


        bset    B_NORPT0,V_Dev_Status    ; signal new OUT report received
*
        bra     HDSR_EXIT
*
*
* --- Update IN/OUT Reports ------------------------------------
HDSR_EP2:
HDSR_IF1:
        ldx     V_Rx_Ptr
        KCPXHI  K_ORPT1_SIZE,HDSR_STALL  ; Report Length > Buffer Size ?
HDSR_IF1_OLOOP:
        lda     (Q_Rx_Buf-1),x         ; copy data from USB Buffer
        sta     (Q_Out1_Rpt-1),x        ;  to OUT Report Buffer Queue
        DBNZX   HDSR_IF1_OLOOP
*
        bset    B_NORPT1,V_Dev_Status   ; signal new OUT report received
		bra		HDSR_EXIT
*

HDSR_STALL1:
		bra		HDSR_STALL
	
* --- Update Feature Report ------------------------------------

HDSR_Feature:
        lda     V_bmReqType
        KCMPEQ  %00100001,HDSR_F_CHK_IF
        KCMPEQ  %00100010,HDSR_F_CHK_EPNT
HDSR_F_CHK_IF:
        lda     V_wIndex_L              ; Interface #
		beq		HDSR_F_IF0		; = 0 ?
        cbeqa	#1,HDSR_IF1		; = 1 ?
        bra     HDSR_STALL
HDSR_F_CHK_EPNT:
        lda     V_wIndex_L             ; Endpoint #
        KCMPEQ  1,HDSR_F_EP1              ;  = 1 ?
        KCMPEQ	2,HDSR_F_EP2		;  = 2 ?
        bra     HDSR_STALL
*
* --- Update IN/OUT Feature Reports ------------------------------------
HDSR_F_EP1:
HDSR_F_IF0:
        ldx     V_Rx_Ptr
        KCPXHI  K_Feat0_Size,HDSR_STALL  ; Report Length > Buffer Size ?
HDSR_F_IF0_OLOOP:
        lda     (Q_Rx_Buf-1),x         ; copy data from USB Buffer
        sta     (Q_Feat0_Rpt-1),x        ;  to OUT Report Buffer Queue
        DBNZX   HDSR_F_IF0_OLOOP
*
		jsr		Prog_ICP_Flag

*
        bra     HDSR_EXIT
*
*
* --- Update IN/OUT Reports ------------------------------------
HDSR_F_EP2:
HDSR_F_IF1:
        ldx     V_Rx_Ptr
        KCPXHI  K_ORPT1_SIZE,HDSR_STALL1  ; Report Length > Buffer Size ?
HDSR_F_IF1_OLOOP:
        lda     (Q_Rx_Buf-1),x         ; copy data from USB Buffer
        sta     (Q_Feat1_Rpt-1),x        ;  to OUT Report Buffer Queue
        DBNZX   HDSR_F_IF1_OLOOP
*

*
*
* ------------------------------------------------------------------------
HDSR_EXIT:
        rts




* ====== End of File =====================================================
            end
