*****************************************************************************
*                                         Copyright (c) Motorola 2002		*
*  File Name:	ICP-KBD.ASM													*
*                        													*
*  Description:	A keyboard demo for JB16 USB ICP							*
*                                                                          	*
*  Assembler:	CodeWarrior													*
*  Version:		2.1								                        	*
*																			*
*  Current Revision:	1.0													*
*  Current Revision Release Date: 2002.10.10 by Derek Lau					*
*																			*
*  Updated History															*
*   Rev		YYYY.MM.DD  Author			Description of Change				*
*	---		----------- ------			---------------------				*
*	0.1		2002.01.03	Derek Lau		original release					*
*	1.0		2002.10.10	Derek Lau		general release						*
*****************************************************************************
* This Program is a freeware to demonstrate the operation of Motorola		*
* Microcontroller. Motorola reserves the right to make changes without		*
* further notice to any product herein to improve reliability, function, or *
* design. Motorola does	not assume any liability arising out of the 		*
* application or use of any product, circuit, or software described herein; *
* neither does it convey any license under its patent rights nor the rights *
* of others. Motorola products are not designed, intended, or authorized 	*
* for use as components	in systems intended for surgical implant into the	*
* body, or other applications intended to support life, or for any other	*
* application in which the failure of the Motorola product could create a	*
* situation where personal injury or death may occur. Should Buyer purchase	*
* or use Motorola products for any such intended or unauthorized 			*
* application, Buyer shall indemnify and hold Motorola and its officers,	*
* employees, subsidiaries, affiliates, and distributors harmless against	*
* all claims, costs, damages, and expenses, and reasonable attorney fees	*
* arising out of, directly or indirectly, any claim of personal injury or	*
* death associated with such unintended or unauthorized use, even if such	*
* claim alleges that Motorola was negligent regarding the design or 		*
* manufacture of the part. Motorola	and the Motorola logo* are registered	*
* trademarks of Motorola Inc												*
*****************************************************************************
* To be Done List:
*
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Notes:
* - abbreviation
*     i/p - input, o/p - output, i/o - input/output, o/d - open-drain o/p
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

;; (If use internal D- pullup SET USB_IPullup)
USB_IPullup	set	0             ; internal USB pullup resistor

 base 10

* ------ Include Files ---------------------------------------------------
 INCLUDE    "JB16-EQS.H"		; JB16 Reg. Equates & Variables


* ------ MACRO Definition ------------------------------------------------
 INCLUDE    "MACRO8.h"			; Macro Definitions for HC08


* ------ Revision History Marker -----------------------------------------

 page
* ------ Image of CONFIG Register ----------------------------------------

K_CONFIG   equ %00000011
*                 
*                  ; disable COP
*                  ; enable STOP inst.
*                  ; COP timeout period - 8k OSC cycle
*                  ; Stop mode recovery - 4096 OSC cycle
*		   ; enable LVI
*		   ; USB Reset to generate reset signal


 page
* ------ New I/O Definition ----------------------------------------------

* --- Port A -------------------------------------------------------------
Port_Row	equ	PTA	; Key Matrix - Scan Row 0-7	[o/d]
b_Btn0		equ 0
b_LSCRL		equ 1           ;  -0 SCRLlock LED on           [o/p]
b_LCAP		equ 2           ;  -0 CAP lock LED on           [o/p]
b_LNUM		equ 3           ;  -0 NUM lock LED on           [o/p]

b_Btn1		equ	4
*
*
* --- Port B -------------------------------------------------------------
* (Key Matrix - Scan Column 0-7)				[o/d]
*
* --- Port C -------------------------------------------------------------
* (Key Matrix - Scan Column 8-16)				[o/d]
*
*
* --- Port D -------------------------------------------------------------
b_MseClk	equ 0           ; P/S 2 Mouse Clock (not used)
b_MseData	equ 1		; P/S 2 Mouse Data (not used)
;b_LSCRL		equ 2           ;  -0 SCRLlock LED on           [o/p]
;b_LCAP		equ 0           ;  -0 CAP lock LED on           [o/p]
;b_LNUM		equ 1           ;  -0 NUM lock LED on           [o/p]
*
*
* --- Port E -------------------------------------------------------------
b_COL16         equ 0		; Key Matrix - Scan Column 16	[o/d]
b_COL17		equ 2		; Key Matrix - Scan Column 17	[0/d]
b_PS2Clk	equ 3		; PS/2 Clock for Multiprotocol (not used)
b_PS2Data	equ 4		; PS/2 Data for Multiprotocol (not used)


 page
* ------ Variables Definition --------------------------------------------
*
* Monitor rom subroutines

Mon_Verify		equ		$FC03
Mon_Erase		equ		$FC06
Mon_ProgFLASH	equ		$FC09
Mon_USB_ICP		EQU		$FA19			; Monitor USB ICP routines

 PAGE
* ------ Variables Definition --------------------------------------------
*
        org     RAM_BEG+8


* FOLLOWING VARIABLES SET/ACCESSED BY USER
V_Mon_CTRLBYT	ds.b	1
b_Mass			equ		6                               
V_Mon_CPUSPD	ds.b	1                               
V_Mon_LADDR		ds.b	2                               

		org		$100
Q_Mon_ProgBuf	ds.b	2		;ALLOCATION/USE OF THIS SPACE IS 


DEFAULT_RAM		SECTION		SHORT


                                ;DEPENDENT ON THE DEVICE AND APPLICATION
*
* --- [RAM Routine] copy content of VD_Opd1[x] to VD_Opd2 ----------------
* - called in DEV_HANDLER routine
* equivalent to :  lda     <V_Off_LDA_H:L>,x
*                  sta     <V_Off_STA>
*                  rts
Long_IDX        equ     *       ; <for Device Command Handler>
V_CLRH          ds.b      1       ; Opcode of CLRH = $8C
V_LDA           ds.b      1       ; Opcode of LDA(16-bit Idx) = $D6
V_Off_LDA_H		ds.b      1       ; Offset(High byte) LDA
V_Off_LDA_L		ds.b      1       ; Offset(Low byte) LDA
V_STA           ds.b      1       ; Opcode of STA(direct) = $B7
V_Off_STA       ds.b      1       ; Offset STA
V_RTS           ds.b      1       ; Opcode of RTS = $81
*
*
* --- OC/IC Timer --------------------------------------------------------
K_Fcpu          equ     6000              ; CPU clock freq.       [KHz]
K_CPUSPD		equ		(K_Fcpu/200)	  ; CPU speed*4 in MHz	
K_Ftim          equ     (K_Fcpu/4)        ; Timer clock rate      [KHz]
*
* Tmr CH0 periodic interrupt - for USB suspend timeout
;K_Tch0          equ     3                 ; CH0 OCMP period       [ms]
K_Tch0          equ     8                 ; CH0 OCMP period       [ms]
K_Nch0          equ     (K_Tch0*K_Ftim)   ;                       [cnts]
*
* Tmr CH1 periodic interrupt - for keyboard idle timeout, 40ms time tick
K_Tch1          equ     4                 ; CH1 OCMP period       [ms]
K_Nch1          equ     (K_Tch1*K_Ftim)   ;                       [cnts]
*
* (dec. by timer Ch1 interrupt - every 4ms)
VC_IF0_Idle     ds.b      1       ; Counter for I/F 0 Idle Timeout
                                ; (dec. for every 4ms time tick)
VC_IF1R1_Idle   ds.b      1       ; Counter for I/F 1 Idle Timeout
*                               ; (dec. for every 4ms time tick)
VC_IF1R2_Idle   ds.b      1       ; Counter for I/F 1 Idle Timeout
*                               ; (dec. for every 4ms time tick)
VC_IF1R3_Idle   ds.b      1       ; Counter for I/F 1 Idle Timeout
*                               ; (dec. for every 4ms time tick)
*
* (dec. by timer Ch1 interrupt - every 4ms)
KC_40ms         equ     10      ; init. value for VC_40ms
VC_40ms         ds.b      1       ; counter for 40ms time tick
*
* (dec. by timer Ch1 interrupt - every 4ms)
K_EP12_T_OUT	equ	30	; Time out every 120ms
VC_EP_T_OUT	ds      1       ; Counter for Endpoint 1 & 2 Timeout
*
KC_1s           equ     20      ; dec. by 50ms delay loop
VC_1s           ds.b      1       ; counter for 1s delay
*
VS_Timer        ds.b      1       ; Timer Status Byte
b_T4MS          equ 0           ;  -1  4ms time tick
b_T40MS         equ 1           ;  -1  40ms time tick
b_Tsus          equ 2           ;  -1  suspend timeout
*
*
* --- Delay Loops --------------------------------------------------------
* DELAY routines using instruction cycles
K_Td40us    equ     40                      ; Dly_40us loop       [us]
K_Nd40us    equ     (K_Td40us*K_Fcpu/1000)  ;                     [cnts]
*
* DELAY routines using TIM Overflow
* - all timer interrupts should be disabled
* - TIM Ch0 & Ch1 operation are affected
K_N50ms         equ     (50*K_Ftim)     ; timer counts for 50ms [cnt]
K_N5ms          equ     (5*K_Ftim)      ; timer coutns for 5ms  [cnt]
*
K_N12ms         equ     (12*K_Ftim)     ; timer counts for 12ms [cnt]
*
*
* --- USB Descriptor Type ------------------------------------------------
K_DTYPE_DEV     equ     1       ; DEVICE
K_DTYPE_CFG     equ     2       ; CONFIGURATION
K_DTYPE_STG     equ     3       ; STRING
K_DTYPE_IF      equ     4       ; INTERFACE     Ŀ cannot direct accessed
K_DTYPE_EP      equ     5       ; ENDPOINT       by SET_DESC/GET_DESC req.
K_DTYPE_HID     equ     $21     ; HID           Ŀ
K_DTYPE_RPT     equ     $22     ; Report          Class Descriptor Type
K_DTYPE_PHY     equ     $23     ; Physical      
*
*
* --- Data Packet of SETUP Transaction -----------------------------------
Q_Setup          equ     *      ; queue for SETUP packet
V_bmReqType      ds.b      1      ; Characteristic of Request
b_Rcpt0          equ 0          ;   Ŀ Recipient-$00    Device
b_Rcpt1          equ 1          ;              -$01    Interface
b_Rcpt2          equ 2          ;              -$02    Endpoint
b_Rcpt3          equ 3          ;              -$03    Other
b_Rcpt4          equ 4          ;             -$04-31 <reserved>
b_Type0          equ 5          ;   Ŀ Req. Type-0      Standard
b_Type1          equ 6          ;             -1      Class
*                               ;               -2      Vendor
*                               ;               -3      <reserved>
b_Dir            equ 7          ;   Data Xfer Dir-0     host to device
*                               ;                -1     device to host
*
V_bRequest	 	ds.b      1      ; Request Code
K_GET_STATUS	 equ 0          ;   Ŀ Standard Request Codes
K_CLR_FEATURE    equ 1          ;    
K_SET_FEATURE    equ 3          ;    
K_SET_ADDRESS    equ 5          ;    
K_GET_DESC       equ 6          ;    
K_SET_DESC       equ 7          ;    
K_GET_CONFIG     equ 8          ;    
K_SET_CONFIG     equ 9          ;    
K_GET_IF         equ !10        ;    
K_SET_IF         equ !11        ;    
K_SYNC_FRAME     equ !12        ;   
K_GET_REPORT     equ 0          ;   Ŀ HID Class Request Codes
K_GET_IDLE       equ 1          ;    
K_GET_PROTOCOL   equ 3          ;    
K_SET_REPORT     equ 9          ;    
K_SET_IDLE       equ !10        ;    
K_SET_PROTOCOL   equ !11        ;   
*
V_wValue_L      ds.b      1       ; Value Field for the request
V_wValue_H      ds.b      1       ;
*
V_wIndex_L      ds.b      1       ; Index Field for the request
V_wIndex_H      ds.b      1       ;
*
V_wLength_L     ds.b      1       ; no. of bytes in Data Stage
V_wLength_H     ds.b      1       ;
*
*
*
* --- Device Request Handler ---------------------------------------------
* ----------    -----------------------------------
* V_Transact    Status
* ----------    -----------------------------------
* %00000001     device idle
* %00000000     device stalled
* %00000010     SETUP transact first received
* %xx0001x0     IN DATA stage started
* %xx0010x0     IN DATA stage completed
* %xx0100x0     OUT DATA stage started
* %xx1000x0     OUT DATA stage completed
* %01xxxxx0     STATUS stage started
* %10xxxxx0     STATUS stage completed
* ----------    -----------------------------------
* (x - don't care)
V_Transact      ds.b  1           ; Control Transfer Status Reg.
b_IDLE          equ 0           ;   1- device idle
b_SETUP         equ 1           ;   1- SETUP stage completed
b_IN            equ 2           ;   1- IN DATA stage started
b_IN_END        equ 3           ;   1- IN DATA stage completed
b_OUT           equ 4           ;   1- OUT DATA stage started
b_OUT_END       equ 5           ;   1- OUT DATA stage completed
b_ISTAT         equ 6           ;   1- STATUS stage started
b_STAT_END      equ 7           ;   1- STATUS stage completed
*
V_Transact1	ds  1		; Control Transfer Ststus Reg1
b_OSTAT		equ 0

V_PROCESS       ds.b  1           ; Packet Processing Status
b_sup           equ 0           ;   1- processed SETUP packet
b_data          equ 1           ;   1- processed IN/OUT Data
*
V_SEQ_Bit       ds.b  1           ; SEQ bit(for EP0 OUT transact only)
*
V_UDR_Ptr       ds.b  1           ; ptr to UDR queue
V_UDR_Size      ds.b  1           ; size of UDR
*
* (V_Rx_Ptr & V_Tx_Ptr share the same RAM byte)
V_Rx_Ptr                        ; ptr to next Rx data byte
V_Tx_Ptr        ds.b      1       ; ptr to next Tx data byte
*
* (V_Rx_Size & V_Tx_Size share the same RAM byte)
V_Rx_Size                       ; size of Rx data
V_Tx_Size       ds.b      1       ; size of Tx data
*
*
V_Tx_Limit      ds.b      1
*
V_Dev_State     ds.b      1       ; Device State Byte
*  only one bit is set at any time
b_ATTACH        equ 0           ;  =1  attached state (not used)
b_POWER         equ 1           ;  =1  powered state
b_DEFAULT       equ 2           ;  =1  default state
b_ADDRESS       equ 3           ;  =1  addressed state
b_CONFIG        equ 4           ;  =1  configured state
o_ATTACH		equ %00000001
o_POWER			equ %00000010
o_DEFAULT		equ %00000100

*
* --- Device Settings ----------------------------------------------------
V_Dev_Status    ds.b      1       ; Device Status Reg.
b_SPOWER	equ 0		;  -1  self-powered device
b_RWAKEUP	equ 1		;  -1  enable device remote wakeup
b_UPDADDR	equ 2		;  -1  update dev. addr after IN status
b_NIRPT0        equ 3           ;  -1  IN report pending for IF0
B_NORPT0        equ 4           ;  -1  OUT report pending for IF0
B_NIRPT1        equ 5           ;  -1  IN report pending for IF1
B_NORPT1        equ 6           ;  -1  OUT report pending for IF1 (not used)
B_UPDSPND	equ 7		;  -1  pending to bring device to suspend

*
* (After initialized, all devices default to report protocol.)
K_BOOT_PTCL      equ 0          ;   select Boot Protocol
K_RPT_PTCL       equ 1          ;   select Report Protocol
*
V_ConfigID      ds.b      1       ; ID to select a Config. Desc.
*
*

V_LED_PTA       ds.b      1       ; Image of PTD_LED
V_PTE           ds.b      1       ; Image of PTE
*
Q_OUT0_Rpt      equ     *       ; start of OUT Report Queue for I/F #0
* -----------------------------------------------
VS_LED          ds.b      1       ; LED status byte - NumLock, ScrlLock, CapsLock
b_RPTNUM        equ 0           ;  -0 NUM lock LED on               [o/p]
b_RPTCAP		equ 1           ;  -0 CAP lock LED on               [o/p]
b_RPTSCRL       equ 2           ;  -0 LSCRL lock LED on              [o/p]
* -----------------------------------------------
QEnd_Out0_Rpt   equ     *       ; end of OUT Report Queue
*
K_ORPT0_SIZE    equ     (QEnd_Out0_Rpt-Q_OUT0_Rpt)  ; size of Q_OUT0_Rpt
*
* --- [DEV] Keyboard OUT Report(LED Status) ------------------------------
Q_Out1_Rpt      equ     *       ; start of OUT Report Queue for I/F #0
* -----------------------------------------------
VS_LED2         ds.b      1       ; LED status byte - NumLock, ScrlLock, CapsLock
* -----------------------------------------------
QEnd_Out1_Rpt   equ     *       ; end of OUT Report Queue
*
K_ORPT1_SIZE    equ     (QEnd_Out1_Rpt-Q_Out1_Rpt)  ; size of Q_Out1_Rpt
*
VP_KCode        ds.b      1       ; point to next byte in Q_NKey
VP_Krmv         ds.b      1       ; point to remove byte

* --- Function Key Status ------------------------------------------
V_Fn_Status     ds.b      1
b_last_Fn       equ     0
b_Fn_Key        equ     1
K_Fn_Mask       equ     %00000001       ; Mask to extract the Fn_Status

V_HK_Status		ds.b	1	; Hot Key Status
b_Vol_NRpt	 equ 0		; New Volume report	(Report ID = 1)
b_ACPI_NRpt	 equ 1		; New ACPI Key report	(Report ID = 2)
b_HK_NRPT	 equ 2		; New HOT Key report	(Report ID = 3)
*

* --- Scan Matrix Keyboard -----------------------------------------------
K_ROWMAX        equ     8       ; no. of rows
K_COLMAX        equ     18      ; no. of columns
*
VS_Scan         ds.b      1       ; Keyboard Scan Status Byte
b_PKPRESS       equ     0       ;  1- key press detected for prev. scan
b_CKPRESS       equ     1       ;  1- key press detected for curr. scan
b_K_FN_FND      equ     2
b_K_FN_TGGL     equ     3
b_FN_STATE      equ     4
b_KGHOST        equ     5       ;  1- ghost key detected
b_KOVF          equ     6       ;  1- kbd report overflow
*
V_KeyNum        ds.b      1       ; key number
*
Q_PScan         ds.b      K_COLMAX  ; queue to store result of prev. scan
Q_CScan         ds.b      K_COLMAX  ; queue to store result of curr. scan
*
V_ColVal        ds.b      1       ; row values of curr. column
*
VP_Col          ds.b      1       ; ptr to current column
VP_Row          ds.b      1       ; ptr to current row
*
* (if blank key at col i / row j, then j-th bit of Q_ColMsk[i] - 0)
Q_ColMsk        ds      K_COLMAX  ; column masks of blank keys
*
* --- Ghost Key Detect ------------------------------------------
VC_One          ds.b      1       ; bit '1' counter
VP_ChkCol       ds.b      1       ; storage row values of check column
*
V_GkeyRow       ds.b      1       ; ghost keys overlap pattern
V_Gkeymsk       ds.b      1       ; ghost key rows mask

V_3_Zero        ds.b      1       ; Process of tri-zeros
b_SEND_OVR      equ     4       ; 1 - all bytes are sent
b_INH_3Z        equ     5       ; indicate the ON/OFF state
b_3Z_KEY_P      equ     6       ; indicate the '000' key is pressed
b_3Z_KEY        equ     7       ; the Tri-zero key is pressed

*
V_Temp_Reg	ds	1		; Temp. storage
V_Temp_Reg1	ds	1
* ------------------------------------------------------------------------
V_Rpt_Status	ds.b	1
b_EP12_Enable	 equ	 0
b_EP2_Pend	 equ	 1


OTHER_RAM		SECTION
*
* --- USB Buffer Queue for IN/OUT Transacts ------------------------------
* (Q_TxBuf & Q_RxBuf share the same Buffer Queue)
K_BUF_SIZE      equ     $10         ; buffer size - 16 bytes
Q_Rx_Buf                            ; Buffer Queue for Rx data
Q_Tx_Buf        ds.b      K_BUF_SIZE  ; Buffer Queue for Tx data
*
*
*
*

*
*
*
*
*
* --- Keyboard Status -------------------------------------------
V_Kbd_Alt       ds.b      1       ; Alt. Setting for Kbd. I/F Desc.
*
V_IF0_Ptcl      ds.b      1       ; selected protocol of kbd report
V_IF1_Ptcl      ds.b      1
*
*  For keyboard, the recommended default is 125[500ms].)
KD_IF0_Idle     equ     125     ; default value(for V_Kbd_Idle)
V_IF0_Idle      ds.b      1       ; Idle Rate for I/F 0 [unit: 4ms]

KD_IF1_Idle     equ     0       ; default value infinity
V_IF1R1_Idle    ds.b      1       ; Idle Rate for I/F 1 report ID 1 [unit: 4ms]
V_IF1R2_Idle    ds.b      1       ; Idle Rate for I/F 1 report ID 2 [unit: 4ms]
V_IF1R3_Idle    ds.b      1       ; Idle Rate for I/F 1 report ID 3 [unit: 4ms]
*
V_EP1_UCR1      ds.b      1       ; Kbd. copy for UCR1
V_EP2_UCR2      ds.b      1       ; ACPI copy for UCR2
*
* --- [DEV] Keyboard OUT Report(LED Status) ------------------------------

* --- Keyboard Report ------------------------------------------
QC_IN0_Rpt       equ     *       ; start of kbd. Report
* -----------------------------------------------
V_MKey          ds.b      1       ; Modifier Keys Status
b_LCtrl          equ 0          ;  1- <Left Ctrl> key pressed
b_LShft          equ 1          ;  1- <Left Shift> key pressed
b_LAlt           equ 2          ;  1- <Left Alt> key pressed
b_LGUI           equ 3          ;  1- <Left GUI> key pressed
b_RCtrl          equ 4          ;  1- <Right Ctrl> key pressed
b_RShft          equ 5          ;  1- <Right Shift> key pressed
b_RAlt           equ 6          ;  1- <Right Alt> key pressed
b_RGUI           equ 7          ;  1- <Right GUI> key pressed
*
V_RKey          ds.b      1       ; Reserved(for OEM use)
*
K_KNCSIZE       equ     6       ; no. of keycode bytes
Q_NKey          ds.b      K_KNCSIZE ; normal pressed keys
* --------------------------------------------------------------
QEND_KBD_RPT    equ     *       ; end of kbd. report
*
K_IRPT0_SIZE    equ     (QEND_KBD_RPT-QC_IN0_Rpt)  ; kbd. report size
*
* -- Kbd scan buffer for the FN key correction ----------------
*
Q_N_Key		ds.b	K_KNCSIZE	; Q_N_Key[0..5] = current pressed
					;                 key#
Q_N_FN		ds.b	K_KNCSIZE	; Q_N_FN[0..5] = current pressed
					;                 FN combination
Q_P_Key		ds.b	K_KNCSIZE	; Q_P_Key[0..5] = last pressed
					;                 key#
Q_P_FN		ds.b	K_KNCSIZE	; Q_P_FN[0..5] = last pressed
					;                 FN combination

QP_IN0_RPT       equ     *      ; start of prev. kbd. Report
* -----------------------------------------------
VP_MKey          ds.b      1      ; Modifier Keys Status
*
VP_RKey          ds.b      1      ; Reserved(for OEM use)
*
QP_NKey          ds.b      K_KNCSIZE ; normal pressed keys
* -----------------------------------------------


* --------------------------------------------------------------
*   Hot key buffers
* --------------------------------------------------------------
K_N_IF1RPTID	equ	3
K_H_RPT_SIZE    equ     4       ; Hot key report size (without ID)
K_IRPT1_Size    equ     (K_H_RPT_SIZE+1) ; include report ID

K_ACPI_RPT_ID   equ     1       ; ACPI report ID = 1
K_ACPI_RPT_SIZE equ     2       ; ACPI report size (include report ID)

K_MMK_RPT_ID    equ     2       ; Multimedia Key report ID = 2
K_MMK_RPT_SIZE  equ     5       ; MMK report size (include report ID)

V_IF1_Rpt_ID    ds.b      1       ; report ID for interface 1 (EP2)



* --------------------------------------------------------------
H_RPT_BEGIN:
* --------------------------------------------------------------
V_ACPI_Key      ds.b      1       ; ACPI Keys buffer
b_PwrKey_Prs    equ     0       ;  1= <POWER DOWN> key pressed
b_SlpKey_Prs    equ     1       ;  1= <SLEEP> key pressed
b_WakKey_Prs    equ     2       ;  1= <WAKE UP> key pressed


			
*
V_Hot_Key1	ds.b      1       ; Hot Keys buffer 1
V_Hot_Key2	ds.b      1       ; Hot Keys buffer 2
V_Hot_Key3	ds.b      1       ; Hot Keys buffer 3
V_Hot_Key4	ds.b      1       ; Hot Keys buffer 4
*
*
*
; ----------------------------------------------------------------
;
;   Previous Report Buffer
;
V_PreACPI_Rpt	ds.b	1	; Previous ACPI report
V_PreHot_Rpt1   ds      1       ; Previous Hot Key reports
V_PreHot_Rpt2   ds      1
V_PreHot_Rpt3   ds      1
V_PreHot_Rpt4   ds      1

H_RPT_END:

K_H_BUFF_SIZE	equ	(H_RPT_END - H_RPT_BEGIN)
; ----------------------------------------------------------------


Q_Feat0_Rpt      equ     *       ; start of Feature Report Queue for I/F #0
* -----------------------------------------------
VS_Feat0          ds.b      8       
* -----------------------------------------------
QEnd_Feat0_Rpt   equ     *       ; end of Feature Report Queue
*
K_Feat0_Size    equ     (QEnd_Feat0_Rpt-Q_Feat0_Rpt)  ; size of Q_Feat0_Rpt

Q_Feat1_Rpt      equ     *       ; start of Feature Report Queue for I/F #0
* -----------------------------------------------
VS_Feat1          ds.b      8       
* -----------------------------------------------
QEnd_Feat1_Rpt   equ     *       ; end of Feature Report Queue
*
K_Feat1_Size    equ     (QEnd_Feat1_Rpt-Q_Feat1_Rpt)  ; size of Q_Feat0_Rpt
V_Btn_Inp		ds		1


* ------ Reset Initialization --------------------------------------------

			XDEF	_Startup
myCode		SECTION	

_Startup:
App_Reset_Init:

* ICP detection for development use 
		mov		#(1<<b_KBIE0),KBIER		; enable PTA0 pullup
		brn		$						; wait for pullup takes effect
		brn		$
		brset	b_PTA0,PTA,Normal_App	; PTA0 = 1 => Normal Application

		clr		KBIER					; goto UCB ICP
		mov		#K_CONFIG,CONFIG
 IFEQ USB_IPullup
        mov     #(1<<b_PULLEN),UCR3    ; enable USB pullup
 ENDIF						
		
		jmp		Mon_USB_ICP

*---
Normal_App:
        MOV     #K_CONFIG,CONFIG        ; update CONFIG Register
        SEI                             ; mask all interrupts.
*
        LDHX    #(RAM_END+1)            ; reset stack pointer to
        TXS                             ; top of RAM
        CLRH
*
        clr     VS_LED
        mov     #%00001110,V_LED_PTA
        jsr     RIN_RESET
;        jsr     Init_IO
        mov     #(1<<b_POWER),V_Dev_State        ; device in POWER state
        clr     V_Dev_Status                    ; default remote wakeup
                                                ;  disabled spec s.9.4.5
 IFEQ USB_IPullup
        mov     #%00110100,UCR3                 ; enable USB pullup
 ENDIF						;  STALL EP0 IN OUT

 IFNE USB_IPullup
        mov     #%00110000,UCR3                 ; disable USB pullup
 ENDIF						;  STALL EP0 IN OUT

        brclr   b_RSTF,UIR1,Not_USB_Rst         ; ~USB reset ?
        mov     #(1<<b_DEFAULT),V_Dev_State      ; device in DEFAULT state
        bset    b_RSTFR,UIR2                    ; reset USB reset flag

* --- Initialize USB Module -------------------------------------
        jsr     RST_USB_IF              ; init. Epnt 0 of USB I/F
;        bra     USB_RST

Not_USB_Rst:
*
        mov     #%10000000,UADDR


* --- Initialize Long_IDX routine ----------------------------------------
* where 16-bit offset V_Opd_H:L are RAM bytes
        MOV     #$8C,V_CLRH             ; Opcode of CLRH
        MOV     #$D6,V_LDA              ; Opcode of LDA(16-bit Index)
        MOV     #$B7,V_STA              ; Opcode of STA
        MOV     #$81,V_RTS
*
*
*
USB_RST:
* --- Initialize IRQ Interrupt ------------------------------------------
        bset    b_IMASK1,ISCR           ; disable IRQ interrupt
        bra     RIN_Exit
*
*
*
* --- USB Resume Entry Point ---------------------------------------------
RIN_SYS:
        jsr     RIN_RESET
*
*
*
* ---  Enable Interrupt --------------------------------------------------
RIN_Exit:
        cli

*
Wait_USB_Rst:
        lda     V_Dev_State
        and     #(o_POWER|o_DEFAULT)	; prepare to suspend		
        bne     Main_Suspend			; at any time
;	bne	wait_USB_rst			; for testing use
                                         
* --- Device is now not in POWER or DEFAULT stage
*
* --- Enable USB Interrupt Endpoints 1 ---------------------
;        jsr     EP1N2_Init
;        bset    b_TX1E,UCR1             ; enable EP1 interrupt Tx
;        bset    b_TX2E,UCR2             ; enable EP2 interrupt Tx
*
* --- Start Suspend & Idle Timeout Timers ----------------------
        MOV     #%01010000,TSC0         ; CH0: OC mode, disable TCH0 pin,
                                        ;      interrupt enable
;        MOV     #%01010000,TSC1         ; CH1: OC mode, disable TCH1 pin,
                                        ;      interrupt enable
*
        jsr     Ref_TCH0                ; Add 8ms to TCH0H:L
        jsr     Ref_TCH1                ; Add 4ms to TCH1H:L
*
* --- Force 1st Keyboard Scan ----------------------------------
        MOV     #KC_40ms,VC_40ms        ; re-init 40ms timeout counter
*
        MOV     #(1<<b_T40MS),VS_Timer   ; clr T4MS, Tsus flags,
                                        ; set T40MS flag(force 1st key scan)
*
*
*
 page
* ------ Main Loop -------------------------------------------------------
MAIN:

*
*
* --- Handle Keyboard Report to EndPoint 1 -------------------------------
MAIN_EP1:
        brclr   b_NIRPT0,V_Dev_Status,MEnd_IN_EP1   ; ~new kbd report pending ?
        brclr   b_TX1E,UCR1,Send_EP1_Anyway    ; USB Tx buffer full ?
*
M_Chk_TOut1:
        lda     VC_EP_T_OUT             ; Check time out
        bne     MEnd_IN_EP1
*
Send_EP1_Anyway:
        sei
        jsr     EP1_IN_Transact
*
		lda		V_IF0_Idle				; refresh kbd. idle timeout cntr
		sta		VC_IF0_Idle
        bclr    b_NIRPT0,V_Dev_Status   ; No new keyboard report
        cli
*
* ------------------------------------------------------------------------
MEnd_IN_EP1:
*
Main_EP2:



* --- Handle Keyboard Report to EndPoint 2 ---------------------


* --- Handle Keyboard Report for ACPI key ---------------------
M_ACPI:

		brclr	b_ACPI_NRpt,V_HK_Status,MEnd_ACPI ; ACPI report ?
        brclr   b_TX2E,UCR2,Send_EP2_Anyway2     ; EP2 Tx Buff full ?

M_Chk_Tout3:
	lda	VC_EP_T_OUT						; Check if packet waits for 120ms?
;	beq	Send_EP2_Anyway2        			; Send out anyway if time out.
        bne     MEnd_ACPI
*
Send_EP2_Anyway2:
		sei
        lda     #K_ACPI_RPT_ID
        sta		V_IF1_Rpt_ID 				; set the report ID
		jsr     EP2_IN_TRANSACT           	; push ACPI report to Epnt 2
*
		lda		V_IF1R1_Idle				; refresh ACPI idle timeout cntr
		sta		VC_IF1R1_Idle

*
        bclr    b_ACPI_NRpt,V_HK_Status     ; clear new ACPI report flag
        cli
;        bra	MEND_ACPI

* ------------------------------------------------------------------------
MEnd_ACPI:
*
*
* --- Handle Keyboard Report for HOT KEYs -------------------------
M_HOTK:
		brclr	b_HK_NRPT,V_HK_Status,MEnd_HotK
        brclr   b_TX2E,UCR2,Send_EP2_Anyway3     ; EP2 Tx Buff full ?

; -----------------------------------------
M_Chk_TOut4:
		lda	VC_EP_T_OUT		; Check if packet waits for 120ms?
        bne     Send_EP2_Anyway3

Send_EP2_Anyway3:
		sei

		lmov	#K_MMK_RPT_ID,V_IF1_Rpt_ID ; Set the Report ID
		jsr     EP2_IN_TRANSACT          ; push kbd report to Epnt 2
*
		lda		V_IF1R2_Idle
		sta		VC_IF1R2_Idle			; refresh kbd. idle timeout cntr

*
        bclr    b_HK_NRPT,V_HK_Status   ; clear new kbd report flag
        cli

;        bra	MEnd_HotK
*
MEnd_HotK:
*
*
*
*
*
* --- Handle 40ms Time Tick ----------------------------------------------
M_40MS:
        brclr   b_T40MS,VS_Timer,MEND_40MS  ; 40ms time tick detected ?
        bclr    b_T40MS,VS_Timer
*
* --- Scan Key Matrix ------------------------------------------
;        jsr     Scan_MTX                ; scan key matrix
*
* --- Eliminate Ghost Keys -------------------------------------
;        jsr     Ghost_Key               ; detect ghost keys
*
* --- Handle New Key Matrix Scan -------------------------------
;        jsr     Scan_Hdlr               ; handle new key scan result
*
* ------------------------------------------------------------------------
		jsr		Chk_Btn


MEND_40MS:
*
*
* --- Handle Device Suspend/Resume ---------------------------------------
Main_Suspend:
        brset   b_Tsus,VS_Timer,Main_Suspend1   ; suspend timeout ?
        jmp     MSus_Exit                       ; not suspend

Main_Suspend1:
*
* --- Prepare Remote Wakeup ------------------------------------
*
        bset    b_IMASKK,KBSCR         ; disable kbd. interrupt
        bset    b_ACKK,KBSCR           ; clear the keyboard int. flag
*
        brclr   b_RWAKEUP,V_Dev_Status,MSUS_DEVOFF  ; remote wakeup disabled
MSUS_DEVON:
*
        MOV     #%00000100,KBSCR        ; enable kbd. interrupt
*
* --- Bring USB Module to Suspend Mode -------------------------
MSUS_DEVOFF:
        MOV     #%00100010,TSC           ; disable TOF int., stop TIM
        MOV     #%00010000,TSC0          ; disable CH0F int.
        MOV     #%00010000,TSC1          ; disable CH1F int.
*
*
	MOV     #%00000001,POC		; Enable PTA pullup
*
        clr     PTB			; force low Column 0-7
        MOV     #%11111111,DDRB
*
        clr     PTC                     ; force low Column 8-15
        MOV     #%11111111,DDRC
*
        MOV     #%11111111,PTD          ; force high unsed pins
        MOV     #%11111111,DDRD		; turn off LEDs
*
        MOV     #%00011010,PTE          ; force low Column 16,17
        MOV     #%00000111,DDRE         ; force high unsed pins
*
*
*
* --- Bring MCU to STOP mode ------------------------------------
*

;fix1:
;	mov	UCR1,V_EP1_UCR1
;	bclr	b_TX1E,UCR1
;	ldx	#8
;E1_Fill_00:
;	clr	(UE1D0-1),x	
;	dbnzx	E1_Fill_00

        sei
        bset    b_RESUMFR,UIR2          ; reset resume flag
        bset    b_SUSPND,UIR0           ; set USB module to suspend mode
bstop:
        STOP
astop:
*
	bclr    b_SUSPND,UIR0
	clra
        dbnza   $                       ; wait for USB reset

        bset    b_SUSPND,UIR0
*
;fix2:	ldx	#K_IRPT0_SIZE
;E1_Store_Back:
;	lda	(QC_IN0_Rpt-1),x
;	sta	(UE1D0-1),x
;	dbnzx	E1_Store_Back
;	mov	V_EP1_UCR1,UCR1	

        sei
*
Wait_End_Resume:
        nop
        lda     UIR1
        brset   b_RESUMF,UIR1,MSUS_RESUME       ; ~resume signal from HOST?
        nop

        lda     V_Dev_State
        and     #(o_POWER|o_DEFAULT)
		bne	MSUS_RESUME
*
*
*
* --- Handle Remote Wakeup --------------------------------------
MSUS_RWAKEUP:
*
* --- Delay 5ms ---------------------------
* Check idle state for 5ms before sending resume signal to host
* "USB spec v1.1 s.7.1.7.5 Resume: para.3"
*
MSUS_D5MS:
        MOV     #%00000010,TSC          ; TIM clk - CPU clk / 4,
                                        ; TOF interrupt disable
*
        MOV     #(K_N5ms/256),TMODH     ; set TIM overflow interval
        MOV     #(K_N5ms%256),TMODL     ; (must write high byte first)
*
        bset    b_TRST,TSC              ; reset TIM counter
        bclr    b_TOF,TSC               ; clear TOF flag
MSUS_WD5MS:

        brset   b_RESUMF,UIR1,MSUS_RESUME ; ~resume signal from HOST?
        nop
        brclr   b_TOF,TSC,MSUS_WD5MS    ; delay 5ms
*
*
* --- Force RESUME signal to Host PC -----------------
        bclr    b_SUSPND,UIR0		; restore USB module to normal mode
        bset    b_FRESUM,UCR1		; enable force Resume
*
* --- Delay 12ms (force RESUME for 1-15ms) ----------
MSUS_D12MS:
*
        MOV     #(K_N12ms/256),TMODH     ; set TIM overflow interval
        MOV     #(K_N12ms%256),TMODL     ; (must write high byte first)
*
        bset    b_TRST,TSC              ; reset TIM counter
        bclr    b_TOF,TSC               ; clear TOF flag
MSUS_WD12MS:
        nop
        brclr   b_TOF,TSC,MSUS_WD12MS   ; wait next TOF set after 12ms
*
* --- Release RESUME signal to Host PC ---------------
        bclr    b_FRESUM,UCR1		; disable force Resume
*
* --- Bring USB Module out of Resume Mode ----------------------
MSUS_RESUME:
*
* --- Wait First EOP signal after RESUME(K-state) ----
MSUS_RWEOP:
*
        bclr    b_SUSPND,UIR0
        brclr   b_EOPF,UIR1,MSUS_RWEOP  ; wait end of resume signal
        bset    b_RESUMFR,UIR2
        bset    b_EOPFR,UIR2              ; clear EOPF flag

MSUS_JRIN_SYS:
*
* --- Restart Device ---------------------------------
        jmp     RIN_SYS                 ; re-start system hardware
*
* ------------------------------------------------------------------------
MSus_Exit:
*
*
*
 page
* ------ End of Main Loop ------------------------------------------------
MAIN_EXIT:
        jmp     MAIN


Chk_Btn:
*
* --- Check Buttons ------------------------------------------------------
		lda		PTA						; get button status
		jsr		Dly_40us
		eor		PTA
		bne		CB_Exit

		clra		
		brset	b_Btn0,PTA,CB_No_Btn1
		lda		#low(KEY_m)
CB_No_Btn1:
		sta		V_Btn_Inp 
		eor		Q_NKey
		beq		CB_Exit					; no changes

* --- Signal Button Press Found --------------------
CB_Changes:
		lda		V_Btn_Inp
		sta		Q_NKey
        bset    b_NIRPT0,V_Dev_Status		; signal new key pressed
*
* ------------------------------------------------------------------------
CB_Exit:
        rts

                  ;                       [4] clk


 page
* ------ Subroutines -----------------------------------------------------
* Init_IO	    initialize I/O	
* RIN_RESET	    initialize MCU
* RST_USB_IF        initialize USB module
* EP1N2_INIT        initialize interrupt endpoints 1 & 2
* EP1_IN_Transact   prepare for next IN Interrupt Transact at Epnt 1
* EP2_IN_TRANSACT   prepare for next IN Interrupt Transact at Epnt 2
* Ref_TCH0          refresh TIM Ch0 O/P Compare Counter for next OCMP IRQ
* Ref_TCH1          refresh TIM Ch1 O/P Compare Counter for next OCMP IRQ

* --------------------------------------------------------------------- *
*
* --------------------------------------------------------------------- *
* Init_IO - initialize I/O                                              *
* In      : <nil>                                                       *
* Out     : PTA,  PTB,  PTC,  PTD,  PTE                                 *
*	    DDRA, DDRB, DDRC, DDRD. DDRE				*
*	    POC,  KBIER
* Call    : <nil>                                                       *
* --------------------------------------------------------------------- *

Init_IO:
* --- Initialize I/O Ports -----------------------------------------------
        MOV     #%11100111,POC          ; enable Port A/B/C/E pullup,
*                                       ; enable Port D high current sink
*
* (Init. Port A lines)
;        clr     PTA                     ; Row 0-7 release high
;        clr     DDRA
		mov		#%00001110,PTA
		mov		#%00001110,DDRA			; LED high

*
* (Init. Port B lines)
        clr     PTB                     ; Column 0-7 release high
        clr     DDRB
*
* (Init. Port C lines)
        clr	PTC			; Column 8-15 release high
	clr	DDRC
*
* (Init. Port D lines)
        MOV     #%00011111,PTE          ; Column 16 & 17 release high

        MOV     #%11111111,PTD
        MOV	#%11111111,DDRD          	; Force PTD output open
*
;        MOV     #$FF,KBIER              ; enable pullup at port A
*
* (Init. Port E lines)
*
        MOV     #%00011000,PTE           ; set PTE[2:0] as i/p
        MOV     #%00011000,DDRE          ;
*
* ------------------------------------------------------------------------
        rts
*        
*
*
* --------------------------------------------------------------------- *
* RIN_RESET - initialize MCU                                            *
* In      : <nil>                                                       *
* Out     : TSC		        			                *
*	    TM0DH, TM0DL						*
*           KBSCR                             		                *
* Call    : Init_IO                                                     *
*	    Col_Msk_Init
*	    Scan_Init
* --------------------------------------------------------------------- *
RIN_RESET:
        sei
        jsr     Init_IO
*
*
*
* --- Initialize Timer Interface Module ----------------------------------
        MOV     #%00010010,TSC		; TIM clk - CPU clk / 4
                                        ; TOF interrupt disable
                                        ; reset TIM cntr & prescalar
*
        MOV     #$FF,TMODH		; init. TIM counter modulo
        MOV     #$FF,TMODL
*
        clr     TSC1                    ; disable CH1 interrupt
*
* --- Start Suspend & Idle Timeout Timers ----------------------
        MOV     #%01010000,TSC0         ; CH0: OC mode, disable TCH0 pin,
                                        ;      interrupt enable
                                        ;      interrupt enable
*
        jsr     Ref_TCH0                ; Add 8ms to TCH0H:L
*
* --- Initialize Keyboard Interrupt --------------------------------------
        bset    b_IMASKK,KBSCR		; disable keyboard interrupts
*
*
* --- Initialize Keyboard Scan Handler ----------------------------------
        jsr     Col_Msk_Init
        jsr     Scan_Init               ; init. key matrix scan handler
* ------------------------------------------------------------------------
        rts
*
*
*
* --------------------------------------------------------------------- *
* RST_USB_IF - initialize USB module                                    *
* In      : <nil>                                                       *
* Out     : V_Transact          -(1<<b_IDLE) device idle                 *
*           b_SPOWER            -0          bus-powered device          *
*           b_RWAKEUP           -1          enable remote wakeup        *
*           b_UPDADDR           -0          no set device addr pending  *
*           V_ConfigID          -$01        select Config Desc. #1      *
*                                             I/F                       *
* Call    : <nil>                                                       *
* --------------------------------------------------------------------- *
RST_USB_IF:
*
* --- Initialize USB Interface -------------------------------------------

	MOV     #%10000000,UADDR	; restore default addr($00), enable USB
        MOV     #%10000011,UIR0		; enable EOP, TXD0 & RXD0 int.
	MOV	#%11111111,UIR2         ; reset all USB flags

 IFEQ USB_IPullup
	MOV	#%01110100,UCR3		; reset TX1st, enable USB pullup
 ENDIF					;  STALL EP0 IN/OUT

 IFNE USB_IPullup
	MOV	#%01110000,UCR3		; reset TX1st, disable USB pullup
 ENDIF					;  STALL EP0 IN/OUT

        MOV     #(1<<b_RX0E),UCR0	; disable EP0 Tx, enable Rx

*
*
* --- Initialize USB Variables -------------------------------------------
        MOV     #(1<<b_IDLE),V_Transact  ; device idle
        clr     V_PROCESS               ; clear pkt processing status
*
	clr	V_ConfigID
;        MOV     #$01,V_ConfigID         ; Config ID-1
*
        clr     V_Dev_Status            ; bus-powered device,
                                        ;   default device address,
                                        ;   ~Epnt 0 stalled,
                                        ;   wait enumeration
                                        ;   disable remote wakeup
*
* --- Set Default Status for Remote Wakeup -----------
;        lda     CFG_bmAttrib            ; check remote wakeup default
;        and     #%00100000              ; in CONFIG desc.
;        beq     RSL_NO_RWP              ; - remote wakeup disabled ?
;        bset    b_RWAKEUP,V_Dev_Status    ; default to enable
RSL_NO_RWP:
*
* --- Set Keyboard Protocol & Idle Timeout Rate ------
        clra
        sta     V_IF0_Ptcl              ; use boot protocol for kbd
*
		lda		#KD_IF0_Idle
		sta		V_IF0_Idle				; load default kbd idle timeout
        lda     #KD_IF1_Idle
        sta     V_IF1R1_Idle		 ; load default ACPI idle timeout
        sta     V_IF1R2_Idle
        sta     V_IF1R3_Idle
*
* ------------------------------------------------------------------------
RSL_EXIT:
        rts


* [Embedded Device] --------------------------------------------------- *
* KBD_ADDRESS - bring keyboard from CONFIG to ADDRESS state             *
* In      :  <nil>                                                      *
* Out     :  <nil>                                                      *
* Call    :  <nil>                                                      *
* --------------------------------------------------------------------- *
*
Kbd_Address:

        lda     UIR0
        and     #%11000011
        sta     UIR0                    ; disable TXD2,RXD2,TXD1 int
*
        mov     #%10111100,UIR2         ; Reset EOP,TXD2,RXD2,TXD1 Flag
*
        bset    b_EOPIE,UIR0		; enable EOPF int.
*
*
	lda	UCR3
	and	#%11111100		; disable EP1 & EP2
	sta	UCR3
*
        MOV     #(1<<b_ADDRESS),V_Dev_State  ; device in ADDRESS state
*
        lda     V_Dev_Status              ; keep remote wakeup status,
        and     #(1<<b_RWAKEUP)          ; clr other status
        sta     V_Dev_Status
*
        clr     TSC1                    ; disable TIM CH1
;        clr     VS_Timer                ; clr all time events
*
*
* ------------------------------------------------------------------------
Kbd_Adr_Exit:
        rts

* ---------------------------------------------------------------------- *
* KBD_CONFIG - bring keyboard from ADDRESS to CONFIG state               *
* In      :  <nil>                                                       *
* Out     :  <nil>                                                       *
* Call    :                                                              *
* ---------------------------------------------------------------------- *
Kbd_Config:
* --- Initialize USB Interrupt Endpoint 1 & 2 --------------------------
* --- Init. Interrupt Endpoints 1 & 2 ------------------------------------
        clr     UCR1                    ; disable Tx of Epnt 1
	clr	UCR2			; disable Tx & Rx of Epnt 2
*
        lda     UIR0
        ora     #%10010100		; enable TXD1, TXD2 int
        sta     UIR0                    
*
        mov     #%00111100,UIR2         ; Reset TXD2,RXD2,TXD1 Flag
*
*
*
* --- Init. Variables of Keyboard Report ---------------------------------
		lda		V_IF0_Idle
		sta		VC_IF0_Idle				; start Kbd idle timeout cntr
		lda		V_IF1R1_Idle
		sta		VC_IF1R1_Idle			; start ACPI idle timout cntr

		lda		V_IF1R2_Idle
		sta		VC_IF1R2_Idle			; start Hot Key   timout cntr

		lda		V_IF1R3_Idle
		sta		VC_IF1R3_Idle			;  start           timout cntr
		
*
        clr     VP_KCode                ; reset ptr to Q_NKey
*
*
        bclr    b_NIRPT0,V_Dev_Status
		clr		V_HK_Status


		lda		#%10101000
		sta		V_EP1_UCR1   ; UCR1 value for EP1
		lda		#%10100010
		sta		V_EP2_UCR2   ; UCR2 value for EP2

		lda		UCR3
		ora		#%00000011		; enable EP1 & EP2
		sta		UCR3

		mov	#(1<<b_CONFIG),V_Dev_State	; device to CONFIG state

	jsr	Scan_Init
;	jsr	EP1_IN_Transact
;	set	B_NIRPT1,V_Dev_Status

	jsr	Update_LED

        MOV     #%01010000,TSC1         ; CH1: OC mode, disable TCH1 pin,
                                        ;      interrupt enable
	mov	#%00000010,TSC
	mov	#$FF,TMODH
	mov	#$FF,TMODL
	jsr	Ref_TCH1

	mov		KC_40ms,VC_40ms
	lda		V_IF0_Idle
	sta		VC_IF0_Idle
	mov		#(1<<b_T40MS),VS_Timer	; clr TCh1, TSus flags
									; set TCh1 x10 flag (force 1st key scan)
*
* ------------------------------------------------------------------------
KBD_Cfg_Exit:
	rts
*
*
*
* --------------------------------------------------------------------- *
* EP1N2_INIT - initialize Interrupt Endpoints 1 & 2                     *
* In      :  <nil>                                                      *
* Out     :  <nil>                                                      *
* Call    :  EP1_IN_Transact                                            *
* --------------------------------------------------------------------- *
EP1N2_INIT:
*
* --- Init. Interrupt Endpoints 1 & 2 ------------------------------------
        clr     UCR1                    ; disable Tx of Epnt 1
	clr	UCR2			; disable Tx & Rx of Epnt 2
*
        lda     UIR0
        and     #%11000011
        sta     UIR0                    ; disable TXD2,RXD2,TXD1 int
*
        mov     #%10111100,UIR2         ; Reset EOP,TXD2,RXD2,TXD1 Flag
*
        bset    b_EOPIE,UIR0			; enable EOPF int.
*
 IFEQ USB_IPullup
        MOV     #%00000111,UCR3         ; enable EP1,EP2 no STALL
 ENDIF

 IFNE USB_IPullup
        MOV     #%00000011,UCR3         ; enable EP1,EP2 no STALL
 ENDIF

*
* --- Init. Variables of Keyboard Report ---------------------------------
        lda     V_IF0_Idle
        sta		VC_IF0_Idle   ; start Kbd idle timeout cntr
        lda     V_IF1R1_Idle
        sta		VC_IF1R1_Idle ; start ACPI idle timout cntr
        lda     V_IF1R2_Idle
        sta		VC_IF1R3_Idle ; start Hot Key   timout cntr
        lda     V_IF1R3_Idle
        sta		VC_IF1R3_Idle ; start           timout cntr

*
        lda     #%10100000              ; Tx DATA1 PID, I/F for EP1,
        ora     #K_IRPT0_SIZE           ; update report size(keyboard),
        sta     V_EP1_UCR1              ; enable Tx

        clr     VP_KCode                ; reset ptr to Q_NKey
*
        bsr     EP1_IN_Transact         ; send 1st blank kbd report
*
        bclr    b_NIRPT0,V_Dev_Status
*
* ------------------------------------------------------------------------
EP1N2I_EXIT:
        rts




* --------------------------------------------------------------------- *
* EP1_IN_Transact - prepare for next IN Interrupt Transact at Epnt 1    *
* 1) copy Keyboard Report to Epnt 1 Tx Queue, and                       *
* 2) prepare Epnt 1 to Tx data(clr TXD1F flag)                          *
* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . *
* In      :  QC_IN0_Rpt[0..7]    buffer queue for Keyboard Report        *
* Out     :  UD1R0-7            copy QC_IN0_Rpt[0..7]                    *
*            b_TXD1F            -0      init. next Epnt Tx              *
* Call    :  <nil>                                                      *
* --------------------------------------------------------------------- *
EP1_IN_Transact:
*
* --- Copy QC_IN0_Rpt[0..7] to UD1R[0..7] ---------------------------------
        ldhx     #K_IRPT0_SIZE
E1IT_AGN:
        lda     (QC_IN0_Rpt-1),x
        sta     (UE1D0-1),x
        dbnzx   E1IT_AGN
*
* --- Prepare Epnt 1 to Tx Data ------------------------------------------
        lda     V_EP1_UCR1
        eor     #(1<<b_T1SEQ)            ; toggle Tx PID

        sta     V_EP1_UCR1
        sta     UCR1
*
        bset    b_TXD1FR,UIR2		; clr TXD1F flag
        mov     #K_EP12_T_OUT,VC_EP_T_OUT       ; set time out counter
*
* ------------------------------------------------------------------------
E1IT_EXIT:
        rts


* [Embedded Device] --------------------------------------------------- *
* EP2_IN_TRANSACT - prepare for next IN Interrupt Transact at Epnt 2    *
* 1) copy Keyboard Report to Epnt 2 Tx Queue, and                       *
* 2) prepare Epnt 2 to Tx data(clr TXD1F flag)                          *
* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . *
* In      :  ACPI_KEY[0..2]	The ACPI report packet			*
* Out     :  UD1R0-3            copy QC_IN0_Rpt[0..3]                    *
*            b_TXD2F            =0      init. next Epnt Tx              *
* Call    :  <nil>                                                      *
* --------------------------------------------------------------------- *
EP2_IN_TRANSACT:
*
* --- Copy Hot_KEY[0..2] to UD2R[0..7] ---------------------------------
*
        ldx     #K_H_RPT_SIZE
        lda     V_IF1_Rpt_ID
        sta     UE2D0
        cbeqa   #K_ACPI_RPT_ID,E2ACPI_IT_AGN    ; ACPI report ?
        cbeqa   #K_MMK_RPT_ID,E2Hot_IT_Agn      ; Hot Key report ?
        bra     E2IT_Exit                       ; Wrong ID ?

E2ACPI_IT_AGN:
        lmov     V_ACPI_Key,(UE2D1)
        ldx     #K_ACPI_RPT_SIZE
        bra     Send_EP2_IN

E2Hot_IT_Agn:
        lda     (V_Hot_Key1-1),x
        sta     UE2D0,x
        dbnzx   E2Hot_IT_Agn
        ldx     #K_MMK_RPT_SIZE

Send_EP2_IN:
        lda     V_EP2_UCR2
        eor     #(1<<b_T2SEQ)            ; toggle Tx PID

        stx     V_Temp_Reg
        and     #%11110000              ; set transmit size to 0
        ora     V_Temp_Reg              ; get transmit size

        sta     V_EP2_UCR2
        sta     UCR2
*
        bset    b_TXD2FR,UIR2         	; clr TXD2F flag

        mov	#K_EP12_T_OUT,VC_EP_T_OUT	; Set the time out counter
*
* ------------------------------------------------------------------------
E2IT_Exit:
        rts


* [Embedded Device] --------------------------------------------------- *
* Update_LED - update LEDs status from New OUT report                   *
* In      :  VS_LED     LED Status of I/F #0(in OUT report)             *
*	  :  VS_LED2    LED Status of I/F #1(in OUT report)		*
* Out     :  PTC[2:4]   update LED Status to LED o/p at port C          *
* Call    :  <nil>                                                      *
* --------------------------------------------------------------------- *
Update_LED:
        mov     #%00001110,V_LED_PTA

        brclr   b_RPTNUM,VS_LED,NumOff
        bclr    b_LNUM,V_LED_PTA

NumOff: brclr   b_RPTCAP,VS_LED,CapOff
        bclr    b_LCAP,V_LED_PTA

CapOff: brclr   b_RPTSCRL,VS_LED,ScrlOff
        bclr    b_LSCRL,V_LED_PTA

ScrlOff:
        mov     V_LED_PTA,PTA
*
* ------------------------------------------------------------------------
UPLED_EXIT:
        rts
* --------------------------------------------------------------------- *
* Ref_TCH0 - Refresh TIM Ch0 O/P Compare Counter for next OCMP IRQ      *
*            (i.e. TCNTH:TCNTL + K_Nch0 -> TCH0H:TCH0L)                 *
* In      :  TCNTH:TCNTL        TIM Counter Reg.                        *
*            K_Nch0             timer counts for TIM ch0 OCMP period    *
* Out     :  TCH0H:TCH0L        TIM Ch0 Reg.                            *
* Call    :  <nil>                                                      *
* Note    :- TCNTH reg. should be read before TCNTL reg.                *
*          - TCH0H reg. should be written before TCH0L reg.             *
* --------------------------------------------------------------------- *
Ref_TCH0:
;        lda     TCNTH
;        psha                            ; must load high byte before
;        lda     TCNTL                   ;  low byte

        ldhx    TCNTH           ; [4]
        pshh                    ; [2]
        clrh                    ; [1]
        txa                     ; [1]

        add     #(K_Nch0 % 256) ; [2]
        tax                     ; [1]
*
        pula                    ; [1]
        adc     #(K_Nch0 / 256) ; [2]
        sta     TCH0H           ; [2]
*
        stx     TCH0L           ; [2]
        bclr    b_CH0F,TSC0     ; [4] clear CH0F
*
* ------------------------------------------------------------------------
RTCH0_EXIT:
        rts                     ; [4]




* --------------------------------------------------------------------- *
* Ref_TCH1 - Refresh TIM Ch1 O/P Compare Counter for next OCMP IRQ      *
*            (i.e. TCNTH:TCNTL + K_Nch1 -> TCH1H:TCH1L)                 *
* In      :  TCNTH:TCNTL        TIM Counter Reg.                        *
*            K_Nch1             timer counts for TIM ch1 OCMP period    *
* Out     :  TCH1H:TCH1L        TIM Ch1 Reg.                            *
* Call    :  <nil>                                                      *
* Note    :- TCNTH reg. should be read before TCNTL reg.                *
*          - TCH1H reg. should be written before TCH1L reg.             *
* --------------------------------------------------------------------- *
Ref_TCH1:
        lda     TCNTH                   ; must load high byte before
        psha
        lda     TCNTL                   ;  low byte
        add     #(K_Nch1 % 256)
        tax
*
        pula
        adc     #(K_Nch1 / 256)
        sta     TCH1H
*
        stx     TCH1L
        bclr    b_CH1F,TSC1             ; clear CH0F
*
* ------------------------------------------------------------------------
RTCH1_EXIT:
        rts


 page
* ------ Interrupt Service Routines --------------------------------------


* --------------------------------------------------------------------- *
* TCH0_ISR - TIM Ch0 Output Compare ISR                                 *
* In      :  b_CH0F     -1      CH0F interrupt                          *
* Out     :  b_Tsus     -1      USB I/F suspend timeout(3ms)            *
* Call    :  Ref_TCH0                                                   *
* --------------------------------------------------------------------- *
App_T1CH0_ISR
TCH0_ISR:
        brclr   b_CH0F,TSC0,TCH0I_EXIT  ; ~CH0F interrupt ?
*
*
* --- Signal Suspend Timeout ---------------------------------------------
*
        bclr    b_CH0F,TSC0             ; clear CH0F
*
        bset    b_Tsus,VS_Timer         ; signal suspend timeout
*
* ------------------------------------------------------------------------
TCH0I_EXIT:
        rti




* --------------------------------------------------------------------- *
* TCH1_ISR - TIM Ch1 Output Compare ISR                                 *
* In      :  b_CH1F         -1  CH1F interrupt                          *
* Out     :  b_T4MS         -1  4ms time tick                           *
*            b_Nkbd         -1  new kbd. report available               *
* Call    :  Ref_TCH1                                                   *
* --------------------------------------------------------------------- *
App_T1CH1_ISR
TCH1_ISR:
        brclr   b_CH1F,TSC1,TCH1I_EXIT  ; ~CH1F interrupt ?
*
        jsr     Ref_TCH1                ; refresh TIM Ch1 Reg.
*
        bclr    b_CH1F,TSC1             ; clear CH1F
*
        bset    b_T4MS,VS_Timer         ; set 4ms time tick flag
*
*
* --- 40ms Time Tick -----------------------------------------------------
TISR_40MS:
        dec     VC_40ms                 ; dec. every 4ms
        bne     TIEND_40MS
        MOV     #KC_40ms,VC_40ms         ; re-init timeout counter
        bset    b_T40MS,VS_Timer        ; set 40ms time tick flag
TIEND_40MS:
*
*
* --- Timeout Keyboard Report Idle Period --------------------------------
TISR_KIDLE:
        ldx     V_IF0_Idle
        beq     TIEND_KIDLE             ; infinity Idle Rate ?
*
        dec     VC_IF0_Idle
        bne     TIEND_KIDLE             ; ~Kbd. Idle Period Timeout ?
*
        stx     VC_IF0_Idle                ; reload counter
*
        bset    b_NIRPT0,V_Dev_Status
TIEND_KIDLE:
        jsr     Update_LED
*
TISR_IF1_Idle:
        ldx     V_IF1R1_Idle            ; check if idle forever
	beq	Skip_Rp1_Idle
*
        dec     VC_IF1R1_Idle           ; decrease counter
	bne	Skip_Rp1_Idle
*
        stx     VC_IF1R1_Idle           ; reload counter
*
        bset    b_ACPI_NRpt,V_HK_Status ; force new ACPI report

Skip_Rp1_Idle:
	ldx	V_IF1R2_Idle		; check if idle forever?
	beq	SKIP_RP2_IDLE

	dec	VC_IF1R2_Idle		; decrease counter
	bne	SKIP_RP2_IDLE

	stx	VC_IF1R3_Idle		; reload counter

	bset	b_Vol_NRpt,V_HK_Status	; Force new report of ID #2
; ...................................................................
SKIP_RP2_IDLE:
	ldx	V_IF1R3_Idle		; check if idle forever?
	beq	SKIP_RP3_IDLE

	dec	VC_IF1R3_Idle		; decrease counter
	bne	SKIP_RP3_IDLE

	stx	VC_IF1R3_Idle		; reload counter

	bset	b_HK_NRPT,V_HK_Status	; Force new report of ID #3

SKIP_RP3_IDLE:
* ------------------------------------------------------------------------
*
Check_Time_Out:
* ------------------------------------------------------------------------
;	ldx	VC_EP_T_OUT		; Check if EP1&2 time out
;	beq	SKIP_EP12_TOUT

;	dec	VC_EP_T_OUT		; decrease counter

SKIP_EP12_TOUT:
* ------------------------------------------------------------------------
TCH1I_EXIT:
        rti




* --------------------------------------------------------------------- *
* IRQ_ISR - handle IRQ# interrupt                                       *
* In      :  <nil>                                                      *
* Out     :  <nil>                                                      *
* Call    :  <nil>                                                      *
* --------------------------------------------------------------------- *
App_IRQ_ISR:
	bset	b_ACK1,ISCR		; reset IRQF
	bset	b_IMASK1,ISCR           ; disable IRQ# interrupt
*
* ------------------------------------------------------------------------
        rti




* --------------------------------------------------------------------- *
* KBD_ISR - Keyboard Interrupt Service Routine			        *
* In      :  <nil>                                                      *
* Out     :  <nil>                                                      *
* Call    :  <nil>                                                      *
* --------------------------------------------------------------------- *
App_KBD_ISR:
        nop
        bset	b_ACKK,KBSCR	        ; clear the keybaord int. flag
*
* ------------------------------------------------------------------------
        rti
*
* --------------------------------------------------------------------- *
* DMY_ISR - dummy Interrupt Service Routine(with no operation)          *
* In      :  <nil>                                                      *
* Out     :  <nil>                                                      *
* Call    :  <nil>                                                      *
* --------------------------------------------------------------------- *
App_SCI_TX_ISR
App_SCI_RX_ISR
App_SCI_ERR_ISR
App_T2OF_ISR
App_T2CH01_ISR
App_T2CH1_ISR
App_T2CH0_ISR
App_T1OF_ISR
App_T1CH01_ISR
App_SWI_ISR
DMY_ISR:
        nop



* ------------------------------------------------------------------------
        rti


 page
* ------ Include Files(in EPROM Map) -------------------------------------
 INCLUDE	"appvector.h"


myCode1		SECTION
 INCLUDE    "usb-kbd.h"             	; HID Class(keyboard) Descriptors

myCode2		SECTION	
 INCLUDE    "usb-scan.asm"				; Keyboard Scan Handler Routines

myCode3		SECTION	
 INCLUDE    "usb-key.asm"				; map key location to USB key #
 
myCode4		SECTION	
 INCLUDE    "JB16-int.asm"				; USB Interrupt Service Routines

myCode5		SECTION	
 INCLUDE    "hid-kbd.asm"				; USB Device Framework Routines

myCode6		SECTION	
 INCLUDE	"hid-icp.asm"				; HID ICP command


        ORG     JMP_KBD_ISR
		dc.b	$CC
		dc.w	App_KBD_ISR

        ORG		JMP_SCI_TX_ISR
		dc.b	$CC
		dc.w	App_SCI_TX_ISR

        ORG		JMP_SCI_RX_ISR
		dc.b	$CC
		dc.w	App_SCI_RX_ISR

        ORG		JMP_SCI_ERR_ISR
		dc.b	$CC
		dc.w	App_SCI_ERR_ISR

        ORG		JMP_T2OF_ISR
		dc.b	$CC
		dc.w	App_T2OF_ISR

        ORG		JMP_T2CH01_ISR
		dc.b	$CC
		dc.w	App_T2CH01_ISR

        ORG		JMP_T2CH1_ISR
		dc.b	$CC
		dc.w	App_T2CH1_ISR

        ORG		JMP_T2CH0_ISR
		dc.b	$CC
		dc.w	App_T2CH0_ISR

        ORG		JMP_T1OF_ISR
		dc.b	$CC
		dc.w	App_T1OF_ISR

        ORG		JMP_T1CH01_ISR
		dc.b	$CC
		dc.w	App_T1CH01_ISR

        ORG		JMP_T1CH1_ISR
		dc.b	$CC
		dc.w	App_T1CH1_ISR

        ORG		JMP_T1CH0_ISR
		dc.b	$CC
		dc.w	App_T1CH0_ISR

        ORG		JMP_IRQ_ISR
		dc.b	$CC
		dc.w	App_IRQ_ISR

        ORG		JMP_USB_ISR
		dc.b	$CC
		dc.w	App_USB_ISR

        ORG		JMP_SWI_ISR
		dc.b	$CC
      	dc.w	App_SWI_ISR

        ORG		JMP_Reset_Init
		dc.b	$CC
		dc.w	App_Reset_Init

        dc.w	$0995					;;ICP_Flag (Checksum)
        

* ------ End of File -----------------------------------------------------
            end
