*****************************************************************************
*                                         Copyright (c) Motorola 2002		*
*  File Name:	JB16-INT.ASM												*
*                        													*
*  Description:	USB Interrupt Service Routines and							*
*				Routines for USB Protocol Layer								*
*                                                                          	*
*  Assembler:	CodeWarrior													*
*  Version:		2.1								                        	*
*																			*
*  Current Revision:	1.0													*
*  Current Revision Release Date: 2002.10.10 by Derek Lau					*
*																			*
*  Updated History															*
*   Rev		YYYY.MM.DD  Author			Description of Change				*
*	---		----------- ------			---------------------				*
*	0.1		2002.01.03	Derek Lau		copy from jb8-int version 0.1		*
*****************************************************************************
* This Program is a freeware to demonstrate the operation of Motorola		*
* Microcontroller. Motorola reserves the right to make changes without		*
* further notice to any product herein to improve reliability, function, or *
* design. Motorola does	not assume any liability arising out of the 		*
* application or use of any product, circuit, or software described herein; *
* neither does it convey any license under its patent rights nor the rights *
* of others. Motorola products are not designed, intended, or authorized 	*
* for use as components	in systems intended for surgical implant into the	*
* body, or other applications intended to support life, or for any other	*
* application in which the failure of the Motorola product could create a	*
* situation where personal injury or death may occur. Should Buyer purchase	*
* or use Motorola products for any such intended or unauthorized 			*
* application, Buyer shall indemnify and hold Motorola and its officers,	*
* employees, subsidiaries, affiliates, and distributors harmless against	*
* all claims, costs, damages, and expenses, and reasonable attorney fees	*
* arising out of, directly or indirectly, any claim of personal injury or	*
* death associated with such unintended or unauthorized use, even if such	*
* claim alleges that Motorola was negligent regarding the design or 		*
* manufacture of the part. Motorola	and the Motorola logo* are registered	*
* trademarks of Motorola Inc												*
*****************************************************************************
* Notes:
* - Control Transfer have 3 transaction stages:-
*   1> Setup Stage  - SETUP token + Data packet + ACK packet
*                   - only 1 8-byte Data packet(with PID DATA0) from host
*                   - SETUP transactions cannot be stalled nor NAKed,
*                     and receiving function must accept the Data packet
*   2> Data Stage   - 0 IN/OUT transacts with format
*                       IN/OUT token + Data packet + Handshake packet
*                   - Data packet always start with PID DATA1, and then
*                     successive Data packet toggled between PID DATA0/DATA1
*   3> Status Stage - if IN transact for Data stage,
*                     then OUT transact for Status stage; and vice versa
*                     (Note:- For no Data stage, Status stage is IN transact.)
*                   - always 0-byte Data packet with PID DATA1
* 
* 
* - accept OUT status stage to abort IN transaction
* - on Rxd OUT_STATUS transact, b_TX0E is cleared to block IN transact
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *


* ====== Compile Options =================================================
;
 BASE 10




* ====== MACRO Definition ================================================
;; INCLUDE    "macro8.asm"




 page
* ====== USB Interrupt Service Routines ==================================
* USB_ISR           serve USB interrupts
* TRAN_SETUP        handle SETUP transact
* TRAN_RX           handle OUT transact, or OUT Status stage
* TRAN_TX           handle IN transact, or IN Status stage
* Echo_STALLED      signal Device STALLED
* ECHO_IN_STATUS    response to Control Transfer with IN STATUS stage
* Echo_OUT_Status   response to Control Transfer with OUT STATUS stage


* --------------------------------------------------------------------- *
* USB_ISR - detect USB interrupts                   jmp to              *
*           -----------------------------------     ------------------  *
*           1) Rx Buffer Full[Epnt 0], and                              *
*                P0SUP(USR0) = 1                    TRAN_SETUP routine  *
*                            = 0                    TRAN_RX routine     *
*           2) Tx Buffer Empty[Epnt 0]              TRAN_TX routine     *
* In      :  TXD0F(UIR1)    =1  Tx Buffer Empty Int.[Epnt 0]            *
*            RXD0F(UIR1)    =1  Rx Buffer Full Int.[Epnt 0]             *
*            RXSETUP(USR0)  =1  SETUP transact,  =0  OUT transact[Epnt0]*
*            EOPF(UIR1)     =1  EOP packet Rxd Int.                     *
*            RESUMF(UIR1)   =1  Resume signal from Host                 *
*            (All above flags are controlled by h/w.)                   *
* Out     :  <nil>                                                      *
* Call    :  TRAN_RX, TRAN_TX, TRAN_SETUP, TRAN_INT, USB_Handler        *
* Note    :- USB int. flags are reset before exit USB_ISR               *
*                                                                       *
* --------------------------------------------------------------------- *
App_USB_ISR:
USB_ISR:

	brclr	b_TXSTL,USR1,UISR_EOP	; STALL transmitted ?
UISR_STALL:
	jsr	Echo_STALLED		; STALL both IN and OUT tokens


* --- Check Periodic EOP from Host ---------------------------------------
* (periodic EOP prevent device go suspend mode)
UISR_EOP:
*
        brclr   b_EOPF,UIR1,UISR_EOP_END ; [5] ~EOP detected ?
        bset    b_EOPFR,UIR2		; [4] clear EOPF flag
*
        bclr    b_Tsus,VS_Timer         ; [4] clear suspend timeout flag
	jsr	Ref_TCH0		; [4] refresh suspend timeout cntr
UISR_EOP_END:
*
*
*
* --- Check Rx Buffer Full Interrupt[Epnt 0] -----------------------------
* - after (Rxd SETUP/OUT token + Rxd data packet + Txd ACK handshake)
*   without Electrical & CRC error, b_RBFF is set
* - SETUP transact cannot be stalled nor NAKed, and
*   the receiving function must accept the SETUP packet
*   (Therefore, b_RXD0F is always cleared to accept all SETUP/OUT
*    transacts.)
UISR_RX0:
        brclr   b_RXD0F,UIR1,UISR_RX0_END  ; [5] ~error-free SETUP/OUT transact ?
*
* --- Check SETUP or OUT Transaction ---------------------------
        brset   b_RXSETUP,USR0,UISR_SETUP0 ; [5] SETUP transact ?
UISR_OUT0:
        jsr     TRAN_RX                 ; serve OUT transact
        bset    b_RXD0FR,UIR2		; ready for next OUT/SETUP
        bra     UISR_CHDLR
UISR_SETUP0:
        bsr     TRAN_SETUP              ; [3] serve SETUP transact
        bset    b_RXD0FR,UIR2		; [4] ready for next OUT/SETUP
        bra     UISR_CHDLR
UISR_RX0_END:
*
*
*
* --- Check Tx Buffer Empty Interrupt[Epnt 0] ----------------------------
* - after (Rxd IN token + Txd data packet + Rxd ACK handshake),
*   b_TXD0F is set
UISR_TX0:
        brclr   b_TXD0F,UIR1,UISR_TX0_END  ; ~error-free IN transact ?
*
        jsr     TRAN_TX			; serve IN transact
        bset    b_TXD0FR,UIR2		; clear TXD0F flag
        bra     UISR_CHDLR
UISR_TX0_END:
*
*
* --- Check Tx Buffer Empty Interrupt[Epnt 1] ----------------------------
* - after (Rxd IN token + Txd data packet + Rxd ACK handshake),
*   b_TXD1F is set
UISR_TX1:
        brclr   b_TXD1F,UIR1,UISR_TX1_END  ; ~error-free IN transact ?
*
        bclr    b_TX1E,UCR1             ; diable EP1 Tx
        bset    b_TXD1FR,UIR2           ; clear TXD1F flag
        bra     UISR_EXIT
UISR_TX1_END:
*
*
* --- Check Tx Buffer Empty Interrupt[Epnt 2] ----------------------------
* - after (Rxd IN token + Txd data packet + Rxd ACK handshake),
*   b_TXD2F is set
UISR_TX2:
        brclr   b_TXD2F,UIR1,UISR_TX2_END  ; ~error-free IN transact ?
        bclr    b_TX2E,UCR2             ; diable EP1 Tx
        bset    b_TXD2FR,UIR2           ; clear TXD1F flag
        bra     UISR_EXIT
UISR_TX2_END:
*
*
*
* --- Check Resume Signal from Host --------------------------------------
UISR_RESUME:
*
        brclr   b_RESUMF,UIR1,UISR_EXIT	; ~resume signal from host ?
        bset    b_RESUMFR,UIR2		; clear b_RSME flag
        bra     UISR_EXIT
*
*
*
* --- Handle USB Device Requests at Epnt 0 -------------------------------
UISR_CHDLR:
* [Since RBFF interrupt is always enabled(to Rx next SETUP packet),
*  USB_Handler routine is moved inside this interrupt routine
*  to prevent new Rxd SETUP/OUT packets corrupted processing data]
        jsr     USB_Handler             ; process EP0 transacts

        brclr   b_ISTAT,V_Transact,UISR_EXIT       
        bset    b_TXD0FR,UIR2		; clear TXD0 flag
        MOV     #%10110000,UCR0         ; 0-byte Data packet, PID DATA1,
                                        ; enable IN transact

;;        bra     UISR_EXIT
*
* ------------------------------------------------------------------------
UISR_EXIT:
        rti
*
*
*
* --------------------------------------------------------------------- *
* TRAN_SETUP - Handle SETUP transact                                    *
* In      : b_R0SEQ(USR0)   =0      PID=DATA0                           *
*           b_RP0SZ[3..0]   =8      8-byte SETUP packet                 *
* Out     : Q_Setup[0..7]   =UD0R[0..7]  copy the Rxd Data Packet       *
*           1 -> b_SETUP            SETUP transact completed            *
*           0 -> b_IDLE             device not idle nor stalled         *
*           for IN Data stage,                                          *
*             0 -> b_IN             wait device prepare for IN Data     *
*             0 -> b_IN_END         wait for IN DATA stage completed    *
*             0 -> b_OUT            <not OUT transact>                  *
*             0 -> b_OUT_END        <not OUT transact>                  *
*           for OUT Data stage,                                         *
*             0 -> b_IN             <not IN transact>                   *
*             0 -> b_IN_END         <not IN transact>                   *
*             1 -> b_OUT            OUT DATA stage started              *
*             b_OUT_END     =0      wait for OUT DATA stage completed   *
*                           =1      OUT DATA stage completed            *
*                                   (zero byte DATA stage)              *
* Call    : Echo_STALLED, ECHO_IN_STATUS                                *
* Note    :- for OUT DATA stage, device is initialized to Rx data       *
*          - when b_SETUP first set; all other bits in V_Transact are   *
*            cleared                                                    *
* --------------------------------------------------------------------- *
TRAN_SETUP:
*
* --- Check SETUP Data Packet Format -------------------------------------
        lda     USR0            ; [3]
        and     #%10001111      ; [2]
        cmp     #%00001000      ; [2]   ; Seq=0, Receive Data=8 ?
        bne     TSP_INVALID     ; [3]   ; ~8-byte SETUP data & DATA0 PID ?
*
* --- Valid SETUP Data Packet Received -----------------------------------
TSP_NORMAL:
	bclr	b_ISTALL0,UCR3		; unSTALL EP0 IN
	bclr	b_OSTALL0,UCR3		; unSTALL EP0 OUT
        ldx     #8              ; [2] + 8[11] = [90]
TSP_LOOP:
        lda     (UE0D0-1),x     ; [4]
        sta     (Q_Setup-1),x   ; [4]
        dbnzx   TSP_LOOP        ; [3]   ; Copy UD0R[0..7] to Q_Setup[0..7]
*
        MOV     #(1<<b_SETUP),V_Transact ; [4] SETUP stage completed
	clr	V_Transact1		; [3]
        clr     V_PROCESS               ; [3] clear processing status
        bclr    b_UPDADDR,V_Dev_Status  ; [4] clear pending addr update
*
;	lda	V_wLength_L		; [2]
;	beq	TSP_IN_STATUS		; [3] No Data Stage ?

* --- Prepare for DATA Stage ---------------------------------------------
        brclr   b_Dir,V_bmReqType,TSP_OUT       ; [5] OUT Data stage ?
*
* --- IN Data Stage follows Setup Stage ------------------------
* - IN transact is block temporary, and
*   wait PROC_SETUP routine to prepare for IN Data
TSP_IN:
        bclr    b_TX0E,UCR0     ; [4] signal busy to IN transact
        bclr    b_T0SEQ,UCR0    ; [4] next Tx SEQ to PID DATA1
        bra     TSP_EXIT        ; [3]
*
* --- OUT Data Stage follows Setup Stage -----------------------
TSP_OUT:
        bset    b_OUT,V_Transact        ; [4] OUT DATA stage started
* --- Check Negotiated no. of Rx Bytes -------------
        lda     V_wLength_H             ; [2]
        bne     TSP_STALL               ; [3] (V_wLength_H must be 0)
*
        clr     V_Rx_Ptr                ; [3]
*
        lda     V_wLength_L             ; [2]
        beq     TSP_IN_STATUS           ; [3] 0-byte Data stage ?
        KCMPHI  K_BUF_SIZE,TSP_STALL    ; [6] Rx bytes > buffer size ?
        sta     V_Rx_Size               ; [2] size of Rx data
* --- Prepare to Rx OUT transacts -------------
        MOV     #(1<<b_R0SEQ),V_SEQ_Bit  ; [4] init. SEQ bit to PID DATA1
        bra     TSP_EXIT                ; [3]
*
* --- Invalid SETUP Transaction ------------------------------------------
* - if the SETUP data is corrupted, discard the data & return
*   no handshake
TSP_INVALID:

        bra     TSP_EXIT
*
* --- Echo Device Stalled ------------------------------------------------
TSP_STALL:
        jsr     Echo_STALLED
        bra     TSP_EXIT
*
* --- Prepare for Status stage -------------------------------------------
* - the Status transact returns "success"
*   when the endpoint has successfully completed processing
*   the requested operation
TSP_IN_STATUS:
* - Echo IN Status after processed the request
        bclr    b_OUT,V_Transact
        bset    b_OUT_END,V_Transact    ; OUT DATA stage completed
        bset    b_data,V_PROCESS        ; no need to process data
        jsr     ECHO_IN_STATUS          ; response with IN Status stage
*
* ------------------------------------------------------------------------
TSP_EXIT:
        rts                             ; [4]
*
*
*
* --------------------------------------------------------------------- *
* TRAN_RX - Handle OUT transact and OUT Status stage                    *
* In      : b_OUT           =1      wait for OUT DATA transacts         *
* Out     : b_STAT_END      =1 Ŀ   if OUT STATUS stage completed       *
*           b_OSTAT         =0                                        *
*           b_OUT_END       =1 Ŀ   if OUT DATA stage completed         *
*           b_OUT           =0                                        *
*           Q_Rx_Buf        contain data bytes of OUT Data stage        *
*           V_Rx_Ptr        point to next byte to be Rxd                *
* Call    : Echo_STALLED                                                *
* --------------------------------------------------------------------- *
TRAN_RX:
*
* --- Check for Data Packet or Status stage ------------------------------
        lda     USR0
        and     #%00001111
        sta     V_UDR_Size              ; store OUT transact data size
*
        beq     TRAN_OUT_STATUS         ; 0-byte OUT Status transact ?
        KCMPLS  8,TRAN_OUT              ; <= 8-bytes OUT data transact ?
        bra     TRX_EXIT                ; neglect the OUT transact
*
*
*
* --- Serve OUT Status stage ---------------------------------------------
* - host returns the OUT Status transact IFF
*   1) it has successfully completed processing the IN data; or
*   2) abort current IN transaction
TRAN_OUT_STATUS:
* --- Check Data Packet Format ---------------------------------
        brclr   b_R0SEQ,USR0,TRX_EXIT   ; ~Rx PID DATA1 ?
*
* --- Detected Valid OUT Status --------------------------------
        bclr    b_TX0E,UCR0             ; block further IN transact
*
        bclr    b_OSTAT,V_Transact1
        bset    b_STAT_END,V_Transact   ; OUT STATUS stage completed
;        bset	b_ISTALL0,UCR3		; SATLL IN EP0
	bset	b_OSTALL0,UCR3		; STALL OUT EP0
        bra     TRX_EXIT
*
*
*
* --- Serve OUT Transact -------------------------------------------------
TRAN_OUT:
*
* --- Check OUT Data Packet Format -----------------------------
* - neglect the OUT data transact IF
*   1) wrong PID, or
*   2) packet size > 8 bytes, or
*   3) not waiting for OUT data transact
        brclr   b_OUT,V_Transact,TRX_EXIT  ; ~wait for OUT DATA transact ?
*
        lda     USR0
        and     #(1<<b_R0SEQ)
        CMPNE   V_SEQ_Bit,TRX_EXIT      ; PID(DATA0/1) != SEQ bit ?

        TOGGLE  b_R0SEQ,V_SEQ_Bit       ; toggle SEQ bit

* --- Check Rxd Data Size ------------------------------------------------
*
        lda     V_Rx_Size
        sub     V_Rx_Ptr                ; negotiated data size
        cmp     V_UDR_Size
        blo     TRX_STALL               ; < Rxd data size ?
        beq     TRX_COMPLETE            ; = Rxd data size ?

        lda     V_UDR_Size
        KCMPLO  8,TRX_STALL             ; Rx data end abnormally ?

        bra     TRX_PULL_BYTES

* --- Signal OUT Data Stage Completed ------------------------------------
TRX_COMPLETE:
;; Echo IN Status after processed the request
        bclr    b_OUT,V_Transact
        bset    b_OUT_END,V_Transact

* --- Copy Rx bytes from UDR0[0..7] to Rx Data Queue ---------------------
TRX_PULL_BYTES:
        clr     V_UDR_Ptr
        ldx     V_UDR_Ptr
TRX_LOOP:
        lda     UE0D0,x                 ; UD0R[V_UDR_Ptr]
                                        ; -> Q_Rx_Buf_Rx_Ptr
        ldx     V_Rx_Ptr
        sta     Q_Rx_Buf,x

        inc     V_UDR_Ptr               ; next byte
        inc     V_Rx_Ptr

        ldx     V_UDR_Ptr
        cpxlo   V_UDR_Size,TRX_LOOP
*
*
* ------------------------------------------------------------------------
TRX_EXIT:
        rts

TRX_STALL:
        jsr     Echo_STALLED
        bra     TRX_EXIT


* --------------------------------------------------------------------- *
* TRAN_TX - Handle IN Data Transact, and                                *
*            . check previous IN transact                               *
*            . prepare for next IN transact                             *
*           Handle IN Status Transact                                   *
*            . check IN Status transact                                 *
*            . serve SET_ADDRESS device request                         *
* In      : b_ISTAT         =1 Ŀ   wait for IN STATUS transact         *
*           b_OUT_END       =1                                        *
*           b_IN            =1      wait for IN Data transacts          *
*           Q_Tx_Buf        contain data to be Txd in IN Data stage     *
*                           (pointed by Long_IDX routine)               *
*           V_Tx_Ptr        point to next byte to be Txd                *
* Out     : b_STAT_END      =1 Ŀ   IN STATUS stage completed           *
*           b_ISTAT         =0                                        *
*           b_IN_END        =1 Ŀ   IN DATA stage completed             *
*           b_IN            =0                                        *
* Call    : Echo_STALLED, DEV_SET_ADDRESS                               *
* --------------------------------------------------------------------- *
TRAN_TX:
        bclr    b_TX0E,UCR0             ; signal busy to IN transact
*
* --- Check for Data Packet or Status stage ------------------------------
        brset   b_ISTAT,V_Transact,TRAN_IN_STATUS
                                            ; wait for STATUS transact ?
        brset   b_IN,V_Transact,TRAN_IN     ; wait for IN DATA transacts ?

	brclr	b_IN_END,V_Transact,TTX_NISTALL
;	bset	b_ISTALL0,UCR3
TTX_NISTALL:
        jmp     TTX_EXIT                ; ~negotiated IN transacts

TTX_STALL:
        jsr     Echo_STALLED
        jmp     TTX_EXIT
*
*
*
* --- Serve IN Status stage ----------------------------------------------
TRAN_IN_STATUS:
*
        bclr    b_ISTAT,V_Transact
        bset    b_STAT_END,V_Transact   ; IN STATUS stage completed
;	bset	b_ISTALL0,UCR3		; STALL IN EP0
	bset	b_OSTALL0,UCR3		; SATLL OUT EP0
*
* --- Address Enumeration --------------------------------------
* - device does NOT change its device address until
*   after the status stage of SET_ADDRESS request
*   completed successfully
TIS_ADDR:
        brclr   b_UPDADDR,V_Dev_Status,TTX_EXIT  ; ~pending SET_ADDR request ?
        bclr    b_UPDADDR,V_Dev_Status
*
        brclr   b_MSB,V_wValue_L,Valid_Address ; ~address <= $7F
        bra     TTX_STALL               ; STALL if address >= $80

Valid_Address:
        lda     V_wValue_L
        ora     #(1<<b_USBEN)            ; retain USB I/F enabled
        sta     UADDR                   ; update device addr
*
        lda     V_wValue_L
        beq     TIS_DEF_STATE           ; assigned default address ?

TIS_ADR_STATE:
        mov     #(1<<b_ADDRESS),V_Dev_State ; device to ADDRESS state
;        bset    b_Emul,V_Dev_Status       ; signal address enumerated
        bra     TTX_EXIT

TIS_DEF_STATE:
        mov     #(1<<b_DEFAULT),V_Dev_State ; device to DEFAULT state
        bra     TTX_EXIT

*
*
*
*
* --- Serve IN Transact --------------------------------------------------
TRAN_IN:
* --- Calculate Current Tx Packet Size -------------------------
        lda     V_Tx_Size
        sub     V_Tx_Ptr
        KCMPLO  8,TIN_COMPLETE          ; remaining Tx data < 8 bytes ?
*
        MOV     #8,V_UDR_Size           ; limit Tx packet to 8 bytes
*
        bra     TIN_UPD_PKT
*

* --- Signal IN Data Stage Completed ---------------------------
TIN_COMPLETE:
        sta     V_UDR_Size             ; update Tx packet size
*
        bclr    b_IN,V_Transact
        bset    b_IN_END,V_Transact    ; IN DATA stage completed
* (unable to echo OUT STATUS after processed IN Data, otherwise
*  SETUP transact will also be NAKed)
        jsr     Echo_OUT_Status         ; response with OUT Status stage
*
        tst     V_UDR_Size
        beq     TIN_UPD_UCR0            ; Tx null packet ?
*
* --- Copy bytes from Tx Data Queue to UD0R[0..7] --------------
TIN_UPD_PKT:
        MOV     #UE0D0,V_Off_STA
        ldx     V_Tx_Ptr
*
        txa
        add     V_UDR_Size
        sta     V_Tx_Limit
TIN_UDR_LOOP:
        jsr     Long_IDX              ; VD_Opd1,x --> VD_Opd2
*
        inc     V_Off_STA
        incx
        CPXNE   V_Tx_Limit,TIN_UDR_LOOP
*
        stx     V_Tx_Ptr
*
* --- Tx IN Data Packet ----------------------------------------
TIN_UPD_UCR0:
        lda     UCR0
        and     #(1<<b_T0SEQ)
        eor     #(1<<b_T0SEQ)            ; toggle Tx PID DATA0/1
        ora     V_UDR_Size             ; update Tx data size
        ora     #((1<<b_TX0E)|(1<<b_RX0E))  ; enable Tx & Rx
        sta     UCR0
*
*
* ------------------------------------------------------------------------
TTX_EXIT:
        rts



* --------------------------------------------------------------------- *
* Echo_STALLED - signal Device STALLED                                  *
* In      :  <nil>                                                      *
* Out     :  V_Transact         =$00    device STALLED                  *
*            V_PROCESS          =$FF    disable all processing          *
*            b_STALL0(UCR0)     =1      return STALL handshake          *
* Call    :  <nil>                                                      *
* Note    :- STALLED condition can only be cleared by RESET fm host     *
* --------------------------------------------------------------------- *
Echo_STALLED:
        bset    b_OSTALL0,UCR3          ; return OUT STALL handshake

ECHO_ISTALLED:
        bset    b_ISTALL0,UCR3          ; return IN STALL handshake

        clr     V_Transact              ; signal STALLED
        bclr    b_UPDADDR,V_Dev_Status    ; clear "wait addr update" flag
;        MOV    #%00001010,V_Dev_Status    ; bus-powered device,
                                        ;   enable remote wakeup,
                                        ;   device stalled
        MOV     #$FF,V_PROCESS           ; disable all processing
ESD_EXIT:
        rts


* --------------------------------------------------------------------- *
* ECHO_IN_STATUS - response to Control Transfer with IN STATUS stage    *
*                  (return 0-byte Data packet with PID DATA1)           *
* In      : <nil>                                                       *
* Out     : <nil>                                                       *
* Call    : <nil>                                                       *
* --------------------------------------------------------------------- *
ECHO_IN_STATUS:
        bset    b_ISTAT,V_Transact       ; STATUS stage started
* --- Return 0-byte Data Packet(with PID DATA1) --------------------------
;        bset    b_TXD0FR,UIR2		; clear TXD0 flag
;        MOV     #%10110000,UCR0         ; 0-byte Data packet, PID DATA1,
                                        ; enable IN transact
	bclr	b_ISTALL0,UCR3		; unSTALL EP0 IN
EIS_EXIT:
        rts




* --------------------------------------------------------------------- *
* Echo_OUT_Status - response to Control Transfer with OUT STATUS stage  *
*                  (echo ACK to 0-byte Data packet with PID DATA1)      *
* In      : <nil>                                                       *
* Out     : <nil>                                                       *
* Call    : <nil>                                                       *
* --------------------------------------------------------------------- *
Echo_OUT_Status:
        bset    b_OSTAT,V_Transact1     ; STATUS stage started
	bclr	b_OSTALL0,UCR3		; unSTALL EP0 OUT
* --- Echo ACK Handshake to host 0-byte OUT transact --------------------

EOS_EXIT
        rts




* ====== End of File =====================================================
        end
