
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Motorola Semiconductors (H.K.) Ltd.
* PC Media Department - Technical Marketing Section
*
* FileName  :  USB-KBD.H
* Title     :  Define Descriptors for KH12 Embedded Device(Keyboard)
*                Device Class    :  HID
*                Device SubClass :  Boot
* Author    :  Tony Luk
*
* Compiler  :  CodeWarrior
* Version	:  2.1
*	
* Include File : <nil>
*
*              DD/MM/YY     Rev.    Modified comments.
* History   :  19/10/98     0.1     first release
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Note:
* - a USB device has 1 config. descriptors,
*   each config has 1 interfaces,
*   each interface has 1 endpoints
* - for GET_DESC request of Config Desc., it returns the Config Desc.,
*   all I/F Desc., all HID Desc., and all Endpoint Desc.
*   e.g.  Config Descriptor
*          I/F Descriptor(for keyboard)
*           HID Descriptor(for keyboard Boot Device)
*          all Endpoint Descriptors(for keyboard)
*          I/F Descriptor(for mouse)
*           HID Descriptor(for mouse Boot Device)
*          all Endpoint Descriptors(for mouse)
*   -------------------------------------------------------------------
* - Desc. Type[bDescType]       =1      DEVICE
*                               =2      CONFIGURATION
*                               =3      STRING
*                               =4      INTERFACE
*                               =5      ENDPOINT
*   -------------------------------------------------------------------
* - Class Code[bDevClass]       =$00    <not specified>
*                               =$03    HID
*                               =$FF    <vendor specific>
*   -------------------------------------------------------------------
* - Subclass Code[bDevSClass]   =$00    <not specified>
*                               =$01    Boot Interface
*                               =$FF    <vendor specific>
*   -------------------------------------------------------------------
* - Protocol Code[bDevPtcol]    =$00    <not specified>
*                               =$01    keyboard
*                               =$02    mouse
*                               =$FF    <vendor specific>
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *


* ====== Compile Options =================================================
;
 BASE 10





 page
* [HID - Keyboard] ----------------------------------------------------- *
* Device Descriptor                                                      *
* ---------------------------------------------------------------------- *
DEVICE_DESC:
Dev_bLength     dc  (DEVICE_DESC_END-DEVICE_DESC)
                            ; Desc. Size
DEV_bDescType   dc  $01     ; Desc. Type=DEVICE
DEV_bcdUSB      dc  $10     ; USB spec.=R1.10
                dc  $01     ;
DEV_bDevClass   dc  $00     ; Class Code    Ŀ
DEV_bDevSClass  dc  $00     ; SubClass Code Ĵ to be defined in I/F Desc.
DEV_bDevPtcol   dc  $00     ; Protocol Code 
DEV_bMaxPktSiz0 dc  $08     ; Max. Packet Size(EP0)= 8 bytes
DEV_idVendor    dc  $25     ; Vendor ID=0425 (Motorola Semi. HK Ltd.)
                dc  $04     ;
DEV_idProduct   dc  $02     ; Product ID=$0102 (JB8 keyboard isp_key_keyboard)
                dc  $01     ;
DEV_bcdDevice   dc  $30     ; Device Release #=R0.30
                dc  $00
DEV_iMftr       dc  $01     ; Idx to String Desc. for Manufacturer Name
DEV_iProduct    dc  $02     ; Idx to String Desc. for Product Name
DEV_iSerialNo   dc  $00     ; Idx to String Desc. for Device S/N
DEV_bNumConfig  dc  $01     ; No. of possible Config=1 (no alt. config.)
DEVICE_DESC_END:




 page
* [HID - Keyboard] ----------------------------------------------------- *
* Configuration Descriptor                                               *
* - must be followed by groups of I/F, HID & EP descriptors              *
* ---------------------------------------------------------------------- *
CONFIG_DESC:
CFG_bLength     dc  (CONFIG_DESC_END-CONFIG_DESC)
                            ; Desc. Size
CFG_bDescType   dc  $02     ; Desc. Type=CONFIGURATION
CFG_wTotLength  dc  (EPNT2_DESC_END - CONFIG_DESC)
                dc  $00     ; Total Data Length, including Config. Desc.,
                            ;   and (I/F, HID, & EP Desc.) groups
CFG_NumIF       dc  $02     ; No. of assoc. I/F=2 ( 0=std.kbd, 1=PWRkeys )
CFG_bCfgValue   dc  $01     ; Config. ID=1
CFG_iCfg        dc  $02     ; Idx to String Desc. for the Config.
                            ;   =<no assoc. string desc.>
CFG_bmAttrib    dc  %10100000  ; Config. Characteristics
                            ;   b0-4    <always zero>
                            ;   b5      =1  support remote wakeup
                            ;   b6      =1  self powered
                            ;   b7      =1  <reserved>
CFG_MaxPwr      dc  !25     ; Max. power consumption=50[mA]
CONFIG_DESC_END:



; =============================================================================
;
;   Interface #0 --- Standard Keyboard Interface (Boot Interface)
;
; =============================================================================
 page
* [HID - Std. Keyboard] ----------------------------------------------------- *
* Interface Descriptor                                                   *
* - it is always returned as part of a config desc., and cannot be       *
*   directly accessed with a GET_DESC or SET_DESC Request                *
* - if a config have 2 I/F's, the Config Desc. will be followed by 2 I/F *
*   Desc. with IF_bIFNum equal to 0 and 1.                               *
* - if an I/F with 3 alt. settings, there are 3 I/F Desc. with same      *
*   IF_bIFNum but diff. IF_bAltSetg values.                              *
* ---------------------------------------------------------------------- *
IFACE0_DESC:
IF0_bLength    dc  (IFACE0_DESC_END-IFACE0_DESC)
                           ; Desc. Size
IF0_bDescType  dc  $04     ; Desc. Type=INTERFACE
IF0_bIFNum     dc  $00     ; I/F ID=0
IF0_bAltSetg   dc  $00     ; alt. setting ID=0
IF0_bNumEP     dc  $01     ; no. of assoc. endpoints=1  (not include EP0)
IF0_bIFClass   dc  $03     ; Class Code=$03             (HID Class)
IF0_bIFSClass  dc  $01     ; Subclass Code=$01          (Boot I/F)
IF0_bIFPtcol   dc  $01     ; Protocol Code=$01          (keyboard)
IF0_iIF        dc  $00     ; Idx to String Desc. for the I/F
                           ;   =<no assoc. string desc.>
IFACE0_DESC_END:





 page
* [HID - Std. Keyboard] ----------------------------------------------------- *
* HID Descriptor                                                         *
* (see p.21-23, section 6.2.1 of Device Class Def. for HID[V1.0          *
*  draft#4])                                                             *
* ---------------------------------------------------------------------- *
HID0_DESC:
HID0_bLength    dc  (HID0_DESC_END-HID0_DESC)
                              ; Desc. Size
HID0_bDescType  dc  $21       ; Desc. Type=HID Class
HID0_bcdHID     dc  $00       ; Class Spec. Release no.=V1.00
                dc  $01       ;
HID0_bCtyCode   dc  $00       ; H/W Country Code=$00   (for all country)
HID0_bNumDesc   dc  $01       ; no. of HID Class Desc. follows=1
HID0_bCDesType  dc  $22       ; Class Desc. Type=REPORT
HID0_wCDesLgth  dc  (REPORT0_DESC_END-REPORT0_DESC)
                dc  $00       ; Size of Class Desc.
HID0_DESC_END:

HID0wLenght


 page
* [HID - Std. Keyboard] ----------------------------------------------------- *
* Endpoint Descriptor                                                    *
* ---------------------------------------------------------------------- *
EPNT1_DESC:
EP1_bLength    dc  (EPNT1_DESC_END-EPNT1_DESC)
                              ; Desc. Size
EP1_bDescType  dc  $05        ; Desc. Type=ENDPOINT
EP1_bEPAddr    dc  %10000001  ; Epnt Type=IN, Epnt no.=1
                              ;   b3-0     <endpoint #>
                              ;   b6-4     <always zero>
                              ;   b7       =0  OUT     /=1  IN
EP1_bmAttrib   dc  %00000011  ; Transfer Type
                              ;   b1-0     =0  Control
                              ;            =1  Isochronous
                              ;            =2  Bulk
                              ;            =3  Interrupt
EP1_wMaxPktSiz dc  K_IRPT0_SIZE  ; Max. Packet Size [bytes]
               dc  $00        ;
EP1_bInterval  dc  $0A        ; Epnt Polling Interval=10[ms]
EPNT1_DESC_END:




; =========================================================================
;
;   Interface #1 --- HOT Key Interface (Non-Boot device)
;
; =========================================================================
 page
* [HID - HOT Keyboard] ----------------------------------------------------- *
* Interface Descriptor                                                   *
* - it is always returned as part of a config desc., and cannot be       *
*   directly accessed with a GET_DESC or SET_DESC Request                *
* - if a config have 2 I/F's, the Config Desc. will be followed by 2 I/F *
*   Desc. with IF_bIFNum equal to 0 and 1.                               *
* - if an I/F with 3 alt. settings, there are 3 I/F Desc. with same      *
*   IF_bIFNum but diff. IF_bAltSetg values.                              *
* ---------------------------------------------------------------------- *
IFACE1_DESC:
IF1_bLength    dc  (IFACE1_DESC_END-IFACE1_DESC)
                           ; Desc. Size
IF1_bDescType  dc  $04     ; Desc. Type=INTERFACE
IF1_bIFNum     dc  $01     ; I/F ID=1
IF1_bAltSetg   dc  $00     ; alt. setting ID=0
IF1_bNumEP     dc  $01     ; no. of assoc. endpoints=1  (not include EP0)
IF1_bIFClass   dc  $03     ; Class Code=$03             (HID Class)
IF1_bIFSClass  dc  $00     ; Subclass Code=$00		(No Subclass)
IF1_bIFPtcol   dc  $00     ; Protocol Code=$00		(Non-Std. protocol)
IF1_iIF        dc  $00     ; Idx to String Desc. for the I/F
                           ;   =<no assoc. string desc.>
IFACE1_DESC_END:

; -----------------------------------------------------------------------




 page
* [HID - HOT Keyboard] ------------------------------------------------ *
* HID Descriptor                                                         *
* (see p.21-23, section 6.2.1 of Device Class Def. for HID[V1.0          *
*  draft#4])                                                             *
* ---------------------------------------------------------------------- *
HID1_DESC:
HID1_bLength    dc  (HID1_DESC_END-HID1_DESC)
                              ; Desc. Size
HID1_bDescType  dc  $21       ; Desc. Type=HID Class
HID1_bcdHID     dc  $00       ; Class Spec. Release no.=V1.00
                dc  $01       ;
HID1_bCtyCode   dc  $00       ; H/W Country Code=$00   (for all country)
HID1_bNumDesc   dc  $01       ; no. of HID Class Desc. follows=1
HID1_bCDesType  dc  $22       ; Class Desc. Type=REPORT
HID1_wCDesLgth  dc  (REPORT1_DESC_END-REPORT1_DESC)
                dc  $00       ; Size of Class Desc.
HID1_DESC_END:




 page
* [HID - HOT Keyboard] ------------------------------------------------- *
* Endpoint Descriptor                                                    *
* ---------------------------------------------------------------------- *

EPNT2_DESC:
EP2_bLength    dc  (EPNT2_DESC_END-EPNT2_DESC)
                              ; Desc. Size
EP2_bDescType  dc  $05        ; Desc. Type=ENDPOINT
EP2_bEPAddr    dc  %10000010  ; Epnt Type=IN, Epnt no.=2
                              ;   b3-0     <endpoint #>
                              ;   b6-4     <always zero>
                              ;   b7       =0  OUT     /=1  IN
EP2_bmAttrib   dc  %00000011  ; Transfer Type
                              ;   b1-0     =0  Control
                              ;            =1  Isochronous
                              ;            =2  Bulk
                              ;            =3  Interrupt

EP2_wMaxPktSiz dc  K_IRPT1_Size  ; Max. Packet Size [bytes]
               dc  $00        ;
EP2_bInterval  dc  $FF        ; Epnt Polling Interval=255[ms]
EPNT2_DESC_END:



 page
* [HID - Std. Keyboard] ----------------------------------------------------- *
* String Descriptor                                                      *
* ---------------------------------------------------------------------- *
* Index to i-th String Desc.=(P_STGi - STRING_DESC)
STRING_DESC:
STG0_bLength    dc  (P_STG1-STRING_DESC)  ; Desc. Size
STG0_bDescType  dc  $03                 ; Desc. Type=STRING
STG0_bString    dc  $09,$04             ; Lang ID=$0409 (English)
*
P_STG1:
STG1_bLength    dc  (P_STG2-P_STG1)     ; Desc. Size
STG1_bDescType  dc  $03                 ; Desc. Type=STRING
STG1_bString    dc  'M',0,'o',0,'t',0,'o',0  ; <Manufacturer Name>
                dc  'r',0,'o',0,'l',0,'a',0  ; = "Motorola "
*
P_STG2:
STG2_bLength    dc  (STRING_DESC_END-P_STG2)  ; Desc. Size
STG2_bDescType  dc  $03                 ; Desc. Type=STRING
STG2_bString    dc  'M',0,'o',0,'t',0,'o',0  ;
                dc  'r',0,'o',0,'l',0,'a',0  ;
				dc  ' ',0,'I',0,'C',0,'P',0  ; <Product Name>
                dc  ' ',0,'K',0,'E',0,'Y',0,'_',0  ; = "Motorola ICP Key_Keyboard"
                dc  'K',0,'e',0,'y',0,'b',0
                dc  'o',0,'a',0,'r',0,'d',0
*
STRING_DESC_END:


* pointer to string desc.
STG_PTR_TABLE:
        dc.w    STRING_DESC
        dc.w    P_STG1
        dc.w    P_STG2
STG_PTR_TEND:
*
STG_PTR_NUM     equ     ((STG_PTR_TEND-STG_PTR_TABLE)/2)



 page
* ---------------------------------------------------------------------- *
* Report Descriptor                                                      *
* =================                                                      *
* - provides a description of the data provided by every control         *
*   in a device                                                          *
* - made up of "items" that provide info. about the device               *
* - short & long items                                                   *
*   1> short item                                                        *
*      - 1st byte   b1-0    item size(0,1,2,4 bytes)                     *
*                   b3-2    item type(0-main, 1-global, 2-local)         *
*                   b7-4    item tag                                     *
*      - 0-4 optional data bytes(as specified in item size)              *
*   2> long item                                                         *
*      - 1st byte   must be $FE                                          *
*      - 2nd byte   item size                                            *
*      - 3rd byte   item tag                                             *
*      - 0-255 optional data bytes(as specified in item size)            *
*      (Note:- long item tag are NOT specified & reserved for            *
*              future use.)                                              *
* - 3 types of items :  MAIN, GLOBAL and LOCAL                           *
* - LOCAL items                                                          *
*   . describe the data fields defined by the next main item             *
* - for every control, it must include the following items :-            *
*   . Input, Output or Feature tag                                       *
*   . Logical Minimum                                                    *
*   . Logical Maximum                                                    *
*   . Usage             Ŀ Global Items                                  *
*   . Usage Page                                                        *
*   . Report Size                                                       *
*   . Report Count                                                     *
*   (Note:- Main items are preceded by Global items.)                    *
*                                                                        *
*                                                                        *
*                                                                        *
* ---------------------------------------------------------------------- *
* Reference:                                                             *
* (Device Class Def. for HID[V1.0 draft#4])                              *
* p.28, sect 6.2.2.4    - main item tags & data defination               *
* p.29, sect 6.2.2.4    - bit definations of main item data              *
* ---------------------------------------------------------------------- *




 page
* --- Main Item Tags --------------------------------------------------- *
* - 5 types of Main Items                                                *
*   . INPUT item   - describe data format return from controls           *
*   . OUTPUT item  - describe data format send to controls               *
*   . FEATURE item - define config. options for the device               *
*     (Note:- FEATURE items differs from OUTPUT items that they          *
*             are NOT visible to applications.)                          *
*   . COLLECTION item     - grouping of items                            *
*   . END_COLLECTION item - end of a collection of items                 *
*     (Note:- COLLECTION & END_COLLECTION items are classified           *
*             as non-data items.)                                        *
* - bit definitions of Data for INPUT/OUTPUT/FEATURE items               *
*     -------------------------------------------------------------      *
*     Bit | =0                       | = 1                               *
*     -------------------------------------------------------------      *
*     b0  |Data                      |Constant                           *
*         |- the item is data        |- item is adding bits to pad       *
*         |                          |  the report                       *
*     -------------------------------------------------------------      *
*     b1  |Array                     |Variable                           *
*         |- return an index in each |- return status of all             *
*         |  field corresponds to the|  defined fields                   *
*         |  pressed button          |                                   *
*     -------------------------------------------------------------      *
*     b2  |Absolute                  |Relative                           *
*         |- based on fixed origin   |- indicate change of value         *
*     -------------------------------------------------------------      *
*     b3  |No Wrap                   |Wrap                               *
*         |- data NOT roll-over for  |- data roll-over for extreme       *
*         |  extreme values          |  values                           *
*     -------------------------------------------------------------      *
*     b4  |Linear                    |Non-linear                         *
*     -------------------------------------------------------------      *
*     b5  |with Preferred State      |no Preferred State                 *
*     -------------------------------------------------------------      *
*     b6  |no Null Position          |Null State                         *
*         |                          |- report Null state by return      *
*         |                          |  out-of-range values              *
*     -------------------------------------------------------------      *
*     b7  |Non-volatile              |Volatile                           *
*         |                          |- return value can be              *
*         |                          |  changed by host                  *
*     -------------------------------------------------------------      *
*     b8  |Bit Field                 |Buffered Bytes                     *
*     -------------------------------------------------------------      *
*     b9- |Reserved                  |                                   *
*     b31 |                          |                                   *
*     -------------------------------------------------------------      *
*     (Note:- b7 only applies for OUTPUT & FEATURE items.                *
*           - for Constant item(b0=1), no other bits attributes apply.   *
*           - for Array Input item(b1=0), only b0, b1 & b2 attributes    *
*             apply.                                                 )   *
* - data definitions for COLLECTION & END_COLLECTION items               *
*    =$00      Physical Type                                             *
*              - set of items for one geometric position                 *
*                (e.g. x,y,z position of a joystick)                     *
*    =$01      Application Type                                          *
*              - set of items for particular application                 *
*                (e.g. keyboard, or mouse)                               *
*    =$02      Logical Type                                              *
*              - set of items form a composite data structure            *
*                (e.g. byte count & data of a buffer queue)              *
*    =$03-7F   <reserved for future use>                                 *
*    =$80-FF   <vendor defined>                                          *
* ---------------------------------------------------------------------- *
Mtag_Input      equ     $80     ; Ŀ b1-0   no. of data bytes
Mtag_Output     equ     $90     ;            0 - 0 bytes
Mtag_Feature    equ     $B0     ;            1 - 1 bytes
Mtag_Collectn   equ     $A0     ;            2 - 2 bytes
*                               ;           3 - 4 bytes
Mtag_EndCltn    equ     $C0     ;    b1-0   <always zero>




 page
* --- Global Item Tags ------------------------------------------------- *
* - describe the default attributes for all subsequent data fields       *
*   unless overridden by another GLOBAL item                             *
* - Global items                                                         *
*   . Usage Page      select the Usage Page                              *
*                     (the Usage Tag points to one of the 256 usage      *
*                      in the selected Usage Page)                       *
*   . Logical Min.    min. value of return data                          *
*   . Logical Max.    max. value of return data                          *
*   . Physical Min.   min. value of return physical data(with unit)      *
*   . Physical Max.   max. value of return physical data(with unit)      *
*   . Unit Exponent   exponent of unit[in base 10]                       *
*                     (e.g. if data=$03 for unit exponent item,          *
*                           then unit is multipled by 10^3      )        *
*   . Unit            code represent selected unit                       *
*   . Report Size     specify the size of a report field [in bits]       *
*   . Report ID       all items after the Report ID item are included    *
*                     in a report prefixed by a 1-byte ID                *
*   . Report Count    specify the no. of report fields[for variable item]*
*                     or                                                 *
*                     max. no. of controls that may be included in the   *
*                     report[for array item]                             *
*   . Push            copy the global item state table to top of stack   *
*   . Pop             copy the global item state table from top of stack *
* - resolution = (Logical Max - Logical Min) / [(10^unit_exponent) *     *
*                (Physical Max - Physical Min)]                          *
*   (e.g. a 400-dpi mouse                                                *
*          Logical Min(-127)                                           *
*           Logical Max(127)                                            *
*           Physical Min(-3175)                                         *
*           Physical Max(3175)                                          *
*           Unit Exponent(-4)                                           *
*          Unit(inches)                                                *
*         resolution = (127 - (-127)) / ((3175 - (-3175)) * 10^(-4))     *
*                    = 400 count/inch                                )   *
* - data definitions for Unit item                                       *
*    ------------------------------------------------------------------- *
*          |          |                     Value                        *
*    Nibble| Basic    |------------------------------------------------- *
*          | Units    |  =0     |  =1     |  =2     |  =3     |  =4      *
*    ------------------------------------------------------------------- *
*       0  | System   | <none>  | SI      | SI      | English | English  *
*          |          |         | Linear  | Rotation| Linear  | Rotation *
*    ------------------------------------------------------------------- *
*                                                                        *
*    -----------------------------------------------------------------   *
*          |          |                     System                       *
*    Nibble| Basic    |-----------------------------------------------   *
*          | Units    | SI Linear | SI Rotat'n|Eng Linear |Eng Rotat'n   *
*    -----------------------------------------------------------------   *
*       1  | Length   | cm        | radian    | inch      | degree       *
*       2  | Mass     | gram      | gram      | slug      | slug         *
*       3  | Time     | sec       | sec       | sec       | sec          *
*       4  | Temp     | K         | K         | F         | F            *
*       5  | Current  | A         | A         | A         | A            *
*       6  | Luminous | Candela   | Candela   | Candela   | Candela      *
*       7  | <reserve>| <none>    | <none>    | <none>    | <none>       *
*    ------------------------------------------------------------------- *
*    (Note:- For System nibble, value=7 means vendor-defined.            *
*          - More complex units can be derived from the basic units.     *
*            e.g. Joule = [mass(kg)][length(m)][time(s^-2)]              *
*                       = [mass(gx10^3)][length(cmx10^2)][time(s^-2)]    *
*                 Therefore, data for Unit item = $0000E321 or $E321     *
* ---------------------------------------------------------------------- *
* Reference:                                                             *
* (Device Class Def. for HID[V1.0 draft#4])                              *
* p.37, section 6.2.2.7 Unit item data for some common complex units     *
* p.57, appendix A      definations of Usage Page & Tags                 *
* ---------------------------------------------------------------------- *
Gtag_UPage      equ     $04     ; Ŀ b1-0   no. of data bytes
Gtag_LogMin     equ     $14     ;            0 - 0 bytes
Gtag_LogMax     equ     $24     ;            1 - 1 bytes
Gtag_PhyMin     equ     $34     ;            2 - 2 bytes
Gtag_PhyMax     equ     $44     ;            3 - 4 bytes
Gtag_UnitExp    equ     $54     ;  

Gtag_Unit       equ     $64     ;  

Gtag_RptSize    equ     $74     ;  

Gtag_RptID      equ     $84     ;  

Gtag_RptCnt     equ     $94     ;  

Gtag_Push       equ     $A4     ;  

Gtag_Pop        equ     $B4     ; 




 page
* --- Local Item Tags -------------------------------------------------- *
* - define characteristics of controls                                   *
* - all Local items are unsigned integers                                *
* - Local items                                                          *
*   . Usage           represent a suggested usage for the item, array or *
*                     collection                                         *
*   . Usage Min.      the starting usage associated with an array        *
*   . Usage Max.      the ending usage associated with an array          *
*   . Designator Idx  index points to a designator in the Physical Desc. *
*   . Designator Min. index points to the starting designator associated *
*                     with an array                                      *
*   . Designator Max. index points to the ending designator associated   *
*                     with an array                                      *
*   . String Index    string index for a String Descriptor               *
*   . String Min.     specify the starting string index associated with  *
*                     an array                                           *
*   . String Max.     specify the ending string index associated with    *
*                     an array                                           *
*   . Set Delimiter   define the beginning or end of a set of local items*
*                     (item data: $00=close set / $01=open set)          *
* - apply to controls of following Main item                             *
*   (e.g. if 3 Usage tags before an Input item defining 5 controls       *
*         then 1> the 3 Usages are assigned sequentially                 *
*                 to the first 3 controls, and                           *
*              2> the 3rd Usage is assigned to the 4th & 5th controls)   *
* - 1 alt. set of local items may be associated with a control          *
*   by bracketing each set with Set_Delimiter items                      *
*   (Note:- Alt. sets are optional & may NOT be recognised by the OS.)   *
* ---------------------------------------------------------------------- *
* Reference:                                                             *
* (Device Class Def. for HID[V1.0 draft#4])                              *
* p.57, appendix A      definations of Usage Page & Tags                 *
* ---------------------------------------------------------------------- *
Ltag_Usage      equ     $08     ; Ŀ b1-0   no. of data bytes
Ltag_UsgMin     equ     $18     ;            0 - 0 bytes
Ltag_UsgMax     equ     $28     ;            1 - 1 bytes
Ltag_DgntrIdx   equ     $38     ;            2 - 2 bytes
Ltag_DgntrMin   equ     $48     ;            3 - 4 bytes
Ltag_DgntrMax   equ     $58     ;  
Ltag_StrgIdx    equ     $78     ;  
Ltag_StrgMin    equ     $88     ;  
Ltag_StrgMax    equ     $98     ;  
Ltag_SDlimitr   equ     $A8     ; 




 page
* [HID - Std. Keyboard] ----------------------------------------------------- *
* - for Boot Device(keyboard), the first 8 bytes of the keyboard report  *
*   must conform to the Boot Report descriptor                           *
*   in order for the data to be interpreted by the BIOS                  *
* ---------------------------------------------------------------------- *
REPORT0_DESC:
*            Item Tag     D.Cnt Data      Comment
*           ------------- ----- ----      --------------------------------
        dc  (Gtag_UPage   |$01),$01     ; Usage Page(Generic Desktop)
        dc  (Ltag_Usage   |$01),$06     ; Usage(Keyboard)
        dc  (Mtag_Collectn|$01),$01     ; Collection(Application)

* ------------------------------------------------------------------------
*  For 8 modifier keys (Ctrl, Shift, Alt, GUI
* ------------------------------------------------------------------------
        dc  (Gtag_UPage   |$01),$07     ;  Usage Page(Key Codes)
        dc  (Ltag_UsgMin  |$01),$E0     ;   Usage Min(224)
        dc  (Ltag_UsgMax  |$01),$E7     ;   Usage Max(231)
        dc  (Gtag_LogMin  |$01),$00     ;   Logical Min(0)
        dc  (Gtag_LogMax  |$01),$01     ;   Logical Max(1)
        dc  (Gtag_RptSize |$01),$01     ;   Report Size(1)
        dc  (Gtag_RptCnt  |$01),$08     ;   Report Count(8)
        dc  (Mtag_Input   |$01),$02     ;  Input(Data,Variable,Absolute)


* ------------------------------------------------------------------------
* Define the second byte field a reserved byte
* ------------------------------------------------------------------------
        dc  (Gtag_RptSize |$01),$08     ;  Report Size(8)
        dc  (Gtag_RptCnt  |$01),$01     ;   Report Count(1)
        dc  (Mtag_Input   |$01),$01     ;  Input(Constant)

* ------------------------------------------------------------------------
* Define the third byte field for 6 bytes of key code array
* ------------------------------------------------------------------------
        dc  (Gtag_UPage   |$01),$07     ;  Usage Page(Key Codes)
        dc  (Ltag_UsgMin  |$01),$00     ;   Usage Min(0)
        dc  (Ltag_UsgMax  |$02),$FF,$00 ;   Usage Max(255)
        dc  (Gtag_LogMin  |$01),$00     ;   Logical Min(0)
        dc  (Gtag_LogMax  |$02),$FF,$00 ;   Logical Max(255)
        dc  (Gtag_RptSize |$01),$08     ;   Report Size(8)
        dc  (Gtag_RptCnt  |$01),$06     ;   Report Count(6)
        dc  (Mtag_Input   |$01),$00     ;  Input(Data,Array)

* ------------------------------------------------------------------------
* Define the OUT report for LED display on Keyboard
* ------------------------------------------------------------------------
        dc  (Gtag_UPage   |$01),$08     ;  Usage Page(LED)
        dc  (Gtag_LogMax  |$01),$01	;   Logical Max(1)
        dc  (Ltag_UsgMin  |$01),$01     ;   Usage Min(1)
        dc  (Ltag_UsgMax  |$01),$05     ;   Usage Max(5)
        dc  (Gtag_RptSize |$01),$01     ;   Report Size(1)
        dc  (Gtag_RptCnt  |$01),$05     ;   Report Count(5)
        dc  (Mtag_Output  |$01),$02     ;  Output(Data,Variable,Absolute)
        dc  (Gtag_RptSize |$01),$03     ;  Report Size(3)
        dc  (Gtag_RptCnt  |$01),$01     ;   Report Count(1)
        dc  (Mtag_Output  |$01),$01     ;  Output(Constant)

      	dc  (Ltag_Usage   |$01),$01     ; Usage(Byte Count)
      	dc  (Gtag_RptSize |$01),$08     ; Report Size(8)
      	dc  (Gtag_RptCnt  |$01),$08     ; Report Count(8)
      	dc  (Mtag_Feature  |$01),$02     ; Feature(Data,Variable,Absolute)

        dc  (Mtag_EndCltn)              ; End_Collection
REPORT0_DESC_END:


 page
* [HID - HOT Keyboard] ----------------------------------------------------- *
* - for Boot Device(keyboard), the first 8 bytes of the keyboard report  *
*   must conform to the Boot Report descriptor                           *
*   in order for the data to be interpreted by the BIOS                  *
* ---------------------------------------------------------------------- *
REPORT1_DESC:
*            Item Tag     D.Cnt Data      Comment
*           ------------- ----- ----      --------------------------------
        dc  (Gtag_LogMin  |$01),$00     ; Logical Min(0)
        dc  (Gtag_LogMax  |$01),$01     ; Logical Max(1)
        dc  (Gtag_UPage   |$01),$01     ; Usage Page(Generic Desktop)
        dc  (Ltag_Usage   |$01),$80     ; Usage(System Control)
        dc  (Mtag_Collectn|$01),$01     ; Collection(Application)

* ------------------------------------------------------------------------
* Define the first byte of field of Report ID = 1
*  for ACPI power key report. The rest bytes are reserved
* ------------------------------------------------------------------------
        dc  (Gtag_RptID   |$01),K_ACPI_RPT_ID	; Report ID of ACPI keys
        dc  (Ltag_UsgMin  |$01),$81     ;   Usage Min(POWER DOWN)
        dc  (Ltag_UsgMax  |$01),$83     ;   Usage Max(WAKE UP)
        dc  (Gtag_RptSize |$01),$01     ;   Report Size(1)
        dc  (Gtag_RptCnt  |$01),$03     ;   Report Count(3)
        dc  (Mtag_Input   |$01),$02     ;    Input(Variable,Relative,Preferred State)

        dc  (Gtag_RptSize |$01),$05     ;   Report Size(5)
        dc  (Gtag_RptCnt  |$01),$01     ;   Report Count(1)
        dc  (Mtag_Input   |$01),$01     ;    Input(Constant)
        dc  (Mtag_EndCltn)              ; End_Collection

*-----------------------
        dc  (Gtag_UPage   |$01),$0C     ;    Usage Page (Consumer Device)
        dc  (Ltag_Usage   |$01),$01     ; Usage()
        dc  (Mtag_Collectn|$01),$01     ; Collection(Application)
* ========================================================================
*  Define the first byte field of MMK Report ID for Multi-Media Key Report
* ========================================================================
        dc  (Gtag_RptID   |$01),K_MMK_RPT_ID	; Report ID of Multimedia key

* ------------------------------------------------------------------------
* Define the H0 to H7 Hot Key function
* ------------------------------------------------------------------------
        dc  (Gtag_RptSize |$01),$01     ;   Report Size(1)
        dc  (Gtag_RptCnt  |$01),$03     ;   Report Count(3)

        dc  (Ltag_UsgMin  |$01),$B5	;   Usage Min(Scan Next Track)
        dc  (Ltag_UsgMax  |$01),$B7	;   Usage Max(Stop)
        dc  (Mtag_Input   |$01),$06     ;    Input(Variable,Relative,Preferred State)

        dc  (Gtag_RptCnt  |$01),$01     ;   Report Count(1)

        dc  (Ltag_Usage   |$01),$B8	;    Usage(Eject)
        dc  (Mtag_Input   |$01),$06     ;    Input(Variable,Relative,Preferred State)

        dc  (Ltag_Usage   |$01),$CD	;    Usage(Play/Pause)
        dc  (Mtag_Input   |$01),$06     ;    Input(Variable,Relative,Preferred State)

        dc  (Ltag_Usage   |$01),$E2	;    Usage(Volume Mute)
        dc  (Mtag_Input   |$01),$06     ;    Input(Variable,Relative,Preferred State)

        dc  (Gtag_RptCnt  |$01),$02     ;   Report Count(2)

        dc  (Ltag_UsgMin  |$01),$E9     ;   Usage Min(Volume Up)
        dc  (Ltag_UsgMax  |$01),$EA    	;   Usage Max(Volume Down)
        dc  (Mtag_Input   |$01),$06     ;    Input(Variable,Relative,Preferred State)

* ------------------------------------------------------------------------
* Define the H8 to H15 Hot Key function
* ------------------------------------------------------------------------
        dc  (Gtag_RptCnt  |$01),$01     ;   Report Count(1)

        dc  (Ltag_Usage   |$02),$83,$01	;    Usage(Media Select)
        dc  (Mtag_Input   |$01),$06     ;    Input(Variable,Relative,Preferred State)

        dc  (Ltag_Usage   |$02),$8A,$01	;    Usage(Mail)
        dc  (Mtag_Input   |$01),$06     ;    Input(Variable,Relative,Preferred State)

        dc  (Ltag_Usage   |$02),$92,$01	;    Usage(Calculator)
        dc  (Mtag_Input   |$01),$06     ;    Input(Variable,Relative,Preferred State)

        dc  (Ltag_Usage   |$02),$94,$01	;    Usage(My Computer)
        dc  (Mtag_Input   |$01),$06     ;    Input(Variable,Relative,Preferred State)

        dc  (Gtag_RptCnt  |$01),$04     ;   Report Count(4)
        dc  (Mtag_Input   |$01),$01     ;    Input(Constant) - unused keys

* ------------------------------------------------------------------------
* Define the H16 to H23 Hot Key function
* ------------------------------------------------------------------------
        dc  (Gtag_RptCnt  |$01),$07     ;   Report Count(7)

        dc  (Ltag_UsgMin  |$02),$21,$02 ;   Usage Min(WWW Search)
        dc  (Ltag_UsgMax  |$02),$27,$02	;   Usage Max(WWW Refresh)
        dc  (Mtag_Input   |$01),$06     ;    Input(Variable,Relative,Preferred State)

        dc  (Gtag_RptCnt  |$01),$01     ;   Report Count(1)

        dc  (Ltag_Usage   |$02),$2A,$02	;    Usage(WWW Favorite)
        dc  (Mtag_Input   |$01),$06     ;    Input(Variable,Relative,Preferred State)

* ------------------------------------------------------------------------
* The following lines define the H24 to H31 Hot Key function
* ------------------------------------------------------------------------
        dc  (Gtag_RptSize |$01),$01     ;   Report Size(1)
        dc  (Gtag_RptCnt  |$01),$08     ;   Report Count(8)
        dc  (Mtag_Input   |$01),$01     ;    Input(Constant) - unused keys


* =====================================================================================
*  Define the OUT report from Host (For two extra LED display)
* =====================================================================================
*  USAGE:
*
*  This is set by Set_Report( ) request. The Report Type is OUT report. And
*  request is sent thru Control Endpoint(Endpoint 0).
*
*  After the OUT report is received from Host. The data is stored in VS_LED2.
*  In this sample code, the extra LED display is not implemented. If the
*  I/O pin for the LED is determined,  display the VS_LED2 status to I/O pin
*  in the Update_LED subroutine.
*
		dc  (Gtag_UPage   |$01),$08 	; Usage Page(LEDs)
        dc  (Gtag_RptCnt  |$01),$01     ;   Report Count(1)
		dc  (Gtag_RptSize |$01),$01     ;   Report Size(1)

		dc  (Ltag_Usage   |$01),$2A     ;    Usage(On-Line)
		dc  (Mtag_Output  |$01),$02     ;    Output(Variable)

        dc  (Ltag_Usage   |$01),$4B     ;    Usage(Generic Indicator)
		dc  (Mtag_Output  |$01),$02     ;    Output(Variable)

        dc  (Gtag_RptSize |$01),$06     ;   Report Size(6)
        dc  (Gtag_RptCnt  |$01),$01     ;   Report Count(1)
        dc  (Mtag_Output  |$01),$01     ;    Output(Constant)

        dc  (Mtag_EndCltn)              ; End_Collection
; ------------------------------------------------------------------------------------

REPORT1_DESC_END:



* ====== End of File =====================================================
            end
