* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Motorola Semiconductors (H.K.) Ltd.
* PC Media Department - Technical Marketing Section
*
* FileName  :  ALP-KEY-ASM.h
* Title     :  Map Key Matrix of KH12 Keyboard
*              - Key Codes Table (follow USB Keyboard Usage Page)
*              - Key Matrix Map
*              - function to return key code at location (row,column)
* Author    :  Tony Luk
*
* Compiler  :  CodeWarrior
* Version	:  2.1
*
*
*              DD/MM/YY     Rev.    Modified comments.
* History   :  05/06/98     0.1     Created
*              03/08/98     1.0     update keyboard map to 8 row x 18
*                                    column (to match Scan_MTX routine)
*              08/05/00     1.1     support multimedia key
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Disclaimer of All Warranties & Liabilities :-
*  This Program is a freeware to demonstrate the operation of HC08 micro-
*  controller. In no event will Motorola be liable for any damages, or any
*  incidental or consequential damages arising out of the use of or
*  inability to use this program. Motorola does not make any warranties
*  of any kind that the program does not or will not infringe any copyright,
*  patent, trade secret or other intellectual property right of any third
*  party in any country.
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Notes :
* - CASM08 assembler is NOT case sensitive
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; JB8 PCB version 0.1 0.2
KeyType0	set		1
KeyType1	set		1
KeyType2	set		0        ; hotkey keyboard matrix




* ====== Table of Key Codes ==============================================

* --- USB Usage Index for Keyboard/Keypad Page ---------------------------
* Reference :  Device Class Definition for Human Interface Device(HID)
*              Version 1.0 Draft #4
*              - Appendix A.3    Keyboard/Keypad Page(0x07)
KEY_ERO     equ     $01         ; error roll over
Key_POSTF   equ     $02         ; POST fail
Key_EUD     equ     $03         ; error undefined
KEY_a       equ     $04         ; a / A
KEY_b       equ     $05         ; b / B
KEY_c       equ     $06         ; c / C
KEY_d       equ     $07         ; d / D
KEY_e       equ     $08         ; e / E
KEY_f       equ     $09         ; f / F
KEY_g       equ     $0A         ; g / G
KEY_h       equ     $0B         ; h / H
KEY_i       equ     $0C         ; i / I
KEY_j       equ     $0D         ; j / J
KEY_k       equ     $0E         ; k / K
KEY_l       equ     $0F         ; l / L
KEY_m       equ     $10         ; m / M
KEY_n       equ     $11         ; n / N
KEY_o       equ     $12         ; o / O
KEY_p       equ     $13         ; p / P
KEY_q       equ     $14         ; q / Q
KEY_r       equ     $15         ; r / R
KEY_s       equ     $16         ; s / S
KEY_t       equ     $17         ; t / T
KEY_u       equ     $18         ; u / U
KEY_v       equ     $19         ; v / V
KEY_w       equ     $1A         ; w / W
KEY_x       equ     $1B         ; x / X
KEY_y       equ     $1C         ; y / Y
KEY_z       equ     $1D         ; z / Z
*
KEY_1       equ     $1E         ; 1 / !
KEY_2       equ     $1F         ; 2 / @
KEY_3       equ     $20         ; 3 / #
KEY_4       equ     $21         ; 4 / $
KEY_5       equ     $22         ; 5 / %
KEY_6       equ     $23         ; 6 / ^
KEY_7       equ     $24         ; 7 / &
KEY_8       equ     $25         ; 8 / *
KEY_9       equ     $26         ; 9 / (
KEY_0       equ     $27         ; 0 / )
*
KEY_RTN     equ     $28         ; <ENTER>
KEY_ESC     equ     $29         ; <esc>
KEY_BSP     equ     $2A         ; <backspace>
KEY_TAB     equ     $2B         ; <tab>
KEY_SPC     equ     $2C         ; <space bar>
KEY_MIS     equ     $2D         ; - / _
KEY_EQU     equ     $2E         ; = / +
KEY_OQO     equ     $2F         ; [ / (
KEY_EQO     equ     $30         ; ] / )
KEY_BSL     equ     $31         ; \ / |
KEY_NUP     equ     $32         ; non-US# / ~
KEY_COL     equ     $33         ; ; / :
KEY_CC      equ     $34         ; ' / "
KEY_TIL     equ     $35         ; ` / ~
KEY_CMA     equ     $36         ; , / <
KEY_DOT     equ     $37         ; . / >
KEY_SL      equ     $38         ; / / ?
KEY_CLK     equ     $39         ; <CAPS lock>
*
KEY_F1      equ     $3A         ; <F1>
KEY_F2      equ     $3B         ; <F2>
KEY_F3      equ     $3C         ; <F3>
KEY_F4      equ     $3D         ; <F4>
KEY_F5      equ     $3E         ; <F5>
KEY_F6      equ     $3F         ; <F6>
KEY_F7      equ     $40         ; <F7>
KEY_F8      equ     $41         ; <F8>
KEY_F9      equ     $42         ; <F9>
KEY_F10     equ     $43         ; <F10>
KEY_F11     equ     $44         ; <F11>
KEY_F12     equ     $45         ; <F12>
*
KEY_PSC     equ     $46         ; <print screen>
KEY_SLK     equ     $47         ; <SCROLL lock>
KEY_PSE     equ     $48         ; <pause>
KEY_INS     equ     $49         ; <insert>
KEY_HME     equ     $4A         ; <home>
KEY_PUP     equ     $4B         ; <page up>
KEY_DEL     equ     $4C         ; <delete>
KEY_END     equ     $4D         ; <end>
KEY_PDN     equ     $4E         ; <page down>
KEY_RGT     equ     $4F         ; <right arrow>
KEY_LFT     equ     $50         ; <left arrow>
KEY_DWN     equ     $51         ; <down arrow>
KEY_UP      equ     $52         ; <up arrow>
*
KEY_NLK     equ     $53         ; <NUM lock>
KEY_NSL     equ     $54         ; keypad /
KEY_NMU     equ     $55         ; keypad *
KEY_NMI     equ     $56         ; keypad -
KEY_NPL     equ     $57         ; keypad +
KEY_NET     equ     $58         ; keypad enter
KEY_N1      equ     $59         ; keypad 1 / end
KEY_N2      equ     $5A         ; keypad 2 / down arrow
KEY_N3      equ     $5B         ; keypad 3 / page down
KEY_N4      equ     $5C         ; keypad 4 / left arrow
KEY_N5      equ     $5D         ; keypad 5
KEY_N6      equ     $5E         ; keypad 6 / right arrow
KEY_N7      equ     $5F         ; keypad 7 / home
KEY_N8      equ     $60         ; keypad 8 / up arrow
KEY_N9      equ     $61         ; keypad 9 / page up
KEY_N0      equ     $62         ; keypad 0 / insert
KEY_NDT     equ     $63         ; keypad . / delete
KEY_NBS     equ     $64         ; non-US \ / |
KEY_APP     equ     $65         ; application (Win95)
KEY_PWR     equ     $66         ; power                     [MAC/UNIX]
KEY_EQ      equ     $67         ; =                         [MAC]
KEY_F13     equ     $68         ;                           [MAC]
KEY_F14     equ     $69         ;                           [MAC]
KEY_F15     equ     $6A         ;                           [MAC]
KEY_F16     equ     $6B         ;
KEY_F17     equ     $6C         ;
KEY_F18     equ     $6D         ;
KEY_F19     equ     $6E         ;
KEY_F20     equ     $6F         ;
KEY_F21     equ     $70         ;
KEY_F22     equ     $71         ;
KEY_F23     equ     $72         ;
KEY_F24     equ     $73         ;
KEY_UEXE    equ     $74         ;                           [UNIX]
KEY_UHLP    equ     $75         ;                           [UNIX]
KEY_UMNU    equ     $76         ;                           [UNIX]
KEY_USEL    equ     $77         ;                           [UNIX]
KEY_USTP    equ     $78         ;                           [UNIX]
KEY_UAGN    equ     $79         ;                           [UNIX]
KEY_UUDO    equ     $7A         ;                           [UNIX]
KEY_UCUT    equ     $7B         ;                           [UNIX]
KEY_UCPY    equ     $7C         ;                           [UNIX]
KEY_UPTE    equ     $7D         ;                           [UNIX]
KEY_UFND    equ     $7E         ;                           [UNIX]
KEY_UMUT    equ     $7F         ; Mute key                  [UNIX]
KEY_UVUP    equ     $80         ; Volume Up                 [UNIX]
KEY_UVDN    equ     $81         ; Volume Down               [UNIX]
KEY_UCLK    equ     $82         ;                           [UNIX]
KEY_UNLK    equ     $83         ;                           [UNIX]
KEY_USLK    equ     $84         ;                           [UNIX]
*
KEY_NCMA    equ     $85         ; keypad ,
KEY_NEQU    equ     $86         ; keypad =
KEY_Kanji1  equ     $87         ; kanji1
KEY_Kanji2  equ     $88         ; kanji2
KEY_Kanji3  equ     $89         ; kanji3
KEY_Kanji4  equ     $8A         ; kanji4
KEY_Kanji5  equ     $8B         ; kanji5
KEY_Kanji6  equ     $8C         ; kanji6
KEY_Kanji7  equ     $8D         ; kanji7
KEY_Kanji8  equ     $8E         ; kanji8
KEY_Kanji9  equ     $8F         ; kanji9
KEY_LANG1   equ     $90         ; Language 1
KEY_LANG2   equ     $91         ; Language 2
KEY_LANG3   equ     $92         ; Language 3
KEY_LANG4   equ     $93         ; Language 4
KEY_LANG5   equ     $94         ; Language 5
KEY_LANG6   equ     $95         ; Language 6
KEY_LANG7   equ     $96         ; Language 7
KEY_LANG8   equ     $97         ; Language 8
KEY_LANG9   equ     $98         ; Language 9
KEY_AERSE   equ     $99         ; alternate erase
KEY_SYSREQ  equ     $9A         ; SysReq / Attention
KEY_CANCEL  equ     $9B         ; cancel
KEY_CLEAR   equ     $9C         ; clear
KEY_PRIOR   equ     $9D         ; prior
KEY_RETURN  equ     $9E         ; return
KEY_SEPTOR  equ     $9F         ; separator
KEY_OUT     equ     $A0         ; out
KEY_OPER    equ     $A1         ; oper
KEY_AGAIN   equ     $A2         ; clear / again
KEY_CRSEL   equ     $A3         ; CrSel / Props
KEY_EXSEL   equ     $A4         ; ExSel
*
* ------------------------------------------------------------
*  Usage $A5 to $DF are reserved
* ------------------------------------------------------------
*
* --- Win98 Power Control Key Codes --------------------------------------
* (in Power Control Usage Page)
PKEY_PWR    equ     $A8         ; <power down>
PKEY_SLP    equ     $A9         ; <sleep>
PKEY_WKE    equ     $AA         ; <wake up>
*
KEY_000     equ     $AB         ; REPEAT SEND 3 TIMES 0
*
PKEY_STANDBY	equ	$CB	; One key for all ACPI function
*
KEY_FN	    equ	    $AC		; FN key to alternate key function
*
* -------------------------------------------------------------
*
*                               ; FN - off       FN - on
*                               ;--------------------------
SKEY_0      equ     $C8         ;
SKEY_1      equ     $C9         ;
SKEY_2      equ     $CA         ;
SKEY_3      equ     $CB         ;
SKEY_4      equ     $CC         ;
SKEY_5      equ     $CD         ;
SKEY_6      equ     $CE         ;
SKEY_7      equ     $CF         ;
SKEY_8      equ     $D0         ;
SKEY_9      equ     $D1         ;
SKEY_10     equ     $D2         ;
SKEY_11     equ     $D3         ;
SKEY_12     equ     $D4         ;
SKEY_13     equ     $D5         ;
SKEY_14     equ     $D6         ;
SKEY_15     equ     $D7         ;
*
*
*
KEY_LCTL    equ     $E0         ; <left ctrl>
KEY_LSFT    equ     $E1         ; <left shift>
KEY_LALT    equ     $E2         ; <left alt>
KEY_LGUI    equ     $E3         ; <left GUI>
KEY_RCTL    equ     $E4         ; <right ctrl>
KEY_RSFT    equ     $E5         ; <right shift>
KEY_RALT    equ     $E6         ; <right alt>
KEY_RGUI    equ     $E7         ; <right GUI>
*
* ------------------------------------------------------------
*  Usage code $E8 to $FF are reserved
* ------------------------------------------------------------
*
KEY_H0      equ     $E8		; H0 - HH23 are reported thru
KEY_H1      equ     $E9		;   I/F #1 (Endpoint 2)
KEY_H2      equ     $EA
KEY_H3      equ     $EB
KEY_H4      equ     $EC
KEY_H5      equ     $ED
KEY_H6      equ     $EE
KEY_H7      equ     $EF

KEY_H8      equ     $F0
KEY_H9      equ     $F1
KEY_H10     equ     $F2
KEY_H11     equ     $F3
KEY_H12     equ     $F4
KEY_H13     equ     $F5
KEY_H14     equ     $F6
KEY_H15     equ     $F7

KEY_H16     equ     $F8
KEY_H17     equ     $F9
KEY_H18     equ     $FA
KEY_H19     equ     $FB
KEY_H20     equ     $FC
KEY_H21     equ     $FD
KEY_H22     equ     $FE
KEY_H23     equ     $FF

*
*
* --- Key Codes for internal use only ------------------------------------
* (not reported as USB usage index)
*
CKEY_NA     equ     $00         ; <NULL> - no key



 page
* ====== Key Matrix Table(Type 0) ========================================
* - for Win98 keyboard[Model# K280W]
*   "POWER" key at
*   "SLEEP" key at
*   "WAKE" key at (col_10,row_7)
 IFEQ KeyType0
KEY_TABLE:
*
        dc      KEY_END,KEY_HME,KEY_LFT,CKEY_NA      ; col 10 |  row 0-3
        dc      KEY_UP,KEY_NET,CKEY_NA,KEY_NPL       ; PTB0   |  row 4-7
*
        dc      KEY_9,KEY_F8,KEY_APP,KEY_DOT         ; col 11 |  row 0-3
        dc      CKEY_NA,KEY_L,KEY_F7,KEY_O           ; PTB1   |  row 4-7
*
        dc      KEY_8,KEY_EQU,CKEY_NA,KEY_CMA        ; col 12 |  row 0-3
        dc      KEY_F6,KEY_K,KEY_I,KEY_EQO           ; PTB2   |  row 4-7
*
        dc      KEY_7,KEY_6,KEY_N,KEY_M              ; col 13 |  row 0-3
        dc      KEY_H,KEY_J,KEY_Y,KEY_U              ; PTB3   |  row 4-7
*
        dc      KEY_4,KEY_5,KEY_B,KEY_V              ; col 14 |  row 0-3
        dc      KEY_G,KEY_F,KEY_T,KEY_R              ; PTB4   |  row 4-7
*
        dc      KEY_3,KEY_F2,CKEY_NA,KEY_C           ; col 15 |  row 0-3
        dc      KEY_F4,KEY_D,KEY_F3,KEY_E            ; PTB5   |  row 4-7
*
        dc      KEY_2,KEY_F1,CKEY_NA,KEY_X           ; col 16 |  row 0-3
        dc      CKEY_NA,KEY_S,KEY_CLK,KEY_W          ; PTB6   |  row 4-7
*
        dc      KEY_1,KEY_TIL,CKEY_NA,KEY_Z          ; col 17 |  row 0-3
        dc      KEY_ESC,KEY_A,KEY_TAB,KEY_Q          ; PTB7   |  row 4-7
*
        dc      KEY_PDN,KEY_PUP,KEY_NMI,KEY_NMU      ; col 9  |  row 0-3
        dc      KEY_NDT,KEY_N3,KEY_N6,KEY_N9         ; PTC0   |  row 4-7
*
        dc      PKEY_SLP,KEY_INS,KEY_RGT,KEY_NSL     ; col 8  |  row 0-3
        dc      KEY_N0,KEY_N2,KEY_N5,KEY_N8          ; PTC1   |  row 4-7
*
        dc      PKEY_PWR,KEY_DEL,KEY_DWN,KEY_NLK     ; col 7  |  row 0-3
        dc      KEY_SPC,KEY_N1,KEY_N4,KEY_N7         ; PTC2   |  row 4-7
*
        dc      CKEY_NA,CKEY_NA,CKEY_NA,CKEY_NA      ; col 6  |  row 0-3
        dc      CKEY_NA,KEY_RSFT,KEY_LSFT,PKEY_WKE   ; PTC3   |  row 4-7
*
        dc      KEY_PSC,CKEY_NA,KEY_RALT,CKEY_NA     ; col 3  |  row 0-3
        dc      KEY_LALT,CKEY_NA,CKEY_NA,KEY_SLK     ; PTC4   |  row 4-7
*
        dc      KEY_F5,KEY_LCTL,CKEY_NA,KEY_RCTL     ; col 2  |  row 0-3
        dc      CKEY_NA,CKEY_NA,CKEY_NA,KEY_PSE      ; PTC5   |  row 4-7
*
        dc      CKEY_NA,CKEY_NA,CKEY_NA,CKEY_NA      ; col 1  |  row 0-3
        dc      CKEY_NA,CKEY_NA,KEY_LGUI,CKEY_NA     ; PTC6   |  row 4-7
*
        dc      CKEY_NA,CKEY_NA,CKEY_NA,CKEY_NA      ; col 0  |  row 0-3
        dc      CKEY_NA,KEY_RGUI,CKEY_NA,CKEY_NA     ; PTC7   |  row 4-7
*
        dc      KEY_0,KEY_MIS,KEY_SL,CKEY_NA         ; col 4  |  row 0-3
        dc      KEY_CC,KEY_COL,KEY_OQO,KEY_P         ; PTE0   |  row 4-7
*
        dc      KEY_F10,KEY_F9,KEY_F12,KEY_RTN       ; col 5  |  row 0-3
        dc      KEY_F11,KEY_BSL,KEY_BSP,CKEY_NA      ; PTE2   |  row 4-7
*
SKEY_TABLE_OFF   equ     *
        dc      KEY_7,KEY_8,KEY_9,KEY_0		; SKEY_0, 1, 2, 3
        dc      KEY_u,KEY_i,KEY_o,KEY_p         ;      4, 5, 6, 7
*
        dc      KEY_j,KEY_k,KEY_l,KEY_COL       ; SKEY_8, 9, 10, 11
        dc      KEY_RTN,KEY_m,KEY_DOT,KEY_LCTL    ;      12, 13, 14, 15
*

SKEY_TABLE_ON    equ     *
        dc      KEY_N7,KEY_N8,KEY_N9,KEY_NMU	; SKEY_0, 1, 2, 3
        dc      KEY_N4,KEY_N5,KEY_N6,KEY_NMI	;      4, 5, 6, 7
*
        dc      KEY_N1,KEY_N2,KEY_N3,KEY_NPL    ; SKEY_8, 9, 10, 11
        dc      KEY_NET,KEY_N0,KEY_NDT,KEY_RCTL  ;      12, 13, 14, 15


 page
 ENDIF
*
*
 page
* ====== Key Matrix Table(Type 1) ========================================
* for KH12 demo kit keyboard matrix
* - for Win98 keyboard[Model# K280W]
*   "POWER" key at (col_14,row_7)
*   "SLEEP" key at (col_13,row_7)
*   "WAKE" key at (col_10,row_7)
 IFEQ KeyType1
KEY_TABLE:
*
        dc      KEY_CLK,KEY_W,CKEY_NA,KEY_S         ; col 0  |  row 0-3
        dc      CKEY_NA,KEY_X,KEY_2,KEY_F1          ; PTB0   |  row 4-7
*
        dc      KEY_TAB,KEY_Q,KEY_ESC,KEY_A         ; col 1  |  row 0-3
        dc      CKEY_NA,KEY_Z,KEY_1,KEY_TIL         ; PTB1   |  row 4-7
*
        dc      KEY_F3,KEY_E,KEY_F4,KEY_D           ; col 2  |  row 0-3
        dc      CKEY_NA,KEY_C,KEY_3,KEY_F2          ; PTB2   |  row 4-7
*
        dc      KEY_T,KEY_R,KEY_G,KEY_F             ; col 3  |  row 0-3
        dc      KEY_B,KEY_V,KEY_4,KEY_5             ; PTB3   |  row 4-7
*
        dc      KEY_Y,KEY_U,KEY_H,KEY_J             ; col 4  |  row 0-3
        dc      KEY_N,KEY_M,KEY_7,KEY_6             ; PTB4   |  row 4-7
*
        dc      KEY_EQO,KEY_I,KEY_F6,KEY_K          ; col 5  |  row 0-3
        dc      CKEY_NA,KEY_CMA,KEY_8,KEY_EQU       ; PTB5   |  row 4-7
*
        dc      KEY_F7,KEY_O,CKEY_NA,KEY_L          ; col 6  |  row 0-3
        dc      KEY_APP,KEY_DOT,KEY_9,KEY_F8        ; PTB6   |  row 4-7
*
        dc      CKEY_NA,KEY_NPL,KEY_UP,KEY_NET      ; col 7  |  row 0-3
        dc      KEY_LFT,CKEY_NA,KEY_END,KEY_HME     ; PTB7   |  row 4-7
*
        dc      CKEY_NA,CKEY_NA,CKEY_NA,KEY_RGUI    ; col 17 |  row 0-3
        dc      CKEY_NA,CKEY_NA,CKEY_NA,CKEY_NA     ; PTC0   |  row 4-7
*
        dc      KEY_LGUI,CKEY_NA,CKEY_NA,CKEY_NA    ; col 16 |  row 0-3
        dc      CKEY_NA,CKEY_NA,CKEY_NA,CKEY_NA     ; PTC1   |  row 4-7
*
        dc      CKEY_NA,KEY_PSE,CKEY_NA,CKEY_NA     ; col 15 |  row 0-3
        dc      CKEY_NA,KEY_RCTL,KEY_F5,KEY_LCTL    ; PTC2   |  row 4-7
*
        dc      CKEY_NA,KEY_SLK,KEY_LALT,CKEY_NA    ; col 14 |  row 0-3
        dc      KEY_RALT,CKEY_NA,KEY_PSC,CKEY_NA    ; PTC3   |  row 4-7
*
        dc      KEY_LSFT,PKEY_WKE,CKEY_NA,KEY_RSFT  ; col 11 |  row 0-3
        dc      CKEY_NA,CKEY_NA,CKEY_NA,CKEY_NA     ; PTC4   |  row 4-7
*
        dc      KEY_N4,KEY_N7,KEY_SPC,KEY_N1        ; col 10 |  row 0-3
        dc      KEY_DWN,KEY_NLK,PKEY_PWR,KEY_DEL    ; PTC5   |  row 4-7
*
        dc      KEY_N5,KEY_N8,KEY_N0,KEY_N2         ; col 9  |  row 0-3
        dc      KEY_RGT,KEY_NSL,PKEY_SLP,KEY_INS    ; PTC6   |  row 4-7
*
        dc      KEY_N6,KEY_N9,KEY_NDT,KEY_N3        ; col 8  |  row 0-3
        dc      KEY_NMI,KEY_NMU,KEY_PDN,KEY_PUP     ; PTC7   |  row 4-7
*
        dc      KEY_BSP,CKEY_NA,KEY_F11,KEY_BSL     ; col 12 |  row 0-3
        dc      KEY_F12,KEY_RTN,KEY_F10,KEY_F9      ; PTE0   |  row 4-7
*
        dc      KEY_OQO,KEY_P,KEY_CC,KEY_COL        ; col 13 |  row 0-3
        dc      KEY_SL,CKEY_NA,KEY_0,KEY_MIS        ; PTE2   |  row 4-7
*
SKEY_TABLE_OFF   equ     *
        dc      KEY_7,KEY_8,KEY_9,KEY_0		; SKEY_0, 1, 2, 3
        dc      KEY_u,KEY_i,KEY_o,KEY_p         ;      4, 5, 6, 7
*
        dc      KEY_j,KEY_k,KEY_l,KEY_COL       ; SKEY_8, 9, 10, 11
        dc      KEY_RTN,KEY_m,KEY_DOT,KEY_LCTL    ;      12, 13, 14, 15
*

SKEY_TABLE_ON    equ     *
        dc      KEY_N7,KEY_N8,KEY_N9,KEY_NMU	; SKEY_0, 1, 2, 3
        dc      KEY_N4,KEY_N5,KEY_N6,KEY_NMI	;      4, 5, 6, 7
*
        dc      KEY_N1,KEY_N2,KEY_N3,KEY_NPL    ; SKEY_8, 9, 10, 11
        dc      KEY_NET,KEY_N0,KEY_NDT,KEY_RCTL  ;      12, 13, 14, 15
*


 ENDIF
*
 IFEQ KeyType2
KEY_TABLE:
* ====== Key Matrix Map ==================================================
* (for Motorola Hot key test Keyboard)
*
*
        dc      SKEY_8,SKEY_9,SKEY_10,SKEY_11           ;           row 0-3 
        dc      SKEY_12,SKEY_13,SKEY_14,KEY_SL          ; PTB0   |  row 4-7 
*
        dc      KEY_N7,KEY_N8,KEY_N9,KEY_NMU		;        |  row 0-3
        dc      KEY_N4,KEY_N5,KEY_N6,KEY_NMI		; PTB1   |  row 4-7
*
        dc      KEY_N1,KEY_N2,KEY_N3,KEY_NPL		;        |  row 0-3
        dc      KEY_NET,KEY_N0,KEY_NDT,KEY_NSL		; PTB2   |  row 4-7
*
        dc      KEY_LFT,KEY_UP,KEY_DWN,KEY_RGT		;        |  row 0-3)
        dc      KEY_HME,KEY_PUP,KEY_PDN,KEY_END		; PTB3   |  row 4-7
*
        dc      SKEY_15,CKEY_NA,CKEY_NA,CKEY_NA		;        |  row 0-3)
        dc      KEY_RCTL,PKEY_PWR,PKEY_SLP,PKEY_WKE	; PTB4   |  row 4-7
*
        dc      KEY_H7,KEY_H0,KEY_H6,KEY_H1 	        ;        |  row 0-3
        dc      KEY_H4,KEY_LALT,KEY_RALT,KEY_Kanji5     ; PTB5   |  row 4-7
*
        dc      KEY_H5,KEY_H8,KEY_LSFT,KEY_RSFT         ;        |  row 0-3
        dc      KEY_H23,KEY_000,KEY_LANG2,KEY_Kanji4    ; PTB6   |  row 4-7
*
        dc      KEY_H2,KEY_LGUI,KEY_H9,KEY_H18          ;        |  row 0-3
        dc      KEY_H10,KEY_H11,KEY_LANG1,KEY_Kanji2    ; PTB7   |  row 4-7
*
        dc      SKEY_0,SKEY_1,SKEY_2,SKEY_3             ;        |  row 0-3 
		dc      SKEY_4,SKEY_5,SKEY_6,SKEY_7             ; PTC0   |  row 4-7 
*
        dc      KEY_LFT,KEY_UP,KEY_DWN,KEY_RGT          ;        |  row 0-3
        dc      KEY_NBS,KEY_Kanji1,KEY_NCMA,KEY_APP     ; PTC1   |  row 4-7
*
        dc      KEY_Kanji3,KEY_INS,KEY_DEL,KEY_NLK      ;        |  row 0-3
        dc      KEY_BSL,KEY_PSC,KEY_SLK,KEY_PSE         ; PTC2   |  row 4-7
*
		dc      KEY_F4,KEY_F5,KEY_F6,KEY_F7             ;        |  row 0-3
        dc      KEY_F8,KEY_F10,KEY_F11,KEY_F12          ; PTC3   |  row 4-7
*
        dc      KEY_z,KEY_x,KEY_c,KEY_v                 ;        |  row 0-3
        dc      KEY_a,KEY_s,KEY_d,KEY_f                 ; PTC4   |  row 4-7
*
		dc      KEY_q,KEY_w,KEY_e,KEY_r                 ;        |  row 0-3
        dc      KEY_1,KEY_2,KEY_3,KEY_4                 ; PTC5   |  row 4-7
*
        dc      KEY_ESC,KEY_F1,KEY_F2,KEY_F3            ;        |  row 0-3
        dc      KEY_TIL,KEY_TAB,KEY_SPC,KEY_CMA         ; PTC6   |  row 4-7

        dc      KEY_FN,KEY_H3,PKEY_SLP,KEY_H8           ;        |  row 0-3
        dc      KEY_CLK,CKEY_NA,CKEY_NA,KEY_RGUI        ; PTC7   |  row 4-7
*
        dc      KEY_5,KEY_t,KEY_g,KEY_b                 ;        |  row 0-3
        dc      KEY_6,KEY_y,KEY_h,KEY_n                 ; PTE0   |  row 4-7
*
        dc      KEY_MIS,KEY_EQU,KEY_BSP,KEY_OQO         ;        |  row 0-3
        dc      KEY_EQO,KEY_CC,KEY_NUP,KEY_F9           ; PTE2   |  row 4-7
*


*


 page

* ====== Alternated Key Matrix Map (for FN key) ==========================
*
*
SKEY_TABLE_OFF   equ     *
        dc      KEY_7,KEY_8,KEY_9,KEY_0		; SKEY_0, 1, 2, 3
        dc      KEY_u,KEY_i,KEY_o,KEY_p         ;      4, 5, 6, 7
*
        dc      KEY_j,KEY_k,KEY_l,KEY_COL       ; SKEY_8, 9, 10, 11
        dc      KEY_RTN,KEY_m,KEY_DOT,KEY_LCTL    ;      12, 13, 14, 15
*

SKEY_TABLE_ON    equ     *
        dc      KEY_N7,KEY_N8,KEY_N9,KEY_NMU	; SKEY_0, 1, 2, 3
        dc      KEY_N4,KEY_N5,KEY_N6,KEY_NMI	;      4, 5, 6, 7
*
        dc      KEY_N1,KEY_N2,KEY_N3,KEY_NPL    ; SKEY_8, 9, 10, 11
        dc      KEY_NET,KEY_N0,KEY_NDT,KEY_RCTL  ;      12, 13, 14, 15
*


 ENDIF

* ====== Subroutines =====================================================

* --------------------------------------------------------------------- *
* LOCATE_KEY - return key number from KET_TABLE[row,col]                *
* In       :  reg_A             row number                              *
*             reg_X             column number                           *
* Out      :  V_KeyNum          key number                              *
* --------------------------------------------------------------------- *
LOCATE_KEY:
*
* --- Calculate Index to KNum_Table ---------------------------------
        sta     V_KeyNum                ; temp storage for row #
*
        lda     #K_ROWMAX
        mul                             ; reg_A * reg_X -> reg_X:reg_A
        add     V_KeyNum
*
* --- Get the [(COL*K_ROWMAX)+ROW]th item in KNUM_Table -------------
        tax
        lda     KEY_TABLE,x
        sta     V_KeyNum                ; store result
*
* -------------------------------------------------------------------
LKEY_EXIT:
        rts




* ====== End of File =====================================================
            end
