* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Motorola Semiconductors (H.K.) Ltd.
* Motorola MCU Division - Technical Marketing Section
*
* FileName  :  USB-SCAN-ASM.h
* Title     :  Keyboard Matrix Scanner & Handler
* Author    :  Tony Luk
*
* Compiler  :  CodeWarrior
* Version	:  2.1
*
*
*              DD/MM/YY     Rev.    Modified comments.
* History   :  15/06/98     0.1     created
*              03/08/98     1.0     extend Scan_MTX routine to support
*                                     8 row x 18 column
*              17/10/98     1.1     separate init. column mask table
*                                     from Scan_Init
*                                     (to speed up keyboard reset)
*              24/10/98     1.2     update key matrix scan method
*                                   (see note 1)
*              23/03/99     1.3     skip columns with no key press to
*                                     speed up Ghost_Key & Scan_Hdlr
*                                     routines
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Disclaimer of All Warranties & Liabilities :-
*  This Program is a freeware to demonstrate the operation of HC08 micro-
*  controller. In no event will Motorola be liable for any damages, or any
*  incidental or consequential damages arising out of the use of or
*  inability to use this program. Motorola does not make any warranties
*  of any kind that the program does not or will not infringe any copyright,
*  patent, trade secret or other intellectual property right of any third
*  party in any country.
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Notes :-
* 1) key matrix scan method
*    - whenever ghost keys detected, error codes are reported
*    - error codes are cleared whenever error conditions no longer exist
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *


* ====== Compile Options =================================================
;
 BASE 10




* ====== MACRO Definition ================================================
* ------ Include Files ---------------------------------------------------


* ------ MACRO Definition ------------------------------------------------


 page
* ====== Subroutines =====================================================
* Scan_Init     initialize key matrix scan
* Col_Msk_Init  initialize column mask table of key matrix map
* Scan_MTX      scan matrix keyboard(non-diode)
* Ghost_Key     detect ghost keys, and then mask off rows with ghost keys
* COUNT_ONE     count the no. of '1' in <reg_A>
* Scan_Hdlr     process scan results
* ADD_KEY       add reg_A to USB keyboard report
* BIT_TABLE     return bit position in a byte
* Dly_40us      delay 40us (HC08, Fcpu=6MHz) - by inst. cycles


* --------------------------------------------------------------------- *
* Scan_Init - Initialize key matrix scan                                *
* In       :  <nil>                                                     *
* Out      :  b_CKPRESS         =0  no key pressed(for curr. scan)      *
*             b_KGHOST          =0  no ghost keys detected              *
*             Q_CScan[0..17]    =0  no pressed key latched in table     *
* Call     :  <nil>                                                     *
* --------------------------------------------------------------------- *
Scan_Init:
		clra	
        sta     VS_Scan                 ; clr key press flag,
                                        ; clr ghost key flag
*
*
*
* --- Initialize Scan Results --------------------------------------------
        ldx     #K_COLMAX               ; start with last column
        clra
SINI_PVAGN:
        sta     (Q_CScan-1),x           ; clear curr. scan results
        DBNZX   SINI_PVAGN              ; ~last column ?
*
*
*
        ldx     #K_IRPT0_SIZE
		clra
SINI_RST_NXT:
        sta     (QC_IN0_Rpt-1),x        ; reset kbd report
        DBNZX   SINI_RST_NXT
*
*
        ldx     #K_H_BUFF_SIZE
        clra
SINI_Clr_H_Bfr
        sta     (H_RPT_BEGIN-1),x
        dbnzx   SINI_Clr_H_Bfr
*
*
*
*
* ------------------------------------------------------------------------
SINI_EXIT:
        rts




* --------------------------------------------------------------------- *
* Col_Msk_Init - Initialize column mask table of key matrix map         *
* In       :  KEY_TABLE         key matrix map                          *
*             K_COLMAX          max. # of columns                       *
*             K_ROWMAX          max. # of rows                          *
* Out      :  Q_ColMsk[0..17]   column mask table(0- blank key)         *
* Call     :  LOCATE_KEY                                                *
* Note     :- column mask table is used to mask out blank keys in       *
*             a key matrix map, in order to speed up the ghost key      *
*             detection algorithm                                       *
*           - column mask is only required to initialize once during    *
*             power-on reset                                            *
* --------------------------------------------------------------------- *
Col_Msk_Init:
		clra
        sta     VP_Col                  ; start with column 0
CMI_CAGN:
        sta     V_ColVal                ; temp. byte for column mask
*
        sta     VP_Row                  ; start with row 0
CMI_RAGN:
*
* --- Get Key Code at Location (VP_Row,VP_Col) ---------------------------
        lda     VP_Row
        ldx     VP_Col
        jsr     LOCATE_KEY              ; return key no. in V_KeyNum
*
* --- Update Temp. Column Mask -------------------------------------------
        clc
        lda     V_KeyNum
        KCMPEQ  low(CKEY_NA),CMI_BLANKKEY    ; blank key ?
        sec
CMI_BLANKKEY:
*
        ror     V_ColVal                ; shift in mask bit
*
* --- Check Next Row -----------------------------------------------------
CMI_NROW:
*
        inc     VP_Row
*
        ldx     VP_Row
        KCPXNE  K_ROWMAX,CMI_RAGN       ; ~last row ?
*
* --- Check Next Column --------------------------------------------------
CMI_NCOL:
        lda     V_ColVal
        ldx     VP_Col
        sta     Q_ColMsk,x              ; store column mask
*
        inc     VP_Col
        ldx     VP_Col
        KCPXNE  K_COLMAX,CMI_CAGN       ; ~last column ?
*
*
*
* ------------------------------------------------------------------------
CMI_EXIT:
        rts




* --------------------------------------------------------------------- *
* Scan_MTX - scan matrix keyboard(non-diode)                            *
* In       :  b_CKPRESS          =1  key press detected for prev. scan  *
*             Q_CScan[0..17]     table of prev. scan result             *
*             Q_ColMsk[0..17]    table of column mask(for blank keys)   *
* Out      :  Q_CScan[0..17]     table of curr. scan result             *
*                                (if a bit is set, key at that pos. is  *
*                                 pressed.)                             *
*             b_CKPRESS          =1  key press detected for curr. scan  *
* Call     :  Dly_40us                                                  *
* --------------------------------------------------------------------- *
Scan_MTX:
*
* --- Check for Key(s) Pressed -------------------------------------------
* (pull low all column lines to check if any key pressed)
SMTX_CKPRESS:
*
;; (ensure column data zero)
        clr     PTB                     ; put zero to column data bits
        clr     PTC
*
        lda     V_PTE
        and     #%11111010              ; put zero PTE0,2 col 16, 17
        sta     PTE

*
        lda     #$FF                    ; pull low all column lines
        sta     DDRB                    ; (related data reg. default to $00)
        sta     DDRC
*
        bset    b_COL16,DDRE          ; pull low col 16
        bset    b_COL17,DDRE          ; pull low col 17
*
        jsr     Dly_40us                ; wait column lines stable
*
        lda     Port_Row                ; read back row lines
        coma                            ; key pressed(active low)
        bne     SMTX_SCAN               ; key press detected ?
*
* --- Clear Current Scan Result (for no key press detected) --------------
SMTX_RST:
        bclr    b_CKPRESS,VS_Scan       ; signal no key pressed
*
        clra
        ldx     #K_COLMAX               ; start with last column
SMTX_RNXT:
        sta     (Q_CScan-1),x           ; clear Q_CScan[x]
        DBNZX   SMTX_RNXT               ; ~last column ?
*
        bra     SMTX_EXIT
*
*
*
* --- Scan Keys Matrix ---------------------------------------------------
SMTX_SCAN:
*
        bset    b_CKPRESS,VS_Scan       ; signal key pressed
*
;; (done in SMTX_CKPRESS above)
        clr     PTB                     ; put zero to column data bits
        clr     PTC
*
*
        MOV     #%00000001,DDRB          ; init. $00001 to Col[17..0]
        MOV     #%00000000,DDRC
        bclr    b_COL16,DDRE          ; release high col 16
        bclr    b_COL17,DDRE          ; release high col 17
*
        clr     VP_Col                  ; start with col #0
SMTX_SNCOL:
        jsr     Dly_40us                ; wait column lines stable
*
        ldx     VP_Col                  ; get curr. col ptr
*
        lda     Q_CScan,x               ; copy prev. Q_CScan[x] to
        sta     Q_PScan,x               ;   curr. Q_PScan[x]
*
        lda     Port_Row                ; check row status
        coma                            ; key pressed(active low)
        and     Q_ColMsk,x              ; mask out blank key pos.
        sta     Q_CScan,x               ; store result to scan buffer
*
        lsl     DDRB                    ; force low next column
        rol     DDRC
*
        bcc     SMTX_NC16               ; reach col 16 ?
        bset    b_COL16,DDRE          ; force low col 16

SMTX_NC16:
        KCPXNE  16,SMTX_NC17            ; reach col 17 ?
        bclr    b_COL16,DDRE          ; force high col 16
        bset    b_COL17,DDRE          ; force low col 17

SMTX_NC17:
*
        inc     VP_Col
        ldx     VP_Col
        KCPXNE  K_COLMAX,SMTX_SNCOL     ; ~last column ?
*
*
*
* -------------------------------------------------------------------
SMTX_EXIT:
        clr     DDRB                    ; release high all column lines
        clr     DDRC
*
        bclr    b_COL16,DDRE          ; release high PTE0,2 col 16, 17
        bclr    b_COL17,DDRE
*
        rts




* --------------------------------------------------------------------- *
* Ghost_Key - detect ghost keys, and then mask off rows with ghost keys *
* In       :  Q_CScan       table of current scan result                *
*             b_CKPRESS     =1 key press detected on curr. scan         *
* Out      :  b_KGHOST      =1 ghost key detected                       *
* Call     :  COUNT_ONE                                                 *
* --------------------------------------------------------------------- *
Ghost_Key:
        bclr    b_KGHOST,VS_Scan        ; reset ghost key flag
*
        brclr   b_CKPRESS,VS_Scan,GKEY_EXIT  ; no key press detected ?
*
*
*
* --- Locate Rows with Ghost Keys in Q_CScan[0..K_COLMAX] ----------------
		lda		#(K_COLMAX-1)
		sta		VP_Col				    ; start with last column
GKEY_NELOOP:
*
* --- Count the no. of pressed keys in Current Column ----------
        ldx     VP_Col                  ; get current column value
        lda     Q_CScan,x
        beq     GKEY_NECOL              ; no keys pressed in curr. column ?
        sta     V_ColVal
*
        jsr     COUNT_ONE               ; count no. of '1' in <reg_A>
        lda     VC_One
        lsra
        beq     GKEY_NECOL              ; < 2 keys pressed in curr. column ?
*
* --- Check Ghost Keys overlapped with Other Columns -----------
        lda     VP_Col                  ; start with (VP_Col-1)
        deca
        sta     VP_ChkCol
GKEY_NILOOP:
*
* --- Locate Overlapped Keys -------------------------
        ldx     VP_ChkCol
        lda     Q_CScan,x
        and     V_ColVal
        beq     GKEY_NICOL              ; no overlapped keys ?
*
        jsr     COUNT_ONE               ; count no. of '1' in <reg_A>
        lda     VC_One
        lsra
        beq     GKEY_NICOL              ; < 2 overlapped keys ?
*
* --- Ghost Keys Found -------------------------------
        bset    b_KGHOST,VS_Scan        ; set ghost key flag
        bra     GKEY_EXIT
*
* --- Check Next Overlapped Column -------------------
GKEY_NICOL:
        dec     VP_ChkCol
        bpl     GKEY_NILOOP             ; ~last overlapped column ?
*
* --- Check Next Column ----------------------------------------
GKEY_NECOL:
        dec     VP_Col
        bne     GKEY_NELOOP             ; ~last check column ?
*
*
*
* -----------------------------------------------------------------------
GKEY_EXIT:
        rts




* --------------------------------------------------------------------- *
* COUNT_ONE - count the no. of '1' in <reg_A>                           *
* In       :  <reg_A>       byte under test                             *
* Out      :  VC_One        no. of '1' in <reg_A>                       *
* Call     :  <nil>                                                     *
* --------------------------------------------------------------------- *
COUNT_ONE:
        clr     VC_One                  ; reset 1's counter
        ldx     #K_ROWMAX               ; count the no. of pressed keys
CONE_AGN:
        lsra
        bcc     CONE_ZERO
CONE_ONE:
        inc     VC_One
CONE_ZERO:
        DBNZX   CONE_AGN
*
* -------------------------------------------------------------------
CONE_EXIT:
        Rts




* --------------------------------------------------------------------- *
* Scan_Hdlr - process scan results                                      *
* In       :  Q_CScan[0..17]    current scan result                     *
*             b_CKPRESS         =1 key(s) press detected for curr. scan *
* Out      :  QP_IN0_RPT        prev. keyboard report                   *
*             QC_IN0_Rpt        curr. keyboard report                   *
* Call     :  LOCATE_KEY, ADD_KEY                                       *
* --------------------------------------------------------------------- *
Scan_Hdlr:
	clr	V_KeyNum		; clear the LOCATE_KEY buffer
	bclr	b_K_FN_FND,VS_Scan	; clear the flag first.
* ------------------------------------------------------------------------
*
	brclr	b_3Z_KEY,V_3_Zero,SHDL_END_3Z
	bclr	b_3Z_KEY_P,V_3_Zero	; reset the '000' key flag

	lda	V_3_Zero		; Check if end of sequence
	and	#%00000111
	beq	ALL_SND_OVR		; Always set '0' afterward

	dec	V_3_Zero
	and	#%00000001
	beq	SET_INH_3Z
; ---------------------------------------
CLR_INH_3Z:
	bclr	b_INH_3Z,V_3_Zero	; Set the '0' key ON

	bra	SHDL_END_3Z
; ---------------------------------------
ALL_SND_OVR:

	bset	b_SEND_OVR,V_3_Zero

;	mov		#KC_40ms,VC_40ms	; restore the scan rate

	mov		#4,low(VC_40ms)

	bra		CLR_INH_3Z
; ---------------------------------------
SET_INH_3Z:
	bset	b_INH_3Z,V_3_Zero	; Set the '0' key OFF

SHDL_END_3Z:
; ---------------------------------------

*
*
* --- Handle Ghost Key Detected ------------------------------------------
SHDL_Ghost_Key:
        brclr   b_KGHOST,VS_Scan,SHDL_GKEY_END  ; ~ghost key detect ?
*
		lda		V_MKey
		sta		VP_MKey         ; push V_MKey
		clra
        sta     V_MKey
*
        ldx     #K_KNCSIZE
SHDL_GKEY_NXT:
        lda     (Q_NKey-1),x           ; push keycode report
        sta     (QP_NKey-1),x
*
        lda     #low(KEY_ERO)                ; fill all keycodes with #KEY_EROvr
        sta     (Q_NKey-1),x
        DBNZX   SHDL_GKEY_NXT
* ------------------------------------------------------------------------
        jmp     SHDL_NKEY
*
* --- End of Handle Ghost Key Detected -----------------------------------
SHDL_GKEY_END:
*
*
*
* --- Record Pressed Key(s) to Report ------------------------------------
*
*  Clear the working buffer first
SHDL_ON:
        ldx     #(K_KNCSIZE+K_KNCSIZE) ; move key# and FN status
SHDLN_Clr_Scan_Buf:
        lda     (Q_N_Key-1),x        ; push kbd report
        sta     (Q_P_Key-1),x
*
        clra
        sta     (Q_N_Key-1),x        ; reset kbd report
        DBNZX   SHDLN_Clr_Scan_Buf
* ----------------------------------------------------------------------
 		lda		V_MKey
 		sta		VP_MKey
 		lda		V_RKey
 		sta		VP_RKey
* ----------------------------------------------------------------------
;        mov	V_ACPI_Key,V_PreACPI_Rpt	; push the ACPI KEY STATUS

		lda		V_ACPI_Key
		sta		V_PreACPI_Rpt				; push the ACPI KEY STATUS
		
		lda		V_Hot_Key1,
		sta		V_PreHot_Rpt1				; push the hot key
		
		lda		V_Hot_Key2,
		sta		V_PreHot_Rpt2				; push the hot key
		
		lda		V_Hot_Key3,
		sta		V_PreHot_Rpt3				; push the hot key
		
		lda		V_Hot_Key4,
		sta		V_PreHot_Rpt4				; push the hot key
		
	
* ----------------------------------------------------------------------
		clra
		sta		V_MKey
		sta		V_ACPI_Key	        ; clear ACPI key working buffer
		sta		V_Hot_Key1		; clear hot key buffer
		sta		V_Hot_Key2
		sta		V_Hot_Key3
		sta		V_Hot_Key4

* ----------------------------------------------------------------------
        clr     VP_KCode	; Count # of pressed key ?
*
        brclr   b_CKPRESS,VS_Scan,SHDLN_END  ; ~key press detected in curr.
                                             ;  scan ?
*
* --- Extract Make Key Codes at current Column ----------------------
        clr     VP_Col                  ; start with col #0
SHDLN_NCOL:
        ldx     VP_Col
        lda     Q_CScan,x               ; get curr. column value
        beq     SHDLN_CCOL              ; column with no key press ?
*
        sta     V_ColVal
*
* --- Extract Make Key Codes at current (Row,Column) -----------
        clr     VP_Row                  ; start with row #0
SHDLN_NROW:
        lsr     V_ColVal
        bcc     SHDLN_CROW              ; ~make key at (VP_Col,VP_Row) ?
*
        lda     VP_Row
        ldx     VP_Col
        jsr     LOCATE_KEY              ; return key code in V_KeyNum
*
        lda     V_KeyNum
        jsr     Add_Scan_Buffer
;        jsr     ADD_KEY                 ; add pressed key to keyboard
                                        ;   report
*
* --- Next Row -------------------------------------------------
SHDLN_CROW:
        inc     VP_Row
        ldx     VP_Row
        KCPXNE  K_ROWMAX,SHDLN_NROW     ; ~last Row ?
*
* --- Next Column ---------------------------------------------------
SHDLN_CCOL:
        inc     VP_Col
        ldx     VP_Col
        KCPXNE  K_COLMAX,SHDLN_NCOL     ; ~last Column ?
*
* ------------------------------------------------------------------------
SHDLN_END:

	brclr	b_3Z_KEY,V_3_Zero,SHDL_END_3Z_2

	brset	b_3Z_KEY_P,V_3_Zero,SHDL_END_3Z_2	; SEQ. is not over
							;  and Key is ON
; -----------------------------------------
	brclr	b_SEND_OVR,V_3_Zero,SEQ_NOT_END	; SEQ. is not over and key is off

	clr	V_3_Zero		; ON-OFF sequence is over and
					;     '000' key is released
	bra	SHDL_END_3Z_2
; ----------------------------------------
SEQ_NOT_END:

	brset	b_INH_3Z,V_3_Zero,SHDL_END_3Z_2

	lda		#low(KEY_000)
	jsr		Add_Scan_Buffer			; Force to put a KEY_0

SHDL_END_3Z_2:
* ----------------------------------------------------------------
*
*  Back up the last report to QP_KBD_RPT buffer
*   The Modifier key left unchanged
*
        ldx     #K_KNCSIZE       ; not include the Modifier, reserved keys)
SHDLN_RST_NXT:
        lda     (Q_NKey-1),x        ; back up kbd report
        sta     (QP_NKey-1),x
*

		clra
		sta     (Q_NKey-1),x        ; clear current kbd report
        DBNZX   SHDLN_RST_NXT
* -----------------------------------------------------------------------
	brset	b_K_FN_FND,VS_Scan,FN_K_PRSD

	bclr	b_FN_STATE,VS_Scan	; THe FN key is released

FN_K_PRSD:
* -----------------------------------------------------------------------
	lda	VP_KCode		; Get # of pressed key

	beq	NO_KEY_PRS		; Skip parsing if no key is pressed
*
* ------------------------------------------------------------------------
	clr	VP_KCode		; Reset the index pointer
* ------------------------------------------------------------------------
					; check the content of scan buffer
	jsr	PARSE_KEY		; & translate FN key if exist

	bra	SHDL_NKEY
*					;
*
*
* === Not any key pressed =================================================
NO_KEY_PRS:

        ldx     #(K_KNCSIZE+K_KNCSIZE)
        clra

CLR_FNKY_NXT_1:
        sta     (Q_N_Key-1),x
        sta     (Q_P_Key-1),x

        DBNZX   CLR_FNKY_NXT_1
*

*
*
*
* --- Check Keyboard Report Change ---------------------------------------
SHDL_NKEY:
        ldx     #K_IRPT0_SIZE
SHDL_NKY_NXT:
        lda     (QC_IN0_Rpt-1),x
        cmp     (QP_IN0_RPT-1),x
        beq     SHDL_NKY_CTI            ;  prev. value = curr. value ?
*
        bset    b_NIRPT0,V_Dev_Status    ; signal kbd status update
        bra     SHDL_NKY_END
SHDL_NKY_CTI:
        DBNZX   SHDL_NKY_NXT
; ................................................................
        lda	V_ACPI_Key
        cmp	V_PreACPI_Rpt		; Check if Key status changed?
        beq	SHDL_End_ACPI

        bset    b_ACPI_NRpt,V_HK_Status   ; signal ACPI Key status update
; ................................................................
SHDL_End_ACPI:
*
        ldx     #K_H_RPT_SIZE
Chk_Nxt_Hot:
        lda     (V_Hot_Key1-1),x
        cmp     (V_PreHot_Rpt1-1),x
        beq     No_New_H1               ;  prev. value = curr. value ?
*
        bset    b_HK_NRPT,V_HK_Status   ; signal HOT Key status update
        bra     SHDL_NKY_END

No_New_H1:
        DBNZX   Chk_Nxt_Hot
* ------------------------------------------------------------------------
SHDL_NKY_END:
        rts




* --------------------------------------------------------------------- *
* Add_Scan_Buffer - add reg_A to Keyboard scan buffer                   *
* In        :  	reg_A           key# to be added                        *
*              	b_FN_key	the FN state				*
*		Q_P_Key[0..5]	the last result				*
*		Q_P_FN[0..5]	the last FN state			*
* Out       :  	Q_N_Key[0..5]	update new make key                     *
*     		Q_N_FN[0..5]						*
* Call      :  	<nil>                                                   *
* --------------------------------------------------------------------- *
Add_Scan_Buffer:
*
* --- Handle Keys Reported in Modifier Byte ------------------------------
ASKEY_MODIFIER:
        KCMPLO  low(KEY_LCTL),NOT_MDFY_KEY   ; < Left Ctrl usage index ?
        KCMPHI  low(KEY_RGUI),NOT_MDFY_KEY   ; > Right GUI usage index ?
*
        sub     #low(KEY_LCTL)               ; calculate index to BIT_TABLE
        tax
*
        lda     BIT_TABLE,x             ; set corresp. bit in V_MKey
        ora     V_MKey
        sta     V_MKey
*
	rts
*
*
* --- Handle "000" Keys ---------------------------------------------------
CHK_3Z_STATE:
	brclr	b_INH_3Z,V_3_Zero,GO_SET_0

	rts			; '0' report is inhibited (OFF)
*
*
* --- Handle Normal Keycode Bytes ----------------------------------------
NOT_MDFY_KEY:
*
        KCMPLO  low(KEY_H0),NOT_HOTK1   ; HOT KEY GROUP 1 ??
        KCMPHI  low(KEY_H7),NOT_HOTK1   ;
*
        sub     #low(KEY_H0)               ; calculate index to BIT_TABLE
        tax
*
        lda     BIT_TABLE,x             ; set corresp. bit in HOT_KEY3
        ora     V_Hot_Key1
        sta     V_Hot_Key1
*
        rts
*
NOT_HOTK1:
        KCMPLO  low(KEY_H8),NOT_HOTK2   ; HOT KEY GROUP 2 ??
        KCMPHI  low(KEY_H15),NOT_HOTK2   ;
*
        sub     #low(KEY_H8)               ; calculate index to BIT_TABLE
        tax
*
        lda     BIT_TABLE,x             ; set corresp. bit in HOT_KEY2
        ora     V_Hot_Key2
        sta     V_Hot_Key2
*
	rts
*
NOT_HOTK2:
        KCMPLO  low(KEY_H16),NOT_HOTK3   ; HOT KEY GROUP 3 ??
        KCMPHI  low(KEY_H23),NOT_HOTK3   ;
*
        sub     #low(KEY_H16)               ; calculate index to BIT_TABLE
        tax
*
        lda     BIT_TABLE,x             ; set corresp. bit in HOT_KEY1
        ora     V_Hot_Key3
        sta     V_Hot_Key3
*
        rts
* -----------------------------------------
NOT_HOTK3:

	KCMPNE	low(KEY_000),NOT_TRI_0

	bset	b_3Z_KEY_P,V_3_Zero	; The '000' key is pressed

	brset	b_3Z_KEY,V_3_Zero,CHK_3Z_STATE

	lda		#%11000100
	sta		V_3_Zero	; Set b_3Z_KEY and set counter
					;   to send out two off, two on

	mov		#(KC_40ms/4),VC_40ms	; shorten the scan rate

GO_SET_0:
	lda		#low(KEY_0)			; Put the KEY_0 in the buffer
	bra		AKEY_STORE

NOT_TRI_0:
*
* --- Handle Keys Reported in ACPI Byte ------------------------------
*
	KCMPNE	low(PKEY_PWR),NOT_PWR_KEY	; Check if = POWER DOWN key ?

	lda		#(1<<b_PwrKey_Prs)
	sta		V_ACPI_Key	; Set flag and clear other ACPI key

        rts

NOT_PWR_KEY:
	KCMPNE	low(PKEY_SLP),NOT_SLP_KEY	; Check if = SLEEP key ?

	lda		#(1<<b_SlpKey_Prs)
	sta		V_ACPI_Key	; Set flag and clear other ACPI key

        rts

NOT_SLP_KEY:
	KCMPNE	low(PKEY_WKE),NOT_WAK_KEY	; Check if = WAKE UP key ?

	lda		#(1<<b_WakKey_Prs)
	sta		V_ACPI_Key	; Set flag and clear other ACPI key

        rts

NOT_WAK_KEY:

	
	KCMPNE	low(KEY_FN),NOT_FN_KEY	; Check if = FN key ?

	bset	b_K_FN_FND,VS_Scan	; Set flag that the FN key is pressed

	bset	b_FN_STATE,VS_Scan	; Set the FN state bit

NO_CHNG_FN:

	rts

NOT_FN_KEY:
; ---------------------------------------------------------------------------
*
* When the "000" is pressed, ignore the rest keys before the "000" is
*  sent over.
*
	brset	b_3Z_KEY,V_3_Zero,IGNORE_REST
; ---------------------------------------------------------------------------
*
* --- Put store byte to Q_NKey[VP_KCode++] ---------------------
AKEY_STORE:
        ldx     VP_KCode
        KCPXEQ  low(K_KNCSIZE),ADD_S_OVR  ; keycode queue overflow ?

        sta     Q_N_Key,x               ; store byte
        inc     VP_KCode                ; point to next keycode loc.
*
* -----------------------------------------------------------------------
		KCMPLO	low(SKEY_0),EXIT_ADD_SCAN
        KCMPHI	low(SKEY_15),EXIT_ADD_SCAN	; Skip if not SKEY_#
* -----------------------------------------------------------------------
*
*  It's SKEY_#, so check if it's pressed before?
*
	sta	V_Temp_Reg	; Save it in temp. register first.

	clrx
* -------------------------------
FIND_SAME_SKEY:
	lda	Q_P_Key,x	; Try to find same SKEY in the
				;   previouse scan buffer

*	beq	SET_NEW_FN	; If it's zero, that means no more
				;    key in the rest bytes of buffer

	cmp	V_Temp_Reg	; Compare with current SKEY_#

	beq	SET_OLD_FN	; Get back the previouse FN status
				;   if it's pressed before

	incx
	KCPXNE	low(K_KNCSIZE),FIND_SAME_SKEY
				; If the loop is over, same key is not found
* -----------------------------------------------------------------------
SET_NEW_FN:
 	lda	VS_Scan		; Get the current state of FN key

	and	#(1<<b_FN_STATE)	; Extract the FN status

SET_FN:
	ldx	VP_KCode	; get the current index

	sta	(Q_N_FN-1),x	; because VP_KCode is increased

	rts
* -----------------------------------------------------------------------
SET_OLD_FN:
	lda	Q_P_FN,x	; Get back the last reported FN state

	bra	SET_FN
* -----------------------------------------------------------------------
ADD_S_OVR:
	ldx	#low(K_KNCSIZE + K_KNCSIZE)
	lda	#low(KEY_ERO)

CLR_SCAN_BUFFER_1:
	sta	(Q_N_Key-1),x

	dbnzx	CLR_SCAN_BUFFER_1
* ------------------------------------------------------------------------
EXIT_ADD_SCAN:
IGNORE_REST:
	rts
*
* --------------------------------------------------------------------- *
* PARSE_KEY - Parse the key scan buffer                                 *
*									*
* In        :  	Q_N_Key[0..5]	current KBD scan buffer			*
*		Q_N_FN[0..5]						*
*									*
* Out       :  QC_IN0_Rpt       update new make key                     *
* Call      :  ADD_KEY                                                  *
* --------------------------------------------------------------------- *
PARSE_KEY:
*
* ------------------------------------------------------------------------
*
	clra
	sta		V_Temp_Reg	; reset the index to put data into report
PARSE_LOOP:

	ldx	VP_KCode	; Get the pointer for the buffer

	inc	VP_KCode
*
	lda	Q_N_Key,x	; Get one key code from KBD scan buffer

	beq	EXIT_PARSE	; IT's over if reach end of data
* --------------------------------------
		KCMPLO	low(SKEY_0),NOT_SKEY
        KCMPHI	low(SKEY_15),NOT_SKEY	; Skip if not SKEY_#
* --------------------------------------
* *******************************************************
*  It's SKEY_#, so convert it to a prper key usage code?
* *******************************************************
        sub	#low(SKEY_0)
        sta	V_Temp_Reg1	; Save the index for converting

        lda	Q_N_FN,x	; Get the new FN status for this key code

        ldx	V_Temp_Reg1	; Restore the index for converting

        tsta

        bne	FN_IS_ON	; FN_key >< 0, pressed
*
* ---------------------------------------------
*  FN key is OFF
        lda	SKEY_TABLE_OFF,x	; Convert code with a table
        bra	FN_CONVERTED
* ---------------------------------------------
*  FN key is ON
FN_IS_ON:

	lda	SKEY_TABLE_ON,x		; Convert code with a table

FN_CONVERTED:
* ----------------------------------------------------
* *****************************************************
*  It's NOT a SKEY_#, just put it in the report buffer
* *****************************************************
NOT_SKEY:
*
	jsr	ADD_KEY		; Put the key code in report buffer
*
	lda	VP_KCode
	KCMPNE	K_KNCSIZE,PARSE_LOOP
* ------------------------------------------------------------------------
EXIT_PARSE:

 	rts

* --------------------------------------------------------------------- *
* ADD_KEY - add reg_A to USB keyboard report                            *
* In        :  reg_A            key# to be added                        *
* Out       :  QC_IN0_Rpt       update new make key                     *
* Call      :  <nil>                                                    *
* --------------------------------------------------------------------- *
ADD_KEY:
*
* --- Check if the key is Modifier keys --------------------------------
*
        KCMPLO  low(KEY_LCTL),NOT_MDFY_KEY_1  ; < Left Ctrl usage index ?
        KCMPHI  low(KEY_RGUI),NOT_MDFY_KEY_1   ; > Right GUI usage index ?
*
        sub     #low(KEY_LCTL)               ; calculate index to BIT_TABLE
        tax
*
        lda     BIT_TABLE,x             ; set corresp. bit in V_MKey
        ora     V_MKey
        sta     V_MKey
*
	rts
*
* --- Put store byte to Q_NKey[VP_KCode++] ---------------------
NOT_MDFY_KEY_1:

        ldx     V_Temp_Reg	; get the index to put data into report
*
        sta     Q_NKey,x               ; store byte
        inc     V_Temp_Reg              ; point to next keycode loc.
*
*
* ------------------------------------------------------------------------
AKEY_EXIT:
        rts



* --------------------------------------------------------------------- *
* Dly_40us - delay 40us (HC08, Fcpu=3MHz) - by inst. cycles           *
* In        :  K_Nd40us     no. of CPU clocks for 40us delay            *
* Out       :  <nil>                                                    *
* Call      :  <nil>                                                    *
* --------------------------------------------------------------------- *
* (jsr  Dly_40us                                                [5] clk)
Dly_40us:
        pshx                            ;                       [2] clk
        ldx     #((K_Nd40us-15)/3)      ;                       [2] clk
DLY_NEXT:
        DBNZX   DLY_NEXT                ;                       (3) clk
*
        pulx                            ;                       [2] clk
*
* ------------------------------------------------------------------------
DLY_EXIT:
        rts                             ;                       [4] clk


* --------------------------------------------------------------------- *
* Table   :  return bit position in a byte                              *
* Example :  ldx  <bit position>                                        *
*            lda  Bit_Table,x                                           *
* --------------------------------------------------------------------- *
BIT_TABLE:
        dc.b      %00000001
        dc.b      %00000010
        dc.b      %00000100
        dc.b      %00001000
        dc.b      %00010000
        dc.b      %00100000
        dc.b      %01000000
        dc.b      %10000000






* ====== End of File =====================================================
            end
