*****************************************************************************
*                                         Copyright (c) Motorola 2002		*
*  File Name:	USB_ICP.ASM													*
*                        													*
*  Description:	JB8_ICP is a pre-loaded firmware that allows user to do		*
*				the firmware upgrade through the USB interface				*
*                                                                          	*
*  Assembler:	CodeWarrior													*
*  Version:		2.1								                        	*
*																			*
*  Current Revision:	1.0													*
*  Current Revision Release Date: 2002.10.10 by Derek Lau					*
*																			*
*  Updated History															*
*   Rev		YYYY.MM.DD  Author			Description of Change				*
*	---		----------- ------			---------------------				*
*	0.1		2002.02.01	Derek Lau		original release					*
*	1.0		2002.10.10	Derek Lau		general release, added checksum		*
*****************************************************************************
* This Program is a freeware to demonstrate the operation of Motorola		*
* Microcontroller. Motorola reserves the right to make changes without		*
* further notice to any product herein to improve reliability, function, or *
* design. Motorola does	not assume any liability arising out of the 		*
* application or use of any product, circuit, or software described herein; *
* neither does it convey any license under its patent rights nor the rights *
* of others. Motorola products are not designed, intended, or authorized 	*
* for use as components	in systems intended for surgical implant into the	*
* body, or other applications intended to support life, or for any other	*
* application in which the failure of the Motorola product could create a	*
* situation where personal injury or death may occur. Should Buyer purchase	*
* or use Motorola products for any such intended or unauthorized 			*
* application, Buyer shall indemnify and hold Motorola and its officers,	*
* employees, subsidiaries, affiliates, and distributors harmless against	*
* all claims, costs, damages, and expenses, and reasonable attorney fees	*
* arising out of, directly or indirectly, any claim of personal injury or	*
* death associated with such unintended or unauthorized use, even if such	*
* claim alleges that Motorola was negligent regarding the design or 		*
* manufacture of the part. Motorola	and the Motorola logo* are registered	*
* trademarks of Motorola Inc												*
*****************************************************************************
;* Parameter Equates

		include	"jb16-eqs.h"			; jb8 registers definitions
		include "macro8.h"				; 08 CPU macro


MON_USB_ICP		EQU		$FA19			; Monitor USB ICP routines

USE_USB_IPULLUP	set		0				; 0 - use internal pullup

DEFAULT_RAM		SECTION	SHORT
V_ChkSumH		ds.b	1

			XDEF	_Startup
myCode		SECTION	Short
 
;*==========================================================
;*		
;*		Main Program
;*
;*===========================================================
ICP_Reset_Init:
_Startup:
		lda		JMP_Reset_Init+1		; check if app address valid
		cbeqa	#$FF,USB_ICP			; usb ICP if app address blank
		KCMPLO	(ROM_BEG/256),USB_ICP	; usb ICP if app address invalid

;		jmp		JMP_Reset_Init			; for testing only

		clr		V_ChkSumH				; clear checksum high byte
		clra							; clear acc for calu checksum
		ldhx	#$F600					; checksum starting address

ChkSum_Loop:
		add		,x						; add the bytes in flash
		bcc		Not_Overflow			; overflow ?
		inc		V_ChkSumH				; increase checksum high byte if yes
Not_Overflow:		
		aix		#1						; increase flash address
		cphx	#(ICP_FLAG)				; flash address reaches ICP_FLAG
		bne		ChkSum_Loop				; continue if not finish

		add		ICP_FLAG+1				; sum of flash + ICP_FLAG low byte
		bcc		Not_Overflow1			; overflow ?
		inc		V_ChkSumH				; increase checksum high byte if yes
Not_Overflow1:		

		tsta							; checksum low byte+ICP_FLAG low byte=0 ?
		bne		USB_ICP					; ICP mode if sum <> 0
		lda		ICP_FLAG				; get ICP_FLAG high byte
		add		V_ChkSumH				; add checksum high byte
		bne		USB_ICP					; ICP mode if sum <> 0
		

Jmp_Application		
		jmp		JMP_Reset_Init			; jmp to application program



;====================================================================
;               USB ICP
;====================================================================
USB_ICP:
		sei
		mov	#%00000011,CONFIG

 IFEQ	USE_USB_IPULLUP
		mov	#%00000100,UCR3		; enable USB pullup
		
 ENDIF
 		jmp	Mon_USB_ICP


		INCLUDE		"appvector.h"

* ======================================================================

			ORG		$FFD6
			dc		$01,$02,$03,$04,$05,$06,$07,$08

        	ORG		VECTORS

KBD_INT     dc.w	JMP_KBD_ISR		; Keyboard
SCI_TX_INT  dc.w	JMP_SCI_TX_ISR	; SCI transmit
SCI_RX_INI  dc.w	JMP_SCI_RX_ISR	; SCI receive
SCI_ERR_INT dc.w	JMP_SCI_ERR_ISR	; SCI error
T2OF_INT    dc.w	JMP_T2OF_ISR	; TIM2 overflow	
T2CH01_INT  dc.w	JMP_T2CH01_ISR	; TIM Ch2_10
T2CH1_INT   dc.w	JMP_T2CH1_ISR	; TIM Ch2_1
T2CH0_INT   dc.w	JMP_T2CH0_ISR	; TIM Ch2_0
T1OF_INT    dc.w	JMP_T1OF_ISR	; TIM1 overflow
T1CH01_INT  dc.w	JMP_T1CH01_ISR	; TIM Ch1_10
T1CH1_INT   dc.w	JMP_T1CH1_ISR	; TIM Ch1_1
T1CH0_INT   dc.w	JMP_T1CH0_ISR	; TIM Ch1_0
IRQ1_INT    dc.w    JMP_IRQ_ISR		; IRQ1
USB_INT     dc.w    JMP_USB_ISR		; USB device
SWI_INT     dc.w    JMP_SWI_ISR		; SWI
;RST_IRQ     dc.w    JMP_Reset_Init	; Reset vector


PROG_END:
        	END

